# UPC keywords module


# Initialize keyword lists 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #

# UPC keywords (taken from UPC specification 1.2 section 6)
# --------------------------------------------------------------------------- #
# UPC Predefined identifiers (6.3)
upcspec_identifiers = {'THREADS', 'MYTHREAD', 'UPC_MAX_BLOCK_SIZE',}
# UPC Expressions (6.4)
upcspec_expressions = {'upc_localsizeof', 'upc_blocksizeof', 'upc_elemsizeof',}
# UPC Declarations (6.5)
upcspec_type_qualifiers = {'shared', 'strict', 'relaxed'}
# UPC Statements and blocks (6.6)
upcspec_statements = {'upc_notify', 'upc_wait', 'upc_barrier', 'upc_fence',}
upcspec_loops      = {'upc_forall',}

# UPC Library (taken from UPC specification 1.2 section 7)
# UPC Standard headers (7.1)
upclib_headers = {'upc_strict', 'upc_relaxed', 'upc_collective', 'upc',}
# UPC utilities (7.2)
upclib_exit = {'upc_global_exit',}      # (7.2.1)
upclib_alloc = {'upc_global_alloc', 'upc_all_alloc', 'upc_alloc', 'upc_free',} # (7.2.2)
upclib_shared = {'upc_threadof', 'upc_phaseof', 'upc_resetphase', 'upc_addrfield', 'upc_affinitysize',} # (7.2.3)
upclib_lock = {'upc_lock_t', 'upc_global_lock_alloc', 'upc_all_lock_alloc', 'upc_lock_free', 'upc_lock', 'upc_lock_attempt', 'upc_unlock',} # (7.2.4)
upclib_mem = {'upc_memcpy', 'upc_memget', 'upc_memput', 'upc_memset',} # (7.2.5)
upclib_flag = {'upc_flag_t', 'UPC_IN_NOSYNC', 'UPC_IN_MYSYNC', 'UPC_IN_ALLSYNC', 'UPC_OUT_NOSYNC', 'UPC_OUT_MYSYNC', 'UPC_OUT_ALLSYNC',} # (7.2.6)
upclib_utilities = [upclib_exit, upclib_alloc, upclib_shared, upclib_lock, upclib_mem, upclib_flag,]
# UPC Collective utilities (7.3)
upclib_collectives = {'upc_all_broadcast', 'upc_all_scatter', 'upc_all_gather', 'upc_all_gather_all', 'upc_all_exchange', 'upc_all_permute', 'upc_all_reduce', 'upc_all_prefix_reduce'}
# UPC collective reduction operations (7.3.2)
upclib_operations = {'upc_op_t', 'UPC_ADD', 'UPC_MULT', 'UPC_AND', 'UPC_OR', 'UPC_XOR', 'UPC_LOGAND', 'UPC_LOGOR', 'UPC_MIN', 'UPC_MAX', 'UPC_FUNC', 'UPC_NONCOMM_FUNC',}

# C keywords (...very... abbreviated selection)
# --------------------------------------------------------------------------- #
cspec_statements = {'typedef',}
cspec_type_qualifiers = {'const', 'volatile', 'restrict',}
cspec_type_modifiers = {'short', 'long', 'signed', 'unsigned',}
cspec_storage_class = {'register', 'static', 'extern',}
cspec_base_types = {'char', 'int', 'float', 'double', 'void',}
cspec_more_types = {'short', 'long', 'uint8_t', 'uint16_t', 'uint32_t', 'uint64_t', 'int8_t', 'int16_t', 'int32_t', 'int64_t',}
