/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_NEIGHBOR_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_NEIGHBOR_H_INCLUDED

#include <minimd.h>
#include <lib_compute_minimd.h>
#include <sstmac/software/api/mpi/mpiapi.h>

namespace mini {

  /**
   * Take over the neighbor list management from miniMD.
   */
  class minimd::neighbor : public ptr_type {
  private:
    lib_compute_minimd::ptr execution_;
    mpiapi::ptr mpi_;

  public:
    virtual std::string to_string() const{
            return "neighbor";
          }

    typedef boost::intrusive_ptr<neighbor> ptr;
    // As in miniMD, all key variables are public.
    int every; /// reneighbor every n steps.
    int nbinx, nbiny, nbinz;  // number of global bins.
    double cutneigh; // neighbor cutoff.
    int mbins; // input local bins mocking mbinx*mbiny*mbinz from neighbor::setup in real code
    /// neighbor build loop parameters
    int avgtripsj1;
    int avgtripsj2;
    int avgtripsj3;
    int nstencil;
    int avgallocs;

    
    void init(const lib_compute_minimd::ptr& exe,
              const mpiapi::ptr& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    /// This method is not in the real miniMD.
    void set_bins(int nx, int ny, int nz) {
      nbinx = nx; nbiny = ny; nbinz = nz;
    }

    /// This is a somewhat extensive function in the original.
    void setup() {
      execution_->compute("Neighbor::setup", ptr_type::ptr(), ptr_type::ptr());
    }

    /// Build a neighbor list.
    void build(const boost::intrusive_ptr<atom>& at) {

      execution_->compute("Neighbor::build", at, ptr(this));
    }
  };

} 

#endif
