/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef HPX_HPCCG_H_
#define HPX_HPCCG_H_

#include <sstmac/software/api/hpx/hpx_app.h>

#include <sstmac/sstmacro.h>

#include "HPCCG/HPC_Sparse_Matrix.h"

namespace hpx_hpccg
{

  class hpccg : public hpx::hpx_app
  {

  protected:

    class matrix;
    friend class matrix;

    virtual void
    register_actions();

    virtual void
    register_components();

    class config_params : public sstmac::sizeable
    {
    protected:
      config_params()
      {
      }

    public:
      typedef boost::intrusive_ptr<config_params> ptr;

      static ptr
      construct()
      {
        return ptr(new config_params());
      }

      virtual
      ~config_params()
      {
      }
      virtual std::string
      to_string() const
      {
        return "hpccg::config_params";
      }

      virtual long
      get_byte_length() const
      {
        return 4 * sizeof(int);
      }

      int nx_;
      int ny_;
      int nz_;
      int domains_;

    };

    config_params::ptr par_;



    //  int
    //  HPCCG(HPC_Sparse_Matrix * A, const double * const b, double * const x,
    //      const int max_iter, const double tolerance, int &niters,
    //      double & normr, double * times);

    double
    mytimer();

  public:
    typedef boost::intrusive_ptr<hpccg> ptr;

    hpccg(const sim_parameters::ptr& params) : hpx_app(params) {}

    hpccg(sstmac::sw::softwareid id) :
      hpx_app(id)
    {
    }

    virtual
    ~hpccg()
    {
    }

    virtual void
    hpx_main();

    static ptr
    construct(const sim_parameters::ptr& params){
        return new hpccg(params);
    }

    static ptr
    construct(sstmac::sw::softwareid id)
    {
      ptr retval(new hpccg(id));
      return retval;
    }

    virtual app::ptr
    clone(sstmac::sw::softwareid id)
    {
      return construct(id);
    }

    virtual void
    consume_params(const sstmac::sim_parameters::ptr &params);

    virtual std::string
    to_string() const
    {
      return "hpccg";
    }
  };

}

#endif /* GTC_APP_H_ */
