/*
 * hpccg_matrix.cc
 *
 *  Created on: Nov 7, 2012
 *      Author: ghendry
 */

#include "HPCCG/hpccg_matrix.h"
#include "HPCCG/hpccg_matrix_stubs.h"

#include <sstmac/software/libraries/compute/sstmac_compute.h>
#include "HPCCG/waxpby.h"
#include <sstmac/software/api/hpx/lcos/future_wait.h>

namespace hpx_hpccg
{

#define TICK()  t0 = mytimer() // Use TICK and TOCK to time a code section
#define TOCK(t) t += mytimer() - t0

  using namespace std;

  long hpccg::matrix::bytes_sent_ = 0;

  hpccg::matrix::matrix(hpx::globalid gid) :
    hpx_component(gid)
  {
    tmp_neighbors_lock_ = hpx::lcos::sync::construct();
  //  tmp_neighbors_lock_->lock();

    sends_lock_ = hpx::lcos::sync::construct();
    lengths_lock_ = hpx::lcos::sync::construct();
   // sends_lock_->lock();

    send_list_lock_ = hpx::lcos::sync::construct();
    //send_list_lock_->lock();

    neighbors_lock_ = hpx::lcos::sync::construct();
   // neighbors_lock_->lock();

    ddot_local_lock_ = hpx::lcos::sync::construct();
   // ddot_local_lock_->lock();

    sends_recvd_ = 0;
    lengths_recvd_ = 0;
    element_idx_ = 0;

    times_ = new double[7];

  //  external_recvd_ = 0;
  //  neighbors_recvd_ = 0;
    ddots_accessed_ = 0;
  }

  hpccg::matrix::~matrix()
  {
  }

  void
  hpccg::matrix::register_actions(
      boost::intrusive_ptr<hpx::hpx_thread_manager> threadman)
  {

    threadman->register_action<init_action> (get_action_name(INIT),
        init_action::construct());
    threadman->register_action<get_start_row_action> (
        get_action_name(GET_START_ROW), get_start_row_action::construct());
    threadman->register_action<get_tmp_neighbors_action> (
        get_action_name(GET_TMP_NEIGHBORS),
        get_tmp_neighbors_action::construct());
    threadman->register_action<add_send_neighbor_action> (
        get_action_name(ADD_SEND_NEIGHBOR),
        add_send_neighbor_action::construct());
    threadman->register_action<add_send_length_action> (
        get_action_name(ADD_SEND_LENGTH), add_send_length_action::construct());
    threadman->register_action<add_send_element_action> (
        get_action_name(ADD_SEND_ELEMENT), add_send_element_action::construct());
    threadman->register_action<hpccg_solve_action> (
        get_action_name(HPCCG_SOLVE), hpccg_solve_action::construct());
    threadman->register_action<add_elements_action> (
            get_action_name(ADD_ELEMENTS), add_elements_action::construct());
    threadman->register_action<get_ddot_local_action> (
        get_action_name(GET_DDOT_LOCAL), get_ddot_local_action::construct());

  }

  std::string
  hpccg::matrix::get_action_name(ACTION a)
  {
    switch (a)
      {
    case INIT:
      return "matrix::init";

    case GET_START_ROW:
      return "matrix::get_start_row";
    case GET_TMP_NEIGHBORS:
      return "matrix::get_tmp_neighbors";
    case ADD_SEND_NEIGHBOR:
      return "matrix::add_send_neighbor";
    case ADD_SEND_LENGTH:
      return "matrix::add_send_length";
    case ADD_SEND_ELEMENT:
      return "matrix::add_send_element";
    case HPCCG_SOLVE:
      return "matrix::hpccg_solve";
    case ADD_ELEMENTS:
      return "matrix::add_elements";
    case GET_DDOT_LOCAL:
      return "matrx::get_ddot_local";
    default:
      throw sstmac::ssterror("hpccg::matrix::get_action_name: unknown action");
      }

  }

  int
  hpccg::matrix::init(std::size_t objectid, config_params::ptr par,
      std::vector<stub_ptr> points)
  {

    idx_ = objectid;
    SSTMAC_DEBUG << "hpccg::matrix(" << idx_ << ")::init() \n";

    mstubs_ = points;
    par_ = par;

    if (sstmac::sstmac_env::params->has_param("hpccg_data_file"))
      {
        read_HPC_row(
            sstmac::sstmac_env::params->get_param("hpccg_data_file").c_str(),
            &A_, &x_, &b_, &xexact_);
      }
    else
      {
        generate_matrix(par_->nx_, par_->ny_, par_->nz_, &A_, &x_, &b_,
            &xexact_);
      }

    double t1 = mytimer();
    make_local_matrix(A_);
    double t2 = mytimer();
    times_[6] = t2 - t1;

    SSTMAC_DEBUG << "init:: rank " << idx_ << " has b " << (long)b_ << "\n";

    return HPX_SUCCESS;
  }

  void
   dump(std::string name, double* arr, int num, int r){

     std::cout << "----- rank " << r << " dumping array " << name << " (" << num  << " elements)\n";
     for(int i = 0; i < num; i++){
         std::cout << i << ": " << arr[i] << "\n";
     }
    // exit(0);
   }

  int
  hpccg::matrix::HPCCG_Solve(const int max_iter, const double tolerance)

  {

    SSTMAC_DEBUG << "solve:: rank " << idx_ << " has b " << (long)b_ << "\n";


    double t_begin = mytimer(); // Start timing right away

    double t0 = 0.0, t1 = 0.0, t2 = 0.0, t3 = 0.0, t4 = 0.0;

    double t5 = 0.0;

    int rank = idx_;

    double normr = 0;
    int niters = 0;

    int nrow = A_->local_nrow;
    int ncol = A_->local_ncol;

    double * r = new double[nrow];
    p_ = new double[ncol]; // In parallel case, A is rectangular
    double * Ap = new double[nrow];

    normr = 0.0;
    double rtrans = 0.0;
    double oldrtrans = 0.0;

    int print_freq = max_iter / 10;
    if (print_freq > 50)
      print_freq = 50;
    if (print_freq < 1)
      print_freq = 1;

    // p is of length ncols, copy x to p for sparse MV operation
    TICK();
    waxpby(nrow, 1.0, x_, 0.0, x_, p_);
    TOCK(t2);



    TICK();
    exchange_externals();
    TOCK(t5);

   // dump("b", b_, A_->local_nrow, idx_);

    TICK();
    HPC_sparsemv(p_, Ap);
    //dump("b", b_, A_->local_nrow, idx_);
    TOCK(t3);
    TICK();
    waxpby(nrow, 1.0, b_, -1.0, Ap, r);
    TOCK(t2);
    TICK();
    ddot(nrow, r, r, &rtrans, t4);
    TOCK(t1);
    normr = sqrt(rtrans);

    if (rank == 0)
      cout << "Initial Residual = " << normr << endl;

    for (int k = 1; k < max_iter && normr > tolerance; k++)
      {
        if (k == 1)
          {
            TICK();
            waxpby(nrow, 1.0, r, 0.0, r, p_);
            TOCK(t2);
          }
        else
          {
            oldrtrans = rtrans;
            TICK();
            ddot(nrow, r, r, &rtrans, t4);
            TOCK(t1);// 2*nrow ops
            double beta = rtrans / oldrtrans;
            TICK();
            waxpby(nrow, 1.0, r, beta, p_, p_);
            TOCK(t2);// 2*nrow ops
          }
        normr = sqrt(rtrans);
        if (rank == 0 && (k % print_freq == 0 || k + 1 == max_iter))
          cout << "Iteration = " << k << "   Residual = " << normr << endl;

        TICK();
        exchange_externals( );
        TOCK(t5);

        TICK();
        HPC_sparsemv(p_, Ap);
        TOCK(t3); // 2*nnz ops
        double alpha = 0.0;
        TICK();
        ddot(nrow, p_, Ap, &alpha, t4);
        TOCK(t1); // 2*nrow ops
        alpha = rtrans / alpha;
        TICK();
        waxpby(nrow, 1.0, x_, alpha, p_, x_);// 2*nrow ops
        waxpby(nrow, 1.0, r, -alpha, Ap, r);
        TOCK(t2);// 2*nrow ops
        niters = k;
      }

    if(idx_ == 0){
        std::cout << "bytes sent: " << bytes_sent_ << "\n";
    }

    // Store times
    times_[1] = t1; // ddot time
    times_[2] = t2; // waxpby time
    times_[3] = t3; // sparsemv time
    times_[4] = t4; // AllReduce time

    times_[5] = t5; // exchange boundary time

    delete[] p_;
    delete[] Ap;
    delete[] r;
    times_[0] = mytimer() - t_begin; // Total time. All done...
    return (0);
  }

  void
  hpccg::matrix::exchange_externals()
  {
    int i, j, k;
    int num_external = 0;

    // Extract Matrix pieces
    SSTMAC_DEBUG2 << "exchange_externals(" << idx_ << "): \n";


    int * recv_length = A_->recv_length;
    int * send_length = A_->send_length;
    int * neighbors = A_->neighbors;
    double * send_buffer = A_->send_buffer;
    int total_to_be_sent = A_->total_to_be_sent;
    int * elements_to_send = A_->elements_to_send;

    //
    // Externals are at end of locals
    //
    // x_external_ = (double *) x_ + local_nrow;

    for (i = 0; i < total_to_be_sent; i++){
        SSTMAC_DEBUG3 << "exchange_externals(" << idx_ << "): accesing element " << elements_to_send[i] << "\n";
      send_buffer[i] = p_[elements_to_send[i]];
    }

    //
    // Send to each neighbor
    //

    for (i = 0; i < A_->num_send_neighbors; i++)
      {
        //  int n_send = send_length[i];
        //  MPI_Send(send_buffer, n_send, MPI_DOUBLE, neighbors[i], MPI_MY_TAG,
        //      MPI_COMM_WORLD);
        //  send_buffer += n_send;
        bytes_sent_ += send_length[i];
        mstubs_[neighbors[i]]->add_elements(
            buffer_wrapper<double> (send_buffer, send_length[i], idx_));
        send_buffer += send_length[i];
      }

    neighbors_lock_->lock(A_->num_send_neighbors);


    return;
  }

  int
  hpccg::matrix::add_elements(buffer_wrapper<double> el)
  {
    int found = -1;
    for(int i = 0; i < A_->num_send_neighbors; i++){
        if(A_->neighbors[i] == el.ind()){
            found = i;
        }
    }

    if(found < 0){
        sst_throw(ssterror, "hpccg::matrix::add_elements: didn't find neighbor index");
    }

    int start = A_->local_nrow;
    for(int i = 0; i < A_->num_send_neighbors && i < found; i++){
        start += A_->recv_length[i];
    }

    double* raw = el.get_buf();
    for (int i = 0; i < el.num(); i++)
      {
        p_[start + i] = raw[i];
      }
   // external_recvd_ += el.num();
    neighbors_lock_->unlock();


    return HPX_SUCCESS;

  }

  int
  hpccg::matrix::HPC_sparsemv(const double * const x, double * const y)
  {

    SSTMAC_DEBUG2 << "HPC_sparsemv(" << idx_ << "): \n";

    const int nrow = (const int) A_->local_nrow;

    int countit = 0;

    for (int i = 0; i < nrow; i++)
      {
        double sum = 0.0;
        const double * const cur_vals =
            (const double * const ) A_->ptr_to_vals_in_row[i];

        const int * const cur_inds =
            (const int * const ) A_->ptr_to_inds_in_row[i];

        const int cur_nnz = (const int) A_->nnz_in_row[i];

        for (int j = 0; j < cur_nnz; j++)
          {
            sum += cur_vals[j] * x[cur_inds[j]];
            countit++;
          }

        y[i] = sum;
      }

    //here's some compute modeling
    hpx_app::compute_loop(0, countit, 1);

    return (0);
  }

  int
  hpccg::matrix::ddot(const int n, const double * const x,
      const double * const y, double * const result, double & time_allreduce)
  {
    SSTMAC_DEBUG2 << "ddot(" << idx_ << "): \n";

    ddot_local_result_ = 0.0;
    if (y == x)
      for (int i = 0; i < n; i++)
        ddot_local_result_ += x[i] * x[i];
    else
      for (int i = 0; i < n; i++)
        ddot_local_result_ += x[i] * y[i];

    // a little compute modeling
    hpx_app::compute_loop(0, n, 1);

    SSTMAC_DEBUG2 << "rank " << idx_ << " is done computing it's local ddot: " << ddot_local_result_ << "\n";

    ddot_local_lock_->unlock( mstubs_.size()); //it's ready

    // Use MPI's reduce function to collect all partial sums
    double t0 = mytimer();
    ddot_global_result_ = 0.0;

      {// MPI_Allreduce(&local_result, &global_result, 1, MPI_DOUBLE, MPI_SUM,
        //  MPI_COMM_WORLD);
        typedef std::vector<hpx::lcos::promise<double>::ptr> lazy_results_type;
        lazy_results_type lazy_results;
        for (size_t i = 0; i < mstubs_.size(); i++)
          {
            lazy_results.push_back(mstubs_[i]->get_ddot_local());
          }
        ptr self(this);
        hpx::lcos::wait<double, hpccg::matrix>(lazy_results,
            &hpccg::matrix::reduce_ddot_local, self);
      }

    *result = ddot_global_result_;
    time_allreduce += mytimer() - t0;

   // ddot_local_lock_->lock(); //relock for next time

    return (0);
  }

  double
  hpccg::matrix::get_ddot_local()
  {
    SSTMAC_DEBUG2 << "getting local ddot from rank " << idx_ << "\n";

    ddot_local_lock_->lock();
    SSTMAC_DEBUG2 << "through at rank " << idx_ << "\n";

    return ddot_local_result_;
  }

  bool
  hpccg::matrix::reduce_ddot_local(std::size_t i, double const& val)
  {
    ddot_global_result_ += val;
    return HPX_SUCCESS;
  }
}
