/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SKELETONS_PARALLEX_GTC_FAKE_ARRAY_H_INCLUDED
#define SKELETONS_PARALLEX_GTC_FAKE_ARRAY_H_INCLUDED

//#include <sstmac/common/ptr_type.h>

namespace hpx_gtc
{

  /**
   * fake_array - just holds how big the array is, to model communicating it
   */
  template<typename Type>
    class fake_array
    {
    protected:
      size_t size_;
      Type fake_;

    public:
      fake_array(size_t dim)
      {
        size_ = sizeof(Type) * dim;
      }

      fake_array()
      {
        size_ = 0;
      }

      size_t
      size() const
      {
        return size_;
      }

      // basic item reference
      Type&
      operator[](std::size_t i)
      {
        return fake_;
      }

      Type const&
      operator[](std::size_t i) const
      {
        return fake_;
      }

      std::ostream&
      operator<<(std::ostream &os)
      {
        os << "fake_array(" << size_ << ")";
        return os;
      }

      void
      push_back(Type t)
      {
        size_ += sizeof(Type);
      }

    };

}

#endif
