/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SKELETONS_PARALLEX_GTC_GTC_PARTICLE_H_INCLUDED
#define SKELETONS_PARALLEX_GTC_GTC_PARTICLE_H_INCLUDED

#include <gtc/gtc_app.h>
#include <sstmac/software/api/hpx/components/hpx_component.h>
#include <sstmac/software/api/hpx/lcos/local_mutex.h>
#include <sstmac/sstmacro.h>

namespace hpx_gtc
{

  class gtc_app::gtc_particle : public hpx::components::hpx_component
  {
  protected:

    std::size_t idx_;
    double posx_, posy_, posz_;

    hpx::lcos::local_mutex::ptr mtx_;

    // ---- these are the functions that actually do something,
    // ------- they are protected so that only actions can access them
    void
    init(std::size_t objectid, double posx, double posy, double posz);

    double
    distance(double posx, double posy, double posz);

    // -------------------------------- //
    gtc_particle(hpx::globalid gid) :
        hpx_component(gid)
    {
      mtx_ = hpx::lcos::local_mutex::construct();
    }

  public:
    class gtc_particle_stubs;

    typedef boost::intrusive_ptr<gtc_particle> ptr;
    typedef boost::intrusive_ptr<gtc_particle_stubs> stub_ptr;
    typedef gtc_particle_stubs stubs;

    static std::string particlefile_;

    static ptr
    construct(hpx::globalid gid)
    {
      return ptr(new gtc_particle(gid));
    }

    virtual
    ~gtc_particle()
    {
    }

    static std::string
    component_name()
    {
      return "gtc_app::gtc_particle";
    }

    virtual void
    register_actions(boost::intrusive_ptr<hpx::hpx_thread_manager> threadman);

    virtual hpx::components::hpx_component::ptr
    clone(hpx::globalid gid)
    {
      return construct(gid);
    }

    // ---- action classes ------ //
    class init_action;
    friend class init_action;
    class distance_action;
    friend class distance_action;

  };

}
#endif /* HPX_RMA_ELEMENT_H_ */
