/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <rma/hpx_rma_element_stubs.h>
//#include <rma/hpx_rma_element_actions.h>
#include <sstmac/software/api/hpx/lcos/eager_future.h>
#include <sstmac/software/api/hpx/lcos/future_wait.h>

using namespace sstmac;
using namespace sstmac::sw;

namespace hpx
{
  namespace rma
  {

  char theINIT_NAME[] = "hpx_rma_element::init";
  char theADD_NAME[] = "hpx_rma_element::add";
  char thePRINT_NAME[] = "hpx_rma_element::print";

  void
  hpx_random_mem_access::hpx_rma_element::rma_stubs::init(std::size_t s)
  {
    typedef hpx::lcos::eager_future<int> rma_element_init;

    hpx::hpx_action_params1<std::size_t>::ptr params = hpx::hpx_action_params1<
        std::size_t>::construct(s);
    rma_element_init::ptr ef = rma_element_init::construct(theINIT_NAME, gid_,
        params);
    ef->get();
  }

  void
  hpx_random_mem_access::hpx_rma_element::rma_stubs::add()
  {
    hpx::lcos::promise<int>::ptr pro = add_async();
    hpx::lcos::wait(pro);
  }

  void
  hpx_random_mem_access::hpx_rma_element::rma_stubs::print()
  {

    hpx::lcos::promise<int>::ptr pro = print_async();
    hpx::lcos::wait(pro);
  }

  hpx::lcos::promise<int>::ptr
  hpx_random_mem_access::hpx_rma_element::rma_stubs::add_async()
  {
    typedef hpx::lcos::eager_future<int> rma_element_add;

    hpx::hpx_action_params1<std::size_t>::ptr params;
    rma_element_add::ptr ef = rma_element_add::construct(theADD_NAME, gid_,
        params);
    return ef;
  }

  hpx::lcos::promise<int>::ptr
  hpx_random_mem_access::hpx_rma_element::rma_stubs::print_async()
  {
    typedef hpx::lcos::eager_future<int> rma_element_print;

    hpx::hpx_action_params1<std::size_t>::ptr params;
    rma_element_print::ptr ef = rma_element_print::construct(thePRINT_NAME, gid_,
        params);
    return ef;
  }


} }

