/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_eventheap_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_eventheap_H_INCLUDED

#include <sstmac/common/eventmanager.h>
#include <sstmac/common/sstevent.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/common/stats/stats_common.h>

#include <queue>
#include <vector>

namespace sstmac
{
  namespace native
  {

    /**
     * An event manager that relies on a multimap to manage events.
     */
    class eventheap : public eventmanager
    {
      /// The events sorted by timestamp.
      typedef std::priority_queue<event::ptr, std::vector<event::ptr>, std::greater<event::ptr> > queue_t;
      queue_t queue_;

      /// Sentinel to track whether the event handler is running or not.
      bool running_;


      timestamp last_update_sim_;

      std::list<ptr_type::ptr> finishers_;

      static stat_loghistogram<int64_t>::ptr event_hist_;

      /// Hello.
      eventheap(const timestamp &tv);

      void
      finish();

    public:

      typedef boost::intrusive_ptr<eventheap> ptr;



      /// Hello.
      static eventheap::ptr
      construct(const timestamp &tv = timestamp(0));

      /// Goodbye.
      virtual
      ~eventheap() throw ();

      /// Clear all events and set time back to a zero of your choice.
      /// \throw illformederror if the event manager is running.
      virtual void
      clear(const timestamp &zero_time = timestamp(0));

      /// Run the eventmanager.
      /// The eventmanager shall return control when no more messages remain.
      virtual void
      run();

      virtual void
      do_one_event();

      virtual void
      step();

      virtual void
      cancel_all_messages(const eventmodule::ptr& mod);

      /// Set off the given eventhandler at the given time.
      virtual void
      schedule(const timestamp &start_time, const event::ptr& ev);

      virtual void
      register_finish(ptr_type::ptr p);
    };

  }
} // end of namespace sstmac

#endif
