#ifndef SSTMAC_BACKENDS_NATIVE_EVENTLIST_H
#define SSTMAC_BACKENDS_NATIVE_EVENTLIST_H

/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/eventmanager.h>
#include <sstmac/common/sstevent.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/common/eventmodule.h>

namespace sstmac
{
  namespace native
  {

    class eventunit
    {
    private:
      event::ptr head_;

      event::ptr tail_;

    public:
      eventunit();

      event::ptr
      pop();

      void
      append(const event::ptr& ev);

      bool
      nonempty() const;

      bool
      empty() const;

      void
      clear();
    };

    /**
     * An event manager that relies on a multimap to manage events.
     */
    class eventlist : public eventmanager
    {

    private:
      event::ptr distant_future_head_;

      event::ptr distant_future_tail_;

      eventunit* future_windows_;

      eventunit* event_units_;

      int64_t distant_future_horizon_;

      int64_t current_window_tick_start_;

      int64_t nticks_time_window_;

      int64_t current_window_;

      int64_t nwindows_;

      int64_t window_limit_;

      int64_t max_scheduled_window_;

      /// Sentinel to track whether the event handler is running or not.
      bool running_;

      std::list<ptr_type::ptr> finishers_;

      /// Hello.
      eventlist(long nticks_window, long nwindows);

      void
      finish();

      void
      schedule(int64_t ticks, const event::ptr& ev);

      void
      schedule(const event::ptr& ev);

      void
      find_next_window();

      void
      reschedule_future_events();

    public:

      typedef boost::intrusive_ptr<eventlist> ptr;
      /// Hello.
      static eventlist::ptr
      construct(long nticks_window, long nwindows);

      /// Goodbye.
      virtual
      ~eventlist() throw ();

      /// Clear all events and set time back to a zero of your choice.
      /// \throw illformederror if the event manager is running.
      virtual void
      clear(const timestamp &zero_time = timestamp(0));

      /// Run the eventmanager.
      /// The eventmanager shall return control when no more messages remain.
      virtual void
      run();

      virtual void
      do_one_event();

      virtual void
      step();

      virtual void
      cancel_all_messages(const eventmodule::ptr &handler);

      /// Set off the given eventhandler at the given time.
      virtual void
      schedule(const timestamp &start_time, const event::ptr& ev);

      virtual void
      register_finish(ptr_type::ptr p);
    };

  }
} // end of namespace sstmac

#endif // EVENTLIST_H
