/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/ipc_event.h>
#include <sstmac/backends/native/mpiparallel/mpi_serializer.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/sstmac_env.h>

DeclareSerializable(sstmac::native::ipc_event)
  ;

namespace sstmac
{
  namespace native
  {
    /*nodeaddress::ptr
     ipc_event::get_src_switch()
     {
     return src_switch_;
     }*/

    ipc_event::ptr
    ipc_event::construct(const sst_message::ptr &msg,
        const eventhandler::ptr &handler, const nodeaddress::ptr &dst_switch)
    {
      ipc_event::ptr ret(new ipc_event(msg, handler, dst_switch));
      return ret;
    }

    ipc_event::ipc_event(const sst_message::ptr &msg,
        const eventhandler::ptr &handler, const nodeaddress::ptr &dst_switch) :
      simple_event(msg, handler)
    {
      dest_switch_ = dst_switch;
      canceled_ = false;
    }

    void
    ipc_event::set_event_handler(const eventhandler::ptr &handler)
    {
      handler_ = handler;
    }

    nodeaddress::ptr
    ipc_event::get_dest_switch()
    {
      return dest_switch_;
    }

    void
    ipc_event::set_dest_switch(nodeaddress::ptr dest_switch)
    {
      dest_switch_ = dest_switch;
    }

    void
    ipc_event::serialize_order(sst_serializer* ser)
    {
      ser->do_primitive(time_);
      ser->do_obj(dest_switch_);
      ser->do_obj(msg_to_deliver_);
    }


  }
}
