/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_IPC_EVENT_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_IPC_EVENT_H_INCLUDED

#include <sstmac/common/sstevent.h>
#include <sstmac/common/messages/serializable.h>

namespace sstmac
{
  namespace native
  {
    // Events passed from interprocess links to logical process queues (and vice versa)
    // Needed for routing to switches that don't belong in an LP.
    class ipc_event : public simple_event,
    public serializable_type<ipc_event>
    {
      ImplementSerializable(ipc_event)

    protected:
      // nodeaddress::ptr src_switch_;
      nodeaddress::ptr dest_switch_;
      ipc_event(const sst_message::ptr &msg, const eventhandler::ptr &handler,
          const nodeaddress::ptr &dst_switch);
    public:
      typedef boost::intrusive_ptr<ipc_event> ptr;

      static ptr
      construct(const sst_message::ptr &msg, const eventhandler::ptr &handler,
          const nodeaddress::ptr &dst_switch);

      // nodeaddress::ptr get_src_switch();

      virtual std::string
      to_string() const {
        return "ipc_event";
      }

      nodeaddress::ptr
      get_dest_switch();

      void
      set_dest_switch(nodeaddress::ptr dest_switch);

      void
      set_event_handler(const eventhandler::ptr &handler);

      virtual void
      serialize_order(sst_serializer* ser);


    };

  }
}

#endif
