/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include <sstmac/backends/native/mpiparallel/parallelprocess.h>
#include <sstmac/backends/native/manager.h>
#include <sstmac/hardware/network/interconnect/switchinterconnect.h>
#include <sstmac/backends/native/mpiparallel/par_eventmap.h>
//#include <sstmac/backends/native/mpiparallel/mpipar_ip_queue.h>

namespace sstmac
{
  namespace native
  {
    using namespace sstmac::hw;

    debug parallelprocess::dbg_;

    parallelprocess::parallelprocess(const partition::ptr &part, int id,
        int num) :
      part_(part), myid_(id), num_(num)
    {
      // TODO Auto-generated constructor stub
      if (!dbg_.is_real())
      {
        dbg_ = debug("<debug> parallelprocess | parallel");
        logger::register_logger(&dbg_);
      }
    }

    parallelprocess::~parallelprocess()
    {
      // TODO Auto-generated destructor stub
    }

    timestamp
    parallelprocess::run(timestamp until)
    {

      SSTMAC_DEBUG << "parallelprocess(" << myid_
            << ")::run - constructing interconnect" << "\n";
      manager::ptr mgr;
      try
      {
        interconnect::ptr interconn = interconnect_factory::get_param("network_name",
            sstmac_env::params);
        switchinterconnect::ptr fab = boost::dynamic_pointer_cast<switchinterconnect>(interconn);
        if (!fab){
            sst_throw(illformederror,
                "MPI parallel is only compatible with switch interconnect");
        }

        // hw::dist_traininterconnect::ptr fab =
        //     hw::dist_traininterconnect::construct(a->params, a->top, a->part);
        // fab->add_nodes_for_lp(a->params, a);
        // Setup eventmap.
        SSTMAC_DEBUG << "parallelprocess(" << myid_
              << ")::run - setting up eventmap" << "\n";
        timestamp z(0);
        timestamp iplat = sstmac_env::params->get_time_param(fab->get_interprocess_link_name());
        std::string iplatstr = sstmac_env::params->get_param(fab->get_interprocess_link_name());
        SSTMAC_DEBUG << "parallelprocess(" << myid_ << ")::run - adding ip_latency parameter " << fab->get_interprocess_link_name() <<
            " with value " << iplat << " and string value " << iplatstr << "\n";

        sstmac_env::params->add_param_override("ip_latency", iplatstr);
        par_eventmap::ptr ev_map = par_eventmap::construct(z, myid_, num_,
            part_, iplat);
        evman_ = ev_map;
       // ev_map->make_queues<mpipar_ipq_in, mpipar_ipq_out> (sstmac_env::params);
        ev_map->set_switches(fab);
        ev_map->params_ = sstmac_env::params;

        sstmac_env::evman_ = evman_;

        // Finish interconnect.
        // For the distributed interconnect, the event manager must be set before
        // switches can be connected.
        // fab->set_eventmanager(ev_map);

        //Setup manager.
        SSTMAC_DEBUG << "parallelprocess(" << myid_
              << ")::run - setting up manager" << "\n";
        mgr = manager::construct(ev_map);
        mgr->set_interconnect(fab);

      }
      catch(const std::exception &e)
      {
        std::cout
            << "----- caught exception during initialization for process "
            << myid_ << ": " << e.what() << "\n";
        MPI_Abort(MPI_COMM_WORLD, 1);
      }
      timestamp ret;
      try
      {
        //UNLOCK
        SSTMAC_DEBUG << "parallelprocess(" << myid_ << ")::run - running."
              << "\n";
        ret = mgr->run(until);

      }
      catch(const std::exception &e)
      {
        std::cout << "----- caught exception while running for process "
            << myid_ << ": " << e.what() << "\n";
        MPI_Abort(MPI_COMM_WORLD, 1);
      }
      mgr->finish();

      return ret;
    }

  }
}

