/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/backends/native/nodeid.h>
#include <sstmac/common/errors.h>

DeclareSerializable(sstmac::native::nodeid);

namespace sstmac { namespace native
  {

  long nodeid::unique_id() const
  {
    return id_;
  }

  void nodeid::serialize_order(sst_serializer* ser)
  {
    ser->do_primitive(id_);
  }



  nodeid::~nodeid()
  {

  }

  std::string nodeid::to_string() const
  {
    std::stringstream ss(std::stringstream::in | std::stringstream::out);
    ss << "nodeid(" << "id: " << id() << ")";
    return ss.str();
  }

  bool nodeid::equals(const nodeaddress::ptr &p) const
  {
    nodeid::ptr nid = boost::dynamic_pointer_cast<nodeid>(p);
    return (nid->unique_id() == unique_id());
  }

  long nodeid::id() const
  {
    return this->unique_id();
  }

  void nodeid::id(long i)
  {
    id_ = i;
  }

  } } // end of namespace sstmac


