/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/nodelist.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace native
  {
  
  // Pointer types.
  

  //
  // Hi there.
  //
  nodelist::nodelist() {
  }

  //
  // Welcome to the new tomorrow.
  //
  nodelist::ptr nodelist::construct() {
    return nodelist::ptr(new nodelist);
  }

  //
  // The number of nodes in the list.
  //
  int64_t nodelist::size() const {
    return nodes_.size();
  }

  //
  // Get the given node in the list.
  //
  sstmac::hw::node::ptr nodelist::get(const nodeid::ptr &id) const {
    int64_t index = id->id();
    if(index < 0 || index >= int(nodes_.size())) {
      throw valueerror("nodelist::get:  Index out of range.");
    }
    return nodes_[index];
  }

  void
  nodelist::finish()
  {
      for (int i=0; i < (int) nodes_.size(); ++i){
          hw::node::ptr nd = nodes_[i];
          if (nd){
              nd->finish();
          }
      }
      nodes_.clear();
  }

  //
  // Add the given node to the list.
  //
  nodeid::ptr nodelist::add(const sstmac::hw::node::ptr &the_node) {
    nodeid::ptr new_id = nodeid::construct(this->size());
    nodes_.push_back(the_node);
    return new_id;
  }

} } // end of namespace sstmac
