/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_SIM_PARTITION_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_SIM_PARTITION_H_INCLUDED

#include <sstmac/common/debug.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sim_parameters.h>

#include <boost/unordered_map.hpp>

namespace sstmac
{
  namespace native
  {
    /**
     * A pair of (from_partition, to_partition)
     */
    typedef std::pair<int, int> lppath;

    /**
     * Class for storing all the partitions given to us by METIS.
     * The inherited boost map stores individial partitions, while
     * the parent_map_ member variable stores a flat map of switch to
     * partition.
     */
    class partition : public boost::unordered_map<int, std::list<int> >,
        public ptr_type
    {
      boost::unordered_map<int, int> parent_map_;
      partition(const std::string &partfile, int s);
      void
      read_partition(const std::string &partfilename);

      static debug dbg_;

    public:
      virtual std::string
      to_string() const
      {
        return "partition";
      }

      typedef boost::intrusive_ptr<partition> ptr;
      static partition::ptr
      construct(const std::string &partfile, int s);

      std::list<int>*
      get_partition(int lpid);
      int
      get_parent(int switch_id);
    };

  }
}

#endif /* LOGICALPROCESS_H_ */
