#include <sstmac/common/eventcallback.h>

namespace sstmac {

static donothingcallback::ptr donothing_cb_;
debug callbackhandler::dbg_;

eventcallback::eventcallback()
    : time_(0)
{
}

eventcallback::~eventcallback()
{
}

void
eventcallback::handle(const sst_message::ptr& msg)
{
    callback(msg);
}

std::string
eventcallback::to_string() const
{
    return "event callback";
}

timestamp
eventcallback::get_time() const
{
    return time_;
}

void
eventcallback::set_time(const timestamp& t)
{
    time_ = t;
}

donothingcallback::ptr
donothingcallback::construct()
{
    if (!donothing_cb_)
        donothing_cb_ = new donothingcallback;
    return donothing_cb_;
}

void
donothingcallback::callback(const sst_message::ptr &msg)
{
}

std::string
donothingcallback::to_string() const
{
    return "do nothing callback";
}

callbackhandler::callbackhandler()
{
    if (!dbg_.is_real())
        dbg_ = debug("<debug> callback");
}

callbackhandler::ptr
callbackhandler::construct()
{
    return new callbackhandler();
}

std::string
callbackhandler::to_string() const
{
    return "callbackhandler";
}

void
callbackhandler::handle(const sst_message::ptr &msg)
{
    if (!msg->has_callback()){
        sst_throw(illformederror, "callback handler received msg without callback");
    }
    eventhandler::ptr cb = msg->pop_callback();
    SSTMAC_DEBUG << "doing callback for handler " << cb->to_string()
                 << " for message " << msg->to_string()
                 << "\n";
    cb->handle(msg);
}

event_step_callback::event_step_callback()
    : executed_(false)
{
}

void
event_step_callback::callback(const sst_message::ptr &msg)
{
    if (executed_){
        //we were executed as an event step
        //do nothing
    }
    else {
        //whatever we got passed to did not execute us as an event step
        //now we have to execute in one fell swoop
        callback(msg->get_byte_length(), 0);
    }
}

std::string
event_step_callback::to_string() const
{
    return "event step callback";
}

}

