/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_EVENTHANDLER_H_INCLUDED
#define SSTMAC_COMMON_EVENTHANDLER_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>

#include <sstmac/common/ptr_type.h>

namespace sstmac
{

/**
 * The main interface for something that can respond to an event (sst_message).
 */
class eventhandler : public virtual ptr_type
{
public:

  typedef boost::intrusive_ptr<eventhandler> ptr;

  static eventhandler::ptr ignore;
protected:
  eventhandler();

public:

  virtual std::string to_string() const = 0;

  virtual ~eventhandler();

  virtual void handle(const sst_message::ptr &msg) = 0;

  /**
   * This function returns which logical process this eventhandler is on.
   * This is probably not going to implemented by many components.
   * @return
   */
  virtual int get_lpid() { return -1; }

  };

class donothinghandler :
    public eventhandler
{
    public:
        static ptr construct();

        std::string to_string() const;

        void handle(const sst_message::ptr &msg);

};

} // end of namespace sstmac
#endif
