/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/eventmanager.h>

namespace sstmac
{

  debug eventmanager::dbg_;
  debug eventmanager::updates_;
  eventmanager::ptr eventmanager::global = 0;

  bool eventmanager::stopped_;

  std::list<stat_logger::ptr> eventmanager::stats_;

  eventmanager::eventmanager() :
    nevents_(0), finish_on_stop_(true)
  {
    if (!dbg_.is_real())
    {
      dbg_ = debug("<debug> eventmanager");
      logger::register_logger(&dbg_);
    }

    if (!updates_.is_real())
    {
      updates_ = debug("<debug> eventupdates | updates");
      logger::register_logger(&updates_);
    }
  }

  eventmanager::~eventmanager()
  {
  }

  void
  eventmanager::set_now(const timestamp &ts)
  {
    if (ts < now_)
    {
      sst_throw(ssterror, "eventmanager::set_now - major error, setting time to %lld which is before now %lld", (long long)ts.ticks_int64(), (long long)now_.ticks_int64());
    }
    now_ = ts;
  }

  void
  eventmanager::finish_stats()
  {
    std::list<stat_logger::ptr>::iterator it2, end2 = stats_.end();
    for (it2 = stats_.begin(); it2 != end2; it2++)
    {
      (*it2)->simulation_finished(now());
    }
    stats_.clear();
  }

  void
  eventmanager::schedule(const timestamp &start_time,
      const eventhandler::ptr &handler, const sst_message::ptr &msg)
  {
    if (handler != eventhandler::ignore)
    {
      if (!handler)
      {
        throw nullerror("eventmap::schedule:  Null eventhandler pointer.");
      }

      simple_event::ptr ev = simple_event::construct(msg, handler);
      if (msg)
        msg->set_scheduled(true);

      if (start_time < now())
      {
        std::cerr << "start_time " << ev->time() << " is before current time "
            << now() << "\n";
        schedule(now(), ev);
      }
      else
      {
        schedule(start_time, ev);
      }
    }
  }

}
