/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/common/fileio.h>
#include <sstmac/common/config.h>
#include <sstmac/common/errors.h>

namespace sstmac {
 std::list<std::string> SSTFileIO::file_paths_;

void
SSTFileIO::add_path(const std::string& path)
{
    file_paths_.push_front(path);
}

void
SSTFileIO::init()
{
    if (file_paths_.empty()) //init the file path lookup
    {
        file_paths_.push_back(CONFIG_SRC_INCLUDE_PATH);
        file_paths_.push_back(CONFIG_INSTALL_INCLUDE_PATH);
    }
}

void
SSTFileIO::open_file(std::ifstream& in, const std::string& filename)
{
    //check to see if it is in the current folder
    in.open(filename.c_str());
    if (in.is_open())
    {
        return;
    }

    std::list<std::string>::const_iterator it, end = file_paths_.end();
    for (it=file_paths_.begin(); it != end; ++it)
    {
        std::string fullpath = (*it) + "/" + filename;
        in.open(fullpath.c_str());
        if (in.is_open())
        {
            return;
        }
    }
}

void
SSTFileIO::not_found(const std::string& filename)
{
    std::stringstream ss;
    ss << "File not found: " << filename << ".  Search path is: \n";
    std::list<std::string>::const_iterator it, end = file_paths_.end();
    for (it=file_paths_.begin(); it != end; ++it)
    {
        ss << *it << "\n";
    }

    throw ioerror(ss.str());
}

}
