/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/messages/serializable.h>


ImplementFactory(sstmac::serializable)
;

namespace sstmac
{
  serializable_factory::builder_map* serializable_factory::builders_ = 0;

  long
  serializable_factory::add_builder(serializable_builder* builder)
  {
    if (builders_ == 0){
        builders_ = new builder_map;
    }
    long new_id = builders_->size();
    (*builders_)[new_id] = builder;
    return new_id;
  }

  serializable::ptr
  serializable_factory::get_serializable(long cls_id)
  {
    builder_map::const_iterator it
        = builders_->find(cls_id);
    if (it == builders_->end()){
        sst_throw(valueerror,
            "class id %ld is not a valid serializable id",
            cls_id);
    }
    serializable_builder* builder = it->second;
    return builder->build();
  }

}
