/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/nodeaddress.h>

DeclareBaseClassSerializable(sstmac::nodeaddress)

DeclareSerializable(sstmac::any_node_address);

namespace sstmac
{

  nodeaddress::ptr nodeaddress::any_addr = new any_node_address;

  nodeaddress::address_converter::ptr nodeaddress::converter_;
  nodeaddress::id_to_node_map_t nodeaddress::id_to_node_map_;

  nodeaddress::ptr
  nodeaddress::get_addr(long id)
  {
      id_to_node_map_t::const_iterator it = id_to_node_map_.find(id);
      if (it == id_to_node_map_.end()){
          sst_throw(valueerror, "could not find nodeaddress %ld in lookup table", id);
      }
      return it->second;
  }

  bool
  operator==(const nodeaddress::ptr &a, const nodeaddress::ptr &b)
  {
    return (a->equals(b));
  }

  bool
  operator!=(const nodeaddress::ptr &a, const nodeaddress::ptr &b)
  {
    return (!a->equals(b));
  }

  void
  any_node_address::serialize_order(sst_serializer *ser)
  {
  }



  bool
  any_node_address::equals(const nodeaddress::ptr &p) const
  {
      //always true no matter what!
      return true;
  }

  long
  any_node_address::unique_id() const
  {
      return any_node_address::id;
  }

}

namespace boost
{
  size_t
  hash_value(const sstmac::nodeaddress::ptr &t)
  {
    return t->unique_id();
  }
}
