/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/parameters/command_parameters.h>
#include <sstmac/keywords/keyword_registration.h>

namespace sstmac
{

  command_parameters::command_parameters()
  {
  }

  command_parameters::ptr
  command_parameters::construct()
  {
    return ptr(new command_parameters());
  }

  std::string
  command_parameters::get_param(const std::string& key) const
  {

    maptype::const_iterator it = params_.find(key);
    if (it == params_.end())
    {
      sst_throw(sstmac::valueerror,
          "native_parameters: could not find parameter %s", key.c_str());
    }
    return it->second;
  }

  bool
  command_parameters::has_param(const std::string& key) const
  {
    maptype::const_iterator it = params_.find(key);
    return (it != params_.end());
  }

  void
  command_parameters::add_param(const std::string& key, const std::string& val)
  {
    SSTMAC_DEBUG << "setting key " << key << " to value " << val << "\n";

    if (params_.find(key) != params_.end())
    {
      throw sstmac::valueerror(
          "command_parameters::add_param - key already in params");
    }
    else
    {
      KeywordRegistration::validate_keyword(key, val);
      params_[key] = val;
    }
  }

  void
  command_parameters::add_param_override(const std::string& key,
      const std::string& val)
  {
    SSTMAC_DEBUG << "setting key " << key << " to value " << val << "...";

    if (params_.find(key) != params_.end())
    {
      SSTMAC_DEBUG << "overriding \n";
    }
    else
    {
      SSTMAC_DEBUG << "adding new param \n";
    }
    KeywordRegistration::validate_keyword(key, val);
    params_[key] = val;

  }

  void
  command_parameters::combine_into(sim_parameters::ptr sp)
  {
    BOOST_FOREACH (maptype::value_type it, params_)
          {
            sp->add_param_override(it.first, it.second);
          }
  }

  std::string
  command_parameters::to_string() const
  {
    return "command_parameters";
  }

  command_parameters::~command_parameters()
  {
  }

  void
  command_parameters::remove_param(const std::string & key)
  {
    params_.erase(key);
  }

  command_parameters::command_parameters(const maptype& p) :
    params_(p)
  {

  }

  sim_parameters::ptr
  command_parameters::clone()
  {
    return ptr(new command_parameters(params_));
  }
}
