/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_PARAMETERS_COMMAND_PARAMETERS_H_INCLUDED
#define SSTMAC_COMMON_PARAMETERS_COMMAND_PARAMETERS_H_INCLUDED

#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  class command_parameters : public sstmac::sim_parameters
  {
  protected:
    maptype params_;

  protected:
    command_parameters();

    command_parameters(const maptype& p);

  public:
    typedef boost::intrusive_ptr<command_parameters> ptr;

    static ptr
    construct();

    virtual sim_parameters::ptr
    clone();

    sim_parameters::ptr
    subspace_clone()
    {
      return new command_parameters;
    }

    virtual std::string
    get_param(const std::string& key) const;

    virtual void
    remove_param(const std::string & key);

    virtual bool
    has_param(const std::string& key) const;

    virtual void
    add_param(const std::string& key, const std::string& val);

    virtual void
    add_param_override(const std::string& key, const std::string& val);

    virtual void
    combine_into(sim_parameters::ptr sp);

    virtual std::string
    to_string() const;

    virtual
    ~command_parameters();

    virtual void
    print_params(std::ostream& os = std::cerr) const
    {
      sim_parameters::print_params(params_, os);
    }

  };

}
#endif
