/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_PARAMETERS_FILE_PARAMETERS_H_INCLUDED
#define SSTMAC_COMMON_PARAMETERS_FILE_PARAMETERS_H_INCLUDED

#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  class file_parameters : public sstmac::sim_parameters
  {
  protected:
    maptype params_;

    std::vector<std::string> multisim_includes_;

  protected:
    file_parameters();

    file_parameters(const maptype& p);

  public:
    typedef boost::intrusive_ptr<file_parameters> ptr;

    void
    try_to_parse(const std::string& fname, bool fail_on_existing = false);

    void
    parse_file(const std::string& fname, bool fail_on_existing = false);

    std::vector<std::string> &
    get_multisim_includes(){
      return multisim_includes_;
    }

    static ptr
    construct(const std::string& filename);

    sim_parameters::ptr
    subspace_clone()
    {
      return new file_parameters;
    }


    virtual sim_parameters::ptr
    clone();

    virtual bool
    is_multi_sim() const
    {
      return multisim_params_.size() > 0 || multisim_includes_.size() > 0;
    }

    void
    append_params(const std::string& filename);

    virtual std::string
    get_param(const std::string& key) const;

    virtual void
    remove_param(const std::string & key);

    virtual bool
    has_param(const std::string& key) const;

    virtual void
    add_param(const std::string& key, const std::string& val);

    virtual void
    add_param_override(const std::string& key, const std::string& val);

    virtual void
    combine_into(sim_parameters::ptr sp);

    virtual std::string
    to_string() const;

    virtual
    ~file_parameters();

    virtual void
    print_params(std::ostream& os = std::cerr) const
    {
      sim_parameters::print_params(params_, os);
    }

  };
}
#endif
