

/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_PTR_TYPE_H_INCLUDED
#define SSTMAC_COMMON_PTR_TYPE_H_INCLUDED



namespace sstmac{
  class ptr_type;
  
}

namespace boost
{
  void intrusive_ptr_add_ref(sstmac::ptr_type * p);
  void intrusive_ptr_release(sstmac::ptr_type * p);
  
  void intrusive_ptr_add_ref(const sstmac::ptr_type * p);
  void intrusive_ptr_release(const sstmac::ptr_type * p);
}

#include <boost/intrusive_ptr.hpp>
#include <typeinfo>
#include <sstmac/common/typedefs.h>
#include <sstmac/common/errors.h>

namespace sstmac {
   

  /**
   * Base class to implement functions for intrusive_ptr.  Any child class must use
   * the construct mechanism and never create a local instance or un-smart pointer.
   */
  class ptr_type {
  private:
      mutable long  references;
      friend void ::boost::intrusive_ptr_add_ref(ptr_type * p);
      friend void ::boost::intrusive_ptr_release(ptr_type * p);

      friend void ::boost::intrusive_ptr_add_ref(const ptr_type * p);
      friend void ::boost::intrusive_ptr_release(const ptr_type * p);


    public:
      typedef boost::intrusive_ptr<ptr_type> ptr;

      ptr_type();

      virtual ~ptr_type();

      virtual std::string to_string() const = 0;

      long nreferences() const;
  };

   template <class Out, class In>
   boost::intrusive_ptr<Out>
   __safe_cast__(const boost::intrusive_ptr<In>& in,
             const char* objname,
             const char* file,
             int line)
   {
        boost::intrusive_ptr<Out> out = boost::dynamic_pointer_cast<Out>(in);
        if (!out){
            sst_throw(valueerror, "failed to cast to %s at %s:%d",
                      objname, file, line);
        }
        return out;
   }

   #define safe_cast(obj, type) \
    __safe_cast__<type>(obj, #type, __FILE__, __LINE__)



   template <class maptype, class keytype>
   const typename maptype::mapped_type&
   mapget(const maptype& mapobj, const keytype& keyobj, const char* errormsg){
       typename maptype::const_iterator it = mapobj.find(keyobj);
       if (it == mapobj.end()){
           sst_throw(valueerror, "could not find entry in map: %s", errormsg);
       }
       return it->second;
   }

} // end of namespace sstmac

#include <sstmac/common/types.h>

#endif
