/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/stats/event_trace.h>

namespace sstmac
{

  void
  event_trace::simulation_finished(timestamp end)
  {
    if (is_active() && check_open())
    {
      myfile.close();
    }
  }

  void
  event_trace::collect(int event_typeid,
      const std::string& name,
      const nodeaddress::ptr& node,
      int aff, long threadid,
      int aid, int tid,
      long ticks_begin,
      long num_ticks)
  {
    if (is_active() && check_open() && ticks_begin >= start_ && (ticks_begin + num_ticks) <= stop_)
    {
      myfile << ticks_begin << ", " << num_ticks << ", " << name << ", "
          << threadid << ", " << event_typeid << ", " << node->unique_id() << ", " << aff << "\n";
    }
  }

}
