/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_DISK_VESPER_DISKREQUESTQUEUE_H_INCLUDED
#define SSTMAC_HARDWARE_DISK_VESPER_DISKREQUESTQUEUE_H_INCLUDED

#include <sstmac/hardware/disk/vesper/disk.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/software/libraries/filesystem/disk_message.h>
#include <sstmac/hardware/failure/failable.h>
#include <sstmac/common/stats/stat_time_window.h>

namespace sstmac
{
  namespace hw
  {

    class disk_controller : public eventmodule, public failable
    {
    protected:
      disk_controller(long nodeid, int num, const char *profile,
          const char *name, eventhandler::ptr callWhenDone,
          eventmanager::ptr man);

      virtual void
      do_access(sstmac::sw::disk_message::ptr da);

      static debug dbg_;
      static stat_time_window_sum::ptr stat_readbytes_;
      static stat_time_window_sum::ptr stat_writebytes_;

    public:
      typedef boost::intrusive_ptr<disk_controller> ptr;

      static ptr
      construct(long nodeid, int num, const char *profile, const char *name,
          eventhandler::ptr callWhenDone, eventmanager::ptr man)
      {
        return ptr(new disk_controller(nodeid, num, profile, name, callWhenDone, man));
      }

      static ptr
      construct(long nodeid, sim_parameters::ptr params, eventhandler::ptr handle,
          eventmanager::ptr man)
      {
        int num = params->get_int_param("disk_num_per_cntrl");
        std::string pro = params->get_param("disk_profile");

        return construct(nodeid, num, pro.c_str(), "", handle, man);
      }

      virtual
      ~disk_controller()
      {
      }

      virtual void
      handle(const sst_message::ptr &msg);

      virtual void
      request_access(sst_message::ptr req);

      virtual std::string
      to_string() const
      {
        return "disk_controller";
      }

      virtual long
      failure_id() const
      {
        return nodeid_;
      }

    protected:
      int numdisk_;
      std::vector<Disk::ptr> disk_;
      eventhandler::ptr handler_;
      std::queue<sstmac::sw::disk_message::ptr> q_;
      int lock_;
      sstmac::sw::disk_message::ptr current_;
      long nodeid_;
    };

  }
} /* namespace sstmac */
#endif /* DISKREQUESTQUEUE_H_ */
