/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_FAILURE_FAILURE_MODEL_H_INCLUDED
#define SSTMAC_HARDWARE_FAILURE_FAILURE_MODEL_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>
#include <sstmac/hardware/failure/failure_event.h>
namespace sstmac
{
  class eventmodule;

  namespace hw
  {



    class failure_model : public ptr_type
    {

    protected:
      static debug dbg_;

      failure_model()
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> failure_model");
            logger::register_logger(&dbg_);
          }
      }

    public:
      typedef boost::intrusive_ptr<failure_model> ptr;

      virtual
      ~failure_model()
      {
      }

      virtual std::string
      to_string() const
      {
        return "failure_model";
      }

      virtual void
      next_failure(boost::intrusive_ptr<eventmodule> parentmodel) = 0;

      /**
       * propagate - this function is called when a parent must signal that its
       * children should possibly fail directly because of it (like a power supply)
       */
      virtual void
      propagate() = 0;

    };

  }
}
#endif
