/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_COMPONENT_H_
#define SSTMAC_HARDWARE_FASTFLOWER_COMPONENT_H_

#include <sstmac/hardware/common/component.h>
#include <sstmac/common/debug.h>
#include <sstmac/hardware/fastflower/fastflow_message.h>

namespace sstmac
{
  namespace hw
  {

#define HISTORY_SIZE 20

    struct flowevent
    {
      timestamp time_;
      int numflows_;
      double overcommit_;
    };

    class fastflower_component : virtual public component
    {

    private:
      static debug flow_dbg_;
    protected:

      double unit_bw_;
      double capacity_bw_;

      double current_bw_;
      int numflows_;

      timestamp unit_delay_;

      std::string name_;

      std::list<flowevent*> history_;

      fastflower_component(std::string n = "ff_component") :
        name_(n)
      {
        current_bw_ = 0;
        numflows_ = 0;

        flowevent* f = new flowevent;
        f->time_ = timestamp(0);
        f->numflows_ = 0;
        f->overcommit_ = 0;
        history_.push_front(f);

        if (!flow_dbg_.is_real())
        {
          flow_dbg_ = debug("<debug> fastflow | fastflower", logger::red);
          logger::register_logger(&flow_dbg_);
        }
      }

      void
      init_params(double u, double c, timestamp d)
      {
        unit_bw_ = u;
        capacity_bw_ = c;
        unit_delay_ = d;
      }

    public:
      typedef boost::intrusive_ptr<fastflower_component> ptr;
      typedef boost::intrusive_ptr<const fastflower_component> const_ptr;

      virtual
      ~fastflower_component()
      {
      }

      virtual std::string
      to_string() const
      {
        return "fastflower_component";
      }

      double
      get_current_bw() const {
        return current_bw_;
      }

      virtual timestamp
      model(const fastflow_message::ptr &msg, bool &reschedule);

      virtual void
      finish()
      {
        component::finish();

        while (history_.size() > 0)
        {
          flowevent* f = history_.back();
          delete f;
          history_.pop_back();
        }
      }

      //virtual void
      //forward(fastflow_message::ptr fmsg, timestamp delay) = 0;

    };

  }
}
#endif
