/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_LINK_H_
#define SSTMAC_HARDWARE_FASTFLOWER_LINK_H_

#include <sstmac/hardware/fastflower/fastflower_component.h>

namespace sstmac
{
  namespace hw
  {

    class fastflower_link : public fastflower_component
    {

    protected:
      fastflower_component::ptr from_;
      fastflower_component::ptr to_;

      fastflower_link(fastflower_component::ptr from,
          fastflower_component::ptr to, double u, double c, timestamp t) :
        fastflower_component("link"), from_(from), to_(to)
      {

        init_params(u, c, t);
      }

    public:
      typedef boost::intrusive_ptr<fastflower_link> ptr;

      static ptr
      construct(fastflower_component::ptr from, fastflower_component::ptr to,
          double u, double c, timestamp t)
      {
        return ptr(new fastflower_link(from, to, u, c, t));
      }

      virtual
      ~fastflower_link()
      {
      }

      virtual std::string
      to_string() const
      {
        return "fastflower_link";
      }

      virtual void
      handle(const sst_message::ptr &msg)
      {
        fastflow_message::ptr fmsg = boost::dynamic_pointer_cast<
            fastflow_message>(msg);

        if (!fmsg)
        {
          sst_throw(ssterror, "fastflower_component::handle - can't cast %s to a fastflow_message", msg->to_string().c_str());
        }

        bool re;
        timestamp d = model(fmsg, re);
        if (re)
          schedule(now() + d, ptr(this), msg);
        else
          schedule(now() + d, to_, fmsg);
      }

    };

  }
}
#endif
