/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_FASTFLOWER_NIC_H_
#define SSTMAC_HARDWARE_FASTFLOWER_NIC_H_

#include <sstmac/hardware/fastflower/fastflower_component.h>
#include <sstmac/hardware/fastflower/fastflower_memory.h>
#include <sstmac/hardware/fastflower/fastflower_jector.h>
#include <sstmac/hardware/nic/networkinterface.h>

namespace sstmac
{
  namespace hw
  {

    class fastflower_nic : public networkinterface
    {
    public:
      typedef boost::intrusive_ptr<fastflower_nic> ptr;

    protected:
      fastflower_nic()
      {
        memdonecb_ = msg_callback(ptr(this), &fastflower_nic::mem_done);
      }

      eventcallback::ptr memdonecb_;
      eventcallback::ptr injcb_;
      eventcallback::ptr tailcb_;
      fastflower_memory::ptr mem_;
      fastflower_injector::ptr inj_;

      boost::unordered_map<sst_message::ptr, long> bytes_recv_;

      void recv(const sst_message::ptr& netmsg);

      void finish_recv_ack(const sst_message::ptr& netmsg);

      void finish_recv_req(const sst_message::ptr& netmsg);

    public:

      static ptr
      construct(const sim_parameters::ptr &params)
      {
        return ptr(new fastflower_nic());
      }

      virtual
      ~fastflower_nic()
      {
      }

      virtual std::string
      to_string() const
      {
        return "fastflower_nic";
      }

      virtual void
      init(const node::ptr &parent, const interconnectPtr &interconn);

      virtual void
      finish()
      {
        mem_ = 0;
        memdonecb_ = 0;
        injcb_ = 0;
        networkinterface::finish();
      }

      virtual void
      set_eventmanager(const eventmanager::ptr &m){
        networkinterface::set_eventmanager(m);
        inj_->set_eventmanager(m);
      }

      virtual void
      connect(connection_type_t ty, const connectable::ptr& mod);

      void
      mem_done(const sst_message::ptr &msg);

      void
      tail_going_out(const sst_message::ptr &msg);

      //virtual void
      //handle(const sst_message::ptr &msg);

      /// Get a new nic of the same type as the current one.
      virtual networkinterface::ptr
      clone(const node::ptr& parent, const interconnect::ptr& interconn,
          const sim_parameters::ptr& params) const
      {
        fastflower_nic::ptr ret = fastflower_nic::construct(params);
        ret->init(parent, interconn);
        return ret;
      }

      virtual void
      send(const sst_message::ptr &payload);

    };
  }
}
#endif
