/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/node/node.h>

ImplementFactory(sstmac::hw::memorymodel);

namespace sstmac
{
  namespace hw
  {
    stat_sum::ptr memorymodel::mem_bytes_;
    debug memorymodel::dbg_;


    memorymodel::memorymodel()
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> memory");
          logger::register_logger(&dbg_);
        }

      if (!mem_bytes_)
        {
          mem_bytes_ = stat_sum::construct("<stats> mem_bytes",
              "memory_traffic.csv");
          register_stat( mem_bytes_);
        }
    }

    void
    memorymodel::init(const nodePtr &parent_node, const computescheduler::ptr &sched)
    {
        parent_node_ = parent_node;
        nodeid_ = parent_node->get_id()->unique_id();
        sched_ = sched;
        done_ = parent_node_->get_msg_handler();
    }

    void
    memorymodel::clear_statics()
    {
        mem_bytes_ = 0;
    }

    void
    memorymodel::finish()
    {
        if (finished_)
            return;

        eventmodule::finish();
        parent_node_ = 0;
    }

    memorymodel::~memorymodel()
    {
    }

    timestamp
    memorymodel::link::new_access(timestamp now, long size)
    {
      timestamp n(std::max(now.sec(), last_access_.sec()));
      timestamp access = lat_ + timestamp((double) size / (double) bw_);
      last_access_ = n + access;
      timestamp delta = last_access_ - now;
      return delta;
    }

    void
    memorymodel::link::access_done()
    {

    }

  }
} /* namespace sstmac */
