/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef MEMORYMODEL_H_
#define MEMORYMODEL_H_

#include <sstmac/common/eventmodule.h>
#include <sstmac/hardware/failure/failable.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/common/stats/stats_common.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/common/factories/factory.h>
#include <sstmac/hardware/common/connection.h>


namespace sstmac
{
  namespace hw
  {

    class node;
    typedef boost::intrusive_ptr<node> nodePtr;

    class computescheduler;
    typedef boost::intrusive_ptr<computescheduler> computeschedulerPtr;

    class memorymodel :
            virtual public eventmodule,
            public connectable,
            public failable,
            public factory_type
    {

    protected:
      memorymodel();

      long nodeid_;
      nodePtr parent_node_;
      static debug dbg_;
      eventhandler::ptr done_;
      computescheduler::ptr sched_;
      static stat_sum::ptr mem_bytes_;
      //std::map<connection_type_t,eventhandler::ptr> connections_;

    public:
      static void clear_statics();

      typedef boost::intrusive_ptr<memorymodel> ptr;

      virtual void init(const nodePtr& parent_node, const computescheduler::ptr& sched);

      virtual ~memorymodel();

      //void connect(connection_type_t ty, const connectable::ptr &mod);

      virtual void
      access(const sst_message::ptr& msg) = 0;

      virtual std::string
      to_string() const
      {
        return "memorymodel";
      }

      virtual void finish();

      virtual long
      failure_id() const
      {
        return nodeid_;
      }

      class link : public ptr_type
      {

      protected:
        double bw_;
        timestamp lat_;

        timestamp last_access_;

        link(double bw, timestamp lat) :
          bw_(bw), lat_(lat), last_access_(0)
        {

        }

      public:
        typedef boost::intrusive_ptr<link> ptr;

        static ptr
        construct(double bw, timestamp lat)
        {
          return ptr(new link(bw, lat));
        }

        virtual ~link() { }

        virtual std::string
        to_string() const {
          return "memorymodel::link";
        }

        virtual timestamp
        new_access(timestamp now, long size);

        virtual void
        access_done();
      };

    };

    DeclareFactory2InitParams(memorymodel,nodePtr,computescheduler::ptr);

  }
} /* namespace sstmac */
#endif /* MEMORYMODEL_H_ */
