/*
 * dist_traininterconnect.h
 *
 *  Created on: Jan 27, 2013
 *      Author: amsmit
 */

/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_TRAIN_DIST_TRAININTERCONNECT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TRAIN_DIST_TRAININTERCONNECT_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/hardware/packettrain/trainswitch.h>
#include <boost/unordered_map.hpp>
#include <sstmac/common/driver_util.h>

namespace sstmac
{
  namespace hw
  {

    class dist_traininterconnect :
        public interconnect
    {

    protected:

      dist_traininterconnect(const topology::ptr& top,
                        const sim_parameters::ptr& params,
                        std::list<int> switch_indices);

      topology::maptype switches_;
      int nps_;

    public:
      typedef boost::intrusive_ptr<dist_traininterconnect> ptr;

      virtual std::string
      to_string() const
      {
        return "dist_switchinterconnect";
      }

      static ptr
      construct(const sim_parameters::ptr& params, const topology::ptr& top, std::list<int> switch_indices)
      {
        return new dist_traininterconnect(top, params, switch_indices);
      }

      static ptr
      construct(const sim_parameters::ptr& params)
      {
          sst_throw(ssterror, "Cannot use InterconnectFactory to construct a distributed interconnect. Must use explicit construct call.");
          return NULL;
      }

      void
      connect_switches();

      topology::maptype&
      get_switches()
      {
          return switches_;
      }


      virtual ~dist_traininterconnect();

      virtual void finish();

      /*virtual void
      set_endpoint(const nodeaddress::ptr& addr, const eventhandler::ptr& nic);*/

      /// Send the given message at the given time.
      /// Notify the sender node when the send is complete
      /// and notify the receiver node when receive needs to be performed.
      virtual void
      send(const timestamp &start_time, const sst_message::ptr &msg);

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual std::string
      label(long node_id) const;

    };

  }
}

#endif
// TRAININTERCONNECT_H
