#ifndef sstmac_hardware_network_interconnect_INTERCONNECT_MESSAGE_H
#define sstmac_hardware_network_interconnect_INTERCONNECT_MESSAGE_H

#include <sstmac/hardware/network/network_message.h>
#include <sstmac/hardware/network/topology/routing/routing_info.h>

namespace sstmac {
    namespace hw {

  class interconnect_message :
    virtual public sst_message,
    public serializable_type<interconnect_message>
  {
    ImplementSerializable(interconnect_message)

    protected:
        sst_message::ptr orig_;

        long num_bytes_;

        routing_info::ptr rinfo_;

        interconnect_message(
            const sst_message::ptr& msg,
            long num_bytes,
            sst_message::MESSAGE_TYPES ty);

        interconnect_message(const sst_message::ptr& msg);


        interconnect_message() { }

    public:
        typedef boost::intrusive_ptr<interconnect_message> ptr;

        static ptr
        construct(const sst_message::ptr& orig){
            return new interconnect_message(orig);
        }

        bool routing_initialized() const {
            return rinfo_;
        }

        void init_routing(const routing_info::ptr& rinfo) {
            rinfo_ = rinfo;
        }

        network_message::ptr
        get_network_msg() const;

        nodeaddress::ptr toaddr() const {
            return orig_->toaddr();
        }

        nodeaddress::ptr fromaddr() const {
            return orig_->fromaddr();
        }

        sst_message::ptr orig() const {
            return orig_;
        }

        sst_message::ptr get_parent_msg() const {
            return orig_;
        }



        virtual void
        serialize_order(sst_serializer* ser);



      virtual long
      get_byte_length() const
      {
        return num_bytes_;
      }

      virtual nodeaddress::ptr
      dest()
      {
        return orig_->toaddr();
      }

      routing_info::ptr
      rinfo()
      {
        return rinfo_;
      }
  };


    }
}

#endif // INTERCONNECT_MESSAGE_H
