/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_CIRCUIT_CIRCUITMESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_CIRCUIT_CIRCUITMESSAGE_H_INCLUDED

#include <sstmac/hardware/network/network_message.h>
#include <stack>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {
    class circuitmessage :
        public interconnect_message
    {

    public:
      enum CIRCUIT_TYPE
      {
        SETUP, TEARDOWN, BLOCKED, TURNING, PATH_ACK, DATA
      };
      typedef boost::intrusive_ptr<circuitmessage> ptr;

    protected:
      circuitmessage(sst_message::ptr orig, long size, CIRCUIT_TYPE t) :
        interconnect_message(orig, size, sst_message::NETWORK),
            circtype_(t), backoff_(0)
      {

      }

      CIRCUIT_TYPE circtype_;

    public:

      int backoff_;
      std::stack<discrete_network_switch::ptr> stack_;
      timestamp ttl_;
      timestamp arrive_;

      static ptr
      construct(sst_message::ptr orig, long size, CIRCUIT_TYPE t)
      {
        return ptr(new circuitmessage(orig, size, t));
      }

      virtual nodeaddress::ptr
      dest()
      {
        return orig_->toaddr();
      }

      virtual CIRCUIT_TYPE
      get_circ_type() const
      {
        return circtype_;
      }

      virtual void
      set_circ_type(CIRCUIT_TYPE t)
      {
        circtype_ = t;
      }

      virtual std::string
      to_string() const
      {
        std::string t;
        switch(circtype_){
        case PATH_ACK:
          t = "PATH_ACK";
          break;
        case SETUP:
          t = "SETUP";
          break;
        case BLOCKED:
          t = "BLOCKED";
          break;
        case TEARDOWN:
          t = "TEARDOWN";
          break;
        case TURNING:
          t = "TURNING";
          break;
        case DATA:
          t = "DATA";
          break;
        }
        return "circuitmessage("+t+")";
      }

      virtual bool
      get_needs_ack() const {
        return false;
      }

      virtual
      ~circuitmessage()
      {
      }
    };
  }
}

#endif
