
/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_HYPERCUBE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_HYPERCUBE_H_INCLUDED

#include <sstmac/hardware/network/topology/hdtorus.h>

namespace sstmac
{
  namespace hw
  {

    class hypercube :
        public hdtorus
    {

    protected:
      int radix_;
      int ndim_;
      std::vector<int> dim_to_outport_;

      hypercube();

    public:
      typedef boost::intrusive_ptr<hypercube> ptr;

      static ptr
      construct(const sim_parameters::ptr& params);

      void init_factory_params(const sim_parameters::ptr &params);

      int
      get_productive_dir(
        int &dim,
        const coordinates &src,
        const coordinates &dst) const;

      virtual int
      radix() const {
        return radix_;
      }

      virtual
      ~hypercube(){}

      std::string
      name() const;

      virtual void
      minimal_route_to_coords(
        const coordinates &current_coords,
        const coordinates &dest_coords,
        int &dim, int &dir) const;

      virtual void
      connect_switches(topology::maptype &switches);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual std::string
      to_string() const
      {
        return "hdtorustopology";
      }

      int
      minimal_distance(
        const coordinates& current_coords,
        const coordinates& dest_coords
      ) const;

      virtual void
      display_nodes(const vismaptype &switches,
          const vis::vis_engine::ptr& eng,
          std::list<vis::vis_obj::ptr> &objs);

    };

  }
} //end of namespace sstmac

#endif // HYPERCUBE_H
