#include <sstmac/hardware/network/topology/routing/minad_routing.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac {
    namespace hw {

    SSTRegister("min_ad", routing_algorithm, minimal_adaptive_routing,
                "a routing algorithm for minimal adaptive routing on regular topologies");

    int
    minimal_adaptive_routing::route(const interconnect_message::ptr &msg)
    {
        router::outports ports;
        bool eject  = router_->get_productive_ports_to_node(msg->toaddr(), ports);
        if (eject){
            return ports[0];
        }
        //loop through the ports and find the least-congested
        int min_queue_length = sw_->get_queue_length(ports[0]);
        int min_queue = ports[0];
        SSTMAC_DEBUG << sstprintf("Routing %p from %ld to %ld:"
                                "  path 0 port=%d queue=%d",
                                msg.get(),
                                sw_->get_addr()->unique_id(),
                                msg->toaddr()->unique_id(),
                                ports[0], min_queue_length);

        for (int i=1; i < ports.size(); ++i){
            int test_length = sw_->get_queue_length(ports[i]);
            SSTMAC_DEBUG << sstprintf("  path %d: port=%d queue=%d",
                                    i, ports[i], test_length);
            if (test_length < min_queue_length){
                min_queue = ports[i];
                min_queue_length = test_length;
            }
        }
        SSTMAC_DEBUG << sstprintf("  : Chose %d\n", min_queue);
        return min_queue;
    }

    void
    minimal_adaptive_routing::init_routing(const interconnect_message::ptr &msg) const
    {
        msg->init_routing(min_ad_routing_info::construct());
    }

    }
}
