#include <sstmac/hardware/network/topology/routing/regular_router.h>

namespace sstmac {
    namespace hw{

    SSTRegister("regular", router, regular_router,
                "a router for regular topologies that implements basic routing");

    SSTRegister("ordered", router, dimension_order_router,
                "a router for regular topologies that implements basic routing in dimension order");

    void
    regular_router::init(const nodeaddress::ptr &me, const topology::ptr &top)
    {
        nps_ = top->nps(me);
        router::init(me, top);
    }

    void
    regular_router::minimal_route_to_node(const nodeaddress::ptr& dest_node_addr, int &dim, int &dir)
    {
        nodeaddress::ptr ej_addr = regtop_->node_to_ejector_addr(dest_node_addr, dir);
        if (ej_addr->equals(sw_->get_addr())){
            dim = topology::EJECT;
        }
        else {
            minimal_route_to_switch(ej_addr, dim, dir);
        }
    }

    void
    regular_router::minimal_route_to_switch(const nodeaddress::ptr& dest_sw_addr, int &dim, int &dir)
    {
        regtop_->minimal_route_to_switch(
               sw_->get_addr(),
               dest_sw_addr,
               dim, dir
             );
    }

    int
    regular_router::get_productive_dir(
        int& dim,
        const coordinates &current,
        const coordinates &dst) const
    {
        return regtop_->get_productive_dir(dim, current, dst);
    }

    void
    regular_router::get_productive_outports(
        outports &prod_ports,
        const coordinates &current,
        const coordinates &dst)
    {
        for (int i=0; i < current.size(); ++i)
        {
            if (current[i] != dst[i]){
                int dim = i;
                int dir = get_productive_dir(dim, current, dst);
                prod_ports.push_back(convert_to_outport(dim, dir));
            }
        }
    }

    void
    regular_router::get_productive_ports_to_switch(
        const nodeaddress::ptr &dst, outports &ports)
    {
        coordinates my_coords = regtop_->get_switch_coords(sw_->get_addr());
        coordinates dst_coords = regtop_->get_switch_coords(dst);
        ports.reserve(my_coords.size());
        get_productive_outports(ports, my_coords, dst_coords);
    }

    int
    regular_router::basic_route(const interconnect_message::ptr &msg)
    {
        int dim, dir;
        sw_->get_topology()->minimal_route_to_node(
            sw_->get_addr(),
            msg->toaddr(),
            dim, dir
        );
        return convert_to_outport(dim, dir);
    }

    void
    regular_router::set_switch(const networkswitch::ptr &sw)
    {
        router::set_switch(sw);
        regtop_ = safe_cast(sw->get_topology(), regular_topology);
    }

    dimension_order_router::ptr
    dimension_order_router::construct(const sim_parameters::ptr &params)
    {
        return new dimension_order_router;
    }

    void
    dimension_order_router::get_productive_outports(
        outports &prod_ports,
        const coordinates &current,
        const coordinates &dst)
    {
        for (int i=0; i < current.size(); ++i)
        {
            if (current[i] != dst[i]){
                int dim = i;
                int dir = get_productive_dir(dim, current, dst);
                prod_ports.push_back(convert_to_outport(dim, dir));
                return;
            }
        }
    }


    }
}
