/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTING_INFO_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_ROUTING_ROUTING_INFO_H_INCLUDED

#include <sstmac/common/messages/serializable.h>
#include <boost/unordered_map.hpp>
#include <sstmac/common/nodeaddress.h>
#include <sstmac/common/factories/factory.h>

namespace sstmac
{
  namespace hw
  {

    class router;

    class routing_info :
        public serializable
    {
      public:
        typedef enum { minimal,
                       minimal_adaptive,
                       fully_adaptive,
                       basic,
                       valiant,
                       ugal,
                       system_default }
                       routing_algorithm_t;

        typedef boost::intrusive_ptr<routing_info> ptr;

        virtual routing_algorithm_t
        route_algo() const = 0;

        virtual routing_info::ptr
        clone() = 0;

        static const char*
        tostr(routing_algorithm_t algo);

    };

    class minimal_routing_info :
        public routing_info,
        public serializable_type<minimal_routing_info>
    {
        ImplementSerializable(minimal_routing_info)

        public:
            typedef boost::intrusive_ptr<minimal_routing_info> ptr;

            minimal_routing_info(){}

            routing_algorithm_t
            route_algo() const {
                return minimal;
            }

            static ptr
            construct() {
                return new minimal_routing_info;
            }

            virtual routing_info::ptr
                    clone(){
              return construct();
            }

            virtual std::string
            to_string() const {
                return "minimal routing";
            }



            virtual void serialize_order(sst_serializer *ser)  {
                //do nothing
            }


    };

  }
}
#endif
