/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/hardware/nic/multinic.h>
//#include <sstmac/simulator/networkdata.h>
//#include <sstmac/process/recvhandler.h>
#include <sstmac/common/errors.h>
#include <sstmac/software/process/waiter.h>
#include <sstmac/software/ami/ami.h>
#include <sstmac/common/eventcallback.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/common/driver_util.h>
#include <sstmac/common/prefix_params.h>
#include <sstmac/hardware/network/interconnect/multinetwork.h>

namespace sstmac
{
  namespace hw
  {
    SSTRegister("multinic | multi", networkinterface, multinic);
    //
    // Hi.
    //
    multinic::multinic() :
      networkinterface()
    {

    }

    void
    multinic::init(const node::ptr &parent, const interconnect::ptr &interconn)
    {
      networkinterface::init(parent, interconn);

      multinetwork::ptr multinet = boost::dynamic_pointer_cast<multinetwork>(
          interconn);
      if (!multinet)
        {
          throw ssterror(
              "multinic::ctor - can't cast interconnect to multinetwork, have to use the two together");
        }
      int num = 1;
      std::stringstream pre;
      pre << "multi" << num << "_";
      prefix_params::ptr par = prefix_params::construct(sstmac_env::params,
          pre.str());
      std::string name = pre.str() + "network_name";
      bool go = sstmac_env::params->has_param(name);
      while (go)
        {
          SSTMAC_DEBUG << "multinic: constructing a nic at index " << num
                << ": "
                << sstmac_env::params->get_param(pre.str() + "nic_name")
                << "\n";
          // interconnect::ptr net = construct_circuit(par);
          // networks_[num] = net;
          networkinterface::ptr nic = networkinterface_factory::get_value(
              sstmac_env::params->get_param(pre.str() + "nic_name"), par,
              parent, multinet->get_network(num));

          nics_[num] = nic;
          num++;
          pre.clear();
          pre.str(std::string());
          pre << "multi" << num << "_";
          name = pre.str() + "network_name";
          SSTMAC_DEBUG << "multinic: checking for " << name << "\n";
          go = sstmac_env::params->has_param(name);
          par->set_prefix(pre.str());
        }
      num--;

      div_ = divider_factory::get_param("multi_divider", sstmac_env::params);
      div_->init(num, nics_);

    }

    void
    multinic::send(const sst_message::ptr &payload)
    {
      int n = div_->which(payload);
      SSTMAC_DEBUG << "multinic: sending message " << payload->to_string()
            << " to network " << n << "\n";
      if (nics_.find(n) == nics_.end())
        {

          throw ssterror("multinic::send: no network at index");
        }
      nics_[n]->send(payload);

    }

    networkinterface::ptr
    multinic::clone(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn,
        const sim_parameters::ptr& params) const
    {
      multinic::ptr ret = multinic::construct(params);
      ret->init(parent, interconn);
      return ret;
    }

    void
    multinic::recv(const sst_message::ptr &msg)
    {
      throw ssterror(
          "multinic::recv: should not directly receive messages here");
    }

  }
} // end of namespace sstmac.
