/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_multinic_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_multinic_H_INCLUDED

#include <sstmac/software/services/service.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/network/interconnect/divider.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>
namespace sstmac
{
  namespace hw
  {

    /**
     * A private nested type to handle sending data.
     */

    class multinic : public networkinterface
    {

    public:

      typedef boost::intrusive_ptr<multinic> ptr;

    protected:
      /// Hi.
      multinic();

      divider::ptr div_;
      boost::unordered_map<int, networkinterface::ptr> nics_;

      virtual void
      recv(const sst_message::ptr & netmsg);

    public:
      /// Hello.
      static multinic::ptr
      construct(const sim_parameters::ptr& params)
      {
        return ptr(new multinic());
      }

      virtual void
      init(const node::ptr &parent,
          const sstmac::hw::interconnect::ptr &interconn);

      /// Goodbye.
      virtual
      ~multinic() throw ()
      {
      }

      /// Schedule a send operation to happen at the given time.
      /// \param start_time is the time when the send is started.
      /// \param ppidarg is the parallel process id of the source and target.
      /// \param recver is the global index of the receiver node.
      /// \param payload is the data sent over the network
      /// \param completion is the eventhandler signaled when the send is complete
      virtual void
      send(const sst_message::ptr &payload);

      virtual networkinterface::ptr
      clone(const node::ptr &parent,
          const sstmac::hw::interconnect::ptr &interconn,
          const sim_parameters::ptr& params) const;

      virtual void
      set_eventmanager(const eventmanager::ptr &m)
      {
        networkinterface::set_eventmanager(m);

        boost::unordered_map<int, networkinterface::ptr>::iterator it, end = nics_.end();
        for (it = nics_.begin(); it != end; it++)
          {
            it->second->set_eventmanager(m);
          }
      }

    };
  }
} // end of namespace sstmac.

#endif
