#include <sstmac/hardware/packetflow/packetflow_buffer.h>

namespace sstmac {
    namespace hw {

packetflow_simple_buffer::packetflow_simple_buffer(
    double out_bw,
    const timestamp& out_lat,
    long max_num_bytes)
    : out_bw_(out_bw),
    out_lat_(out_lat),
    max_num_bytes_(max_num_bytes)
{
    sender_ = packetflow_sender::construct(out_lat_, out_bw_, ptr(this));
}

void
packetflow_simple_buffer::handle_credit(const packetflow_credit::ptr &msg)
{
    sender_->add_credit(msg);
}

void
packetflow_simple_buffer::handle_payload(const packetflow_payload::ptr &msg)
{
    sender_->send(msg);
}

void
packetflow_simple_buffer::send_along(const packetflow_payload::ptr &msg)
{
    SSTMAC_DEBUG << sstprintf("On simple buffer %s, sending along {%s}\n",
                              descr_->to_string().c_str(),
                              msg->to_string().c_str());
    schedule(now() + out_lat_, output_, msg);
    packetflow_credit::ptr credit = packetflow_credit::construct(credit_port_, msg->num_bytes());
    schedule_delay(timestamp(msg->ser_delay()), input_, credit);
}

void
packetflow_simple_buffer::set_eventmanager(const eventmanager::ptr &m)
{
    packetflow_handler::set_eventmanager(m);
    sender_->set_eventmanager(m);
}

std::string
packetflow_simple_buffer::to_string() const {
    return std::string("simple buffer ") + descr_->to_string();
}

packetflow_crossbar_buffer::packetflow_crossbar_buffer(
    double out_bw,
    const timestamp& out_lat,
    long max_num_bytes)
    : out_bw_(out_bw),
    out_lat_(out_lat),
    max_num_bytes_(max_num_bytes)
{
}

void
packetflow_crossbar_buffer::set_eventmanager(const eventmanager::ptr &m)
{
    packetflow_handler::set_eventmanager(m);
}

void
packetflow_crossbar_buffer::handle_credit(const packetflow_credit::ptr &msg)
{
    sst_throw(illformederror,
        "packetflow_crossbar_buffer::handle_credit: crossbar buffer should never receive credits");
}

void
packetflow_crossbar_buffer::handle_payload(const packetflow_payload::ptr &msg)
{
    xbar_->arbitrate(my_xbar_port_, msg);
}

void
packetflow_crossbar_buffer::set_input(int outport, const packetflow_handler::ptr &inp)
{
    input_ = inp;
    credit_port_ = outport;
}


void
packetflow_crossbar_buffer::ack(const packetflow_payload::ptr &msg)
{
    if (input_){
        SSTMAC_DEBUG << sstprintf("On %s, acking {%s} to %s\n",
                                  to_string().c_str(),
                                  msg->to_string().c_str(),
                                  input_->to_string().c_str());
        packetflow_credit::ptr credit = packetflow_credit::construct(credit_port_, msg->num_bytes());
        schedule_delay(timestamp(msg->ser_delay()), input_, credit);
        //input_->handle_credit(credit);
    }
}

void
packetflow_crossbar_buffer::set_output(int inport, const packetflow_crossbar::ptr& xbar)
{
    xbar_ = xbar;
    my_xbar_port_ = inport;
}

std::string
packetflow_crossbar_buffer::to_string() const {
    return std::string("crossbar buffer ") + descr_->to_string();
}

packetflow_multiplexer_buffer::packetflow_multiplexer_buffer()
{
}

void
packetflow_multiplexer_buffer::handle_credit(const packetflow_credit::ptr &msg)
{
}

void
packetflow_multiplexer_buffer::handle_payload(const packetflow_payload::ptr &msg)
{
    output_->request_access(my_mpex_port_, msg);
}

void
packetflow_multiplexer_buffer::send_along(  const packetflow_payload::ptr &msg)
{
}

    }
}

