#ifndef ARBITRATOR_H
#define ARBITRATOR_H

#include <sstmac/hardware/packetflow/packetflow_handler.h>

namespace sstmac {
    namespace hw {

class packetflow_multiplexer :
    public packetflow_handler
{
    protected:
        boost::unordered_map<int, packetflow_handler::ptr> inputs_;

        packetflow_handler::ptr output_;

    public:
        typedef boost::intrusive_ptr<packetflow_multiplexer> ptr;

        virtual ~packetflow_multiplexer(){}

        static ptr
        construct(){
            return new packetflow_multiplexer;
        }

        /**
            NOT CONST.  This creates the buffer if not found.
        */
        packetflow_handler::ptr
        get_input_buffer(int);

        void
        set_output(const packetflow_handler::ptr& out){
            output_ = out;
        }

        void request_access(int inport, const packetflow_payload::ptr& msg);

        int add_input(const packetflow_handler::ptr& inp);

        virtual void
        handle_credit(const packetflow_credit::ptr &msg);

        virtual void
        handle_payload(const packetflow_payload::ptr &msg);

        virtual void
        set_eventmanager(const eventmanager::ptr &m);

        long
        initial_num_credits() const {
            return 0;
        }

};

    }
}


#endif // ARBITRATOR_H
