#ifndef PACKETFLOW_NIC_H
#define PACKETFLOW_NIC_H

#ifndef sstmac_hardware_nic_packetflow_nic_H
#define sstmac_hardware_nic_packetflow_nic_H

#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/packetflow/packetflow_endpoint.h>
#include <sstmac/hardware/packetflow/packetflow_startpoint.h>
#include <sstmac/hardware/packetflow/packetflow_switch.h>

namespace sstmac
{
  namespace hw
  {

    /**
     @class packetflow_nic
     Network interface compatible with sending packet trains
     */
    class packetflow_nic :
        public networkinterface
    {

    protected:
      packetflow_nic();

      packetflow_endpoint::ptr endpoint_;

      packetflow_startpoint::ptr startpoint_;

      virtual void
      recv(const sst_message::ptr& netmsg);

    public:

      typedef boost::intrusive_ptr<packetflow_nic> ptr;

      static packetflow_nic::ptr
      construct(const sim_parameters::ptr& params);

      /// Goodbye.
      virtual
      ~packetflow_nic() throw ();

      virtual networkinterface::ptr
      clone(const node::ptr &parent,
          const sstmac::hw::interconnect::ptr &interconn,
          const sim_parameters::ptr& params) const;

      virtual void
      send(const sst_message::ptr &payload);

      void connect(connection_type_t ty, const connectable::ptr &mod);

      std::string
      to_string() const
      {
        return "packetflow_nic";
      }

      virtual void
      init(const node::ptr &parent, const interconnectPtr& interconn);

      /**
       Set up the injection/ejection links to the switch the NIC is connected to
       @param sw The switch that injects/ejects
       */
      void
      set_inj_switch(const packetflow_switch::ptr& sw);

      void
      set_ej_switch(const packetflow_switch::ptr& sw);

      void
      validate_msg_supported(const sst_message::ptr& msg) const;

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual void
      finish();

    };

  }
} // end of namespace sstmac

#endif // packetflow_nic_H


#endif // PACKETFLOW_NIC_H
