//#include <sstmac/backends/native/dist_eventmap.h>
#include <sstmac/hardware/packettrain/trainlink.h>
#include <sstmac/common/eventcallback.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {
  //  using sstmac::native::dist_eventmap;
    int trainlink::num_links_ = 0;

    void
    trainlink::ack(const timestamp &time, const packettrain::ptr &train)
    {
      if (acker_ && train->get_needs_ack())
      {
        SSTMAC_DEBUG << sstprintf("On %s, scheduling ack %s at %8.4e for %s\n",
              to_string().c_str(), train->to_string().c_str(), time.sec(),
              train->orig()->to_string().c_str());
        schedule(time, acker_, train->orig()->clone(sst_message::SEND_ACK));
      }
    }

    void
    trainlink::handle(const sst_message::ptr &msg)
    {
      packettrain::ptr train = safe_cast(msg, packettrain);
      traincongestionpoint::send(train);
    }

    std::string
    trainlink::to_string() const
    {
      return sstprintf("link %d with bw %8.4e B/s", link_num_, bw_bytes_);
    }


    int
    trainlink::get_queue_length() const {
        return arbiter_->get_queue_length(now(), networkswitch::packet_length_bytes());
    }

    void
    trainlink::finish()
    {
      if (finished_)
        return;

      traincongestionpoint::finish();
      acker_ = 0;
      //dest_->finish();
      dest_ = 0;
    }

    void
    trainlink::do_send(const timestamp& t_start,
        const packettrain::ptr& new_train)
    {
      timestamp head_arrival_time = t_start + link_latency_;
      //if (!new_train->negligible_size()){
      SSTMAC_DEBUG << sstprintf("On %s, delivering %s to %s at %e\n",
            to_string().c_str(), new_train->to_string().c_str(),
            dest_->to_string().c_str(), head_arrival_time.sec());
      //}
      schedule(head_arrival_time, dest_, new_train);
    }

    void
    trainlink::set_eventmanager(const eventmanager::ptr &m)
    {
      traincongestionpoint::set_eventmanager(m);
    }

    trainlink::trainlink(const eventmodule::ptr& dest,
        const timestamp& link_latency, double bytes_per_cycle, double freq_hz,
        double payload_fraction, bool enable_multipath, bool ignore_congestion,
        const output_bw_arbitrator::ptr& arb, const std::string& link_type,
        const std::string& link_name) :
          traincongestionpoint(bytes_per_cycle, freq_hz, payload_fraction,
              link_type + "_link", link_name), link_num_(num_links_++),
          dest_(dest), link_latency_(link_latency)
    {
      arbiter_ = arb;
      arbiter_->set_bw(bw_bytes_);
      arbiter_->set_eventmodule(ptr(this));

      ignore_congestion_ = ignore_congestion;
      enable_multipath_ = enable_multipath;

      if (enable_multipath)
      {
        vc_alloc_ = new uncongested_vc_allocator;
      }
      else
      {
        vc_alloc_ = new inorder_vc_allocator;
      }

    }

    trainlink::ptr
    trainlink::construct(const eventmodule::ptr& dest,
        const timestamp& link_latency, double num_bytes_per_send,
        double freq_hz, double payload_fraction, bool enable_multipath,
        bool ignore_congestion, const output_bw_arbitrator::ptr& arb,
        const std::string& link_type, const std::string& link_name)
    {
      return new trainlink(dest, link_latency, num_bytes_per_send, freq_hz,
          payload_fraction, enable_multipath, ignore_congestion, arb,
          link_type, link_name);
    }

    trainlink::ptr
    trainlink::construct(const eventmodule::ptr &dest, bool enable_multipath,
        bool ignore_congestion, const timestamp &lat, double bw,
        double payload_fraction, const output_bw_arbitrator::ptr& arb,
        const std::string& link_type, const std::string& link_name)
    {
      //assume two bytes per send... just because
      double bytes_per_send = 2.0;
      double freq_hz = bw / bytes_per_send; //scale to ghz
      return new trainlink(dest, lat, bytes_per_send, freq_hz,
          payload_fraction, enable_multipath, ignore_congestion, arb,
          link_type, link_name);
    }

  }
}
