/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_PROCESSOR_migratingcomputescheduler_H_INCLUDED
#define SSTMAC_HARDWARE_PROCESSOR_migratingcomputescheduler_H_INCLUDED

#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/software/process/operatingsystem.h>

namespace sstmac
{
  namespace hw
  {

    class migratingcomputescheduler : public computescheduler
    {
    protected:
      migratingcomputescheduler(long nodeid, int ncores, int nsocket, timestamp preempt, const sw::operatingsystem::ptr& os);

      virtual int
      find_open_core(long thread_id);

      virtual void
      schedule_computation(compute_job::ptr jorb);

      virtual long
      failure_id() const
      {
        return nodeid_;
      }

      void
      start_next_suspended();

    public:
      virtual std::string
      to_string() const
      {
        return "migratingcomputescheduler";
      }
      typedef boost::intrusive_ptr<migratingcomputescheduler> ptr;

      virtual ~migratingcomputescheduler();

      static migratingcomputescheduler::ptr
      construct(long nodeid, int cores, int nsocket, timestamp pre, const sw::operatingsystem::ptr& os){
        return ptr(new migratingcomputescheduler(nodeid, cores, nsocket, pre, os));
      }

      virtual void
      schedule_computation(const timestamp& comp,
          const sst_message::ptr& to_deliver,
          const eventhandler::ptr& handler,
          long thread_id);

      virtual void
      reserve(const sst_message::ptr &msg, long thread_id);

      virtual void
      release(const sst_message::ptr &msg, long thread_id);

      virtual void
      handle(const sst_message::ptr &msg);

      static const int no_open_cores = -1;

    protected:
      std::map<sst_message::ptr, compute_job::ptr> slots_compute_;
      std::map<int, bool> compute_slots_;
      std::queue<compute_job::ptr> suspendedQ;

    };

  }
} //end of namespace sstmac
#endif
