#include <sstmac/keywords/keyword_registration.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/ptr_type.h>
#include <cstdio>

namespace sstmac {
boost::unordered_set<std::string>* KeywordRegistration::valid_keywords_ = 0;
bool KeywordRegistration::inited_ = false;
bool KeywordRegistration::do_validation_ = true;

bool
KeywordRegistration::is_valid_keyword(const std::string &name)
{
    init();
    boost::unordered_set<std::string>::const_iterator it = valid_keywords_->find(name);
    if (it == valid_keywords_->end()){
        return false;
    }
    else {
        return true;
    }
}

void
KeywordRegistration::register_keyword(const std::string &name)
{
    init();
    valid_keywords_->insert(name);
}

void
KeywordRegistration::init()
{
    if (inited_) return;

    valid_keywords_ = new boost::unordered_set<std::string>;

    inited_ = true;

    char tmp[1024];
    for (int i=1; i < 20; ++i){
        sprintf(tmp, "launch_app%d", i);
        register_keyword(tmp);
        sprintf(tmp, "launch_app%d_size", i);
        register_keyword(tmp);
        sprintf(tmp, "launch_app%d_start", i);
        register_keyword(tmp);
        sprintf(tmp, "launch_app%d_ntask_per_node", i);
        register_keyword(tmp);
        sprintf(tmp, "launch_app%d_argv", i);
        register_keyword(tmp);
        sprintf(tmp, "launch_app%d_cmd", i);
        register_keyword(tmp);
    }

    for (int i=0; i < 10; ++i){
        for (int j=0; j < 10; ++j){
            if (i==j)
                continue;

            sprintf(tmp, "numa_hypertransport_bandwidth_%d_%d", i, j);
            register_keyword(tmp);
        }
    }

}

void
KeywordRegistration::validate_keyword(const std::string &name, const std::string &val)
{
  if(do_validation_){
    bool valid = is_valid_keyword(name);
    if (!valid){
        sst_throw(valueerror, "unknown keyword name %s with value %s", name.c_str(), val.c_str());
    }
  }
}

StaticNumberedKeywordRegister::StaticNumberedKeywordRegister(int num_keywords, const char *format_str)
{
    for (int i=0; i < num_keywords; ++i){
        std::string param_name = sstprintf(format_str, i);
        KeywordRegistration::register_keyword(param_name);
    }
}

StaticKeywordRegister::StaticKeywordRegister(int num_keywords, const char *keywords[])
{
    for (int i=0; i < num_keywords; ++i){
        const char* keyword = keywords[i];
        KeywordRegistration::register_keyword(keyword);
    }
}

}
