/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SSTMAC_H_INCLUDED
#define SSTMAC_SSTMAC_H_INCLUDED

#include <sstmac/backends/native/driver.h>
#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/parameters/command_parameters.h>

#include <sys/types.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <sys/wait.h>

#define PARSE_OPT_SUCCESS 0
#define PARSE_OPT_EXIT_SUCCESS 1
#define PARSE_OPT_EXIT_FAIL 2

struct opts
{
  int help;
  std::string debug;
  std::string configfile;
  sstmac::timestamp stoptime;
  sstmac::command_parameters::ptr params;
  std::string mode;
  bool color;
  std::string dumpimetafile;
  std::string network;
  bool print_apps;
  bool print_walltime;
  bool print_params;
  int run_num;

  opts() :
    help(0), debug(""), run_num(-1), configfile(""), stoptime(-1),
        mode("normal"), color(false), dumpimetafile(""), network(""),
        print_apps(false), print_walltime(true), print_params(false)
  {
  }
};

namespace sstmac
{
  class sim_manager;
}

static inline double
get_real_time()
{
  timeval t_st;
  gettimeofday(&t_st, 0);
  double t = t_st.tv_sec + 1e-6 * t_st.tv_usec;
  return t;
}

int
parse_opts(int argc, char **argv, opts &oo);

void
print_help(int argc, char **argv);

int
try_sstmacmain(int argc, char **argv,
    boost::intrusive_ptr<sstmac::sim_manager> simmgr);

void
run_debugger(sstmac::sim_parameters::ptr params, opts oo);

#endif
