#include <sstmac/software/ami/ami.h>
#include <sstmac/common/errors.h>

namespace sstmac {
    namespace sw {


#define enumcase(x) case x: return #x

const char*
AMI::tostr(AMI_CAT cat)
{
    switch(cat)
    {
      enumcase(AMI_COMPUTE);
      enumcase(AMI_COMM);
      enumcase(AMI_DISP);
      enumcase(AMI_QUERY);
      enumcase(AMI_HW);
    }
    sst_throw(illformederror, "Invalid AMI_CAT %d received", cat);
}

const char*
AMI::tostr(AMI_COMM_FUNC func)
{
    switch(func)
    {
        enumcase(AMI_COMM_SEND);
        enumcase(AMI_COMM_BCAST);
        enumcase(AMI_COMM_BARRIER);
        enumcase(AMI_COMM_NIC_SEND_ACK);
    }
    sst_throw(illformederror, "Invalid AMI_COMM_FUNC %d received", func);
}

const char*
AMI::tostr(AMI_DISP_FUNC func)
{
    switch(func)
    {
        enumcase(AMI_DISP_BORDER);
        enumcase(AMI_DISP_FILL);
    }
    sst_throw(illformederror, "Invalid AMI_DISP_FUNC %d received", func);
}

const char*
AMI::tostr(AMI_HW_FUNC func)
{
    switch(func)
    {
        enumcase(AMI_CATASTROPHIC_INTERRUPT);
    }
    sst_throw(illformederror, "Invalid AMI_HW_FUNC %d received", func);
}

const char*
AMI::tostr(AMI_QUERY_FUNC func)
{
    switch(func)
    {
        enumcase(AMI_QUERY_ADDRESS);
    }
    sst_throw(illformederror, "Invalid AMI_QUERY_FUNC %d received", func);
}

const char*
AMI::tostr(AMI_COMP_FUNC func)
{
    switch(func)
    {
        enumcase(AMI_COMP_TIME);
        enumcase(AMI_COMP_SCHEDULE);
        enumcase(AMI_COMP_RELEASE);
        enumcase(AMI_COMP_INSTR);
        enumcase(AMI_COMP_INSTR_TIME);
        enumcase(AMI_COMP_MEMREAD);
        enumcase(AMI_COMP_MEMWRITE);
        enumcase(AMI_COMP_DISKACCESS);
        enumcase(AMI_COMP_STOCH);
        enumcase(AMI_COMP_SLEEP);
        enumcase(AMI_COMP_EIGER);
        enumcase(AMI_COMP_REGISTER_THREAD);
        enumcase(AMI_GPU_MEMCPY);
        enumcase(AMI_GPU_EX_ASYNC);
    }
    sst_throw(illformederror, "Invalid AMI_COMP_FUNC %d received", func);
    return 0;
}

void
AMI::execute_kernel(AMI::AMI_COMP_FUNC func, const boost::intrusive_ptr<sst_message>& data)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s with msg", tostr(func));
}

void
AMI::execute_kernel(AMI::AMI_COMM_FUNC func, const boost::intrusive_ptr<sst_message>& data)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s with msg", tostr(func));
}

void
AMI::execute_kernel(AMI::AMI_QUERY_FUNC func, const boost::intrusive_ptr<sst_message>& data)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s with msg", tostr(func));
}

void
AMI::execute_kernel(AMI::AMI_DISP_FUNC func, const boost::intrusive_ptr<sst_message>& data)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s with msg", tostr(func));
}

void
AMI::execute_kernel(AMI::AMI_HW_FUNC func, const boost::intrusive_ptr<sst_message>& data)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s with msg", tostr(func));
}

ptr_type::ptr
AMI::execute_kernel(AMI::AMI_COMP_FUNC func)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s without msg", tostr(func));
}

ptr_type::ptr
AMI::execute_kernel(AMI::AMI_COMM_FUNC func)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s without msg", tostr(func));
}

ptr_type::ptr
AMI::execute_kernel(AMI::AMI_QUERY_FUNC func)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s without msg", tostr(func));
}

ptr_type::ptr
AMI::execute_kernel(AMI::AMI_DISP_FUNC func)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s without msg", tostr(func));
}

ptr_type::ptr
AMI::execute_kernel(AMI::AMI_HW_FUNC func)
{
    sst_throw(illformederror, "ami::execute_kernel: requested AMI does not support kernel %s without msg", tostr(func));
}

bool
AMI::kernel_supported(AMI::AMI_COMM_FUNC func) const
{
    return false;
}

bool
AMI::kernel_supported(AMI::AMI_COMP_FUNC func) const
{
    return false;
}

bool
AMI::kernel_supported(AMI::AMI_QUERY_FUNC func) const
{
    return false;
}

bool
AMI::kernel_supported(AMI::AMI_DISP_FUNC func) const
{
    return false;
}

bool
AMI::kernel_supported(AMI::AMI_HW_FUNC func) const
{
    return false;
}

    }
}
