/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include "armci.h"

#include <sstmac/software/api/armci/armci.h>
#include <sstmac/software/api/openshmem/shmem.h>

#include <sstmac/software/process/os_api.h>
#include <sstmac/software/api/mpi/mpiapp.h>

namespace sstmac
{
  namespace sw
  {

  }
}

using namespace sstmac;
using namespace sstmac::sw;

extern "C" int
ARMCI_Init()
{
    return ARMCI_SUCCESS;
}

extern "C" int
ARMCI_Finalize()
{
    return ARMCI_SUCCESS;
}

extern "C" void
ARMCI_Cleanup()
{

}

extern "C" void
ARMCI_Error(char *message, int code)
{

}

extern "C" int
ARMCI_PutV(armci_giov_t *dsrc_arr, int arr_len, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutV - not implemented yet");
}

extern "C" int
ARMCI_NbPutV(armci_giov_t *dsrc_arr, int arr_len, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutV - not implemented yet");
}

extern "C" int
ARMCI_GetV(armci_giov_t *dsrc_arr, int arr_len, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetV - not implemented yet");
}

extern "C" int
ARMCI_NbGetV(armci_giov_t *dsrc_arr, int arr_len, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbGetV - not implemented yet");
}

extern "C" int
ARMCI_PutS(void* src_ptr, int src_stride_ar[], void* dst_ptr,
    int dst_stride_ar[], int count[], int stride_levels, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutS - not implemented yet");
}

extern "C" int
ARMCI_NbPutS(void* src_ptr, int src_stride_ar[], void* dst_ptr,
    int dst_stride_ar[], int count[], int stride_levels, int proc,
    armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutS - not implemented yet");
}

extern "C" int
ARMCI_GetS(void *src_ptr, int src_stride_ar[], void* dst_ptr,
    int dst_stride_ar[], int count[], int stride_levels, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetS - not implemented yet");
}

extern "C" int
ARMCI_NbGetS(void *src_ptr, int src_stride_ar[], void* dst_ptr,
    int dst_stride_ar[], int count[], int stride_levels, int proc,
    armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbGetS - not implemented yet");
}

extern "C" int
ARMCI_Put(void* src, void* dst, int bytes, int proc)
{
  shmem_char_put((char*) dst, (char*) src, bytes, proc);

  return ARMCI_SUCCESS;
}

extern "C" int
ARMCI_NbPut(void* src, void* dst, int bytes, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPut - not implemented yet");
}

extern "C" int
ARMCI_Get(void* src, void* dst, int bytes, int proc)
{
  shmem_char_get((char*) dst, (char*) src, bytes, proc);

  return ARMCI_SUCCESS;
}

extern "C" int
ARMCI_NbGet(void* src, void* dst, int bytes, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbGet - not implemented yet");
}

extern "C" int
ARMCI_AccV(int datatype, void *scale, armci_giov_t *dsrc_arr, int arr_len,
    int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_AccV - not implemented yet");
}

extern "C" int
ARMCI_NbAccV(int datatype, void *scale, armci_giov_t *dsrc_arr, int arr_len,
    int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbAccV - not implemented yet");
}

extern "C" int
ARMCI_AccS(int datatype, void *scale, void* src_ptr, int src_stride_ar[],
    void* dst_ptr, int dst_stride_ar[], int count[], int stride_levels,
    int proc)
{
  sst_throw(unimplementederror, "ARMCI_AccS - not implemented yet");
}

extern "C" int
ARMCI_NbAccS(int datatype, void *scale, void* src_ptr, int src_stride_ar[],
    void* dst_ptr, int dst_stride_ar[], int count[], int stride_levels,
    int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbAccS - not implemented yet");
}

extern "C" int
ARMCI_Acc(int datatype, void *scale, void* src, void* dst, int bytes, int proc)
{
  sst_throw(unimplementederror, "ARMCI_Acc - not implemented yet");
}

extern "C" int
ARMCI_NbAcc(int datatype, void *scale, void* src, void* dst, int bytes,
    int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbAcc - not implemented yet");
}

extern "C" int
ARMCI_PutValueLong(long src, void* dst, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutValueLong - not implemented yet");
}

extern "C" int
ARMCI_NbPutValueLong(long src, void* dst, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutValueLong - not implemented yet");
}

extern "C" long
ARMCI_GetValueLong(void *src, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetValueLong - not implemented yet");
}

extern "C" int
ARMCI_PutValueInt(int src, void* dst, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutValueInt - not implemented yet");
}

extern "C" int
ARMCI_NbPutValueInt(int src, void* dst, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutValueInt - not implemented yet");
}

extern "C" int
ARMCI_GetValueInt(void *src, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetValueInt - not implemented yet");
}

extern "C" int
ARMCI_PutValueDouble(double src, void* dst, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutValueDouble - not implemented yet");
}

extern "C" int
ARMCI_NbPutValueDouble(double src, void* dst, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutValueDouble - not implemented yet");
}

extern "C" double
ARMCI_GetValueDouble(void *src, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetValueDouble - not implemented yet");
}

extern "C" int
ARMCI_PutValueFloat(float src, void* dst, int proc)
{
  sst_throw(unimplementederror, "ARMCI_PutValueFloat - not implemented yet");
}

extern "C" int
ARMCI_NbPutValueFloat(float src, void* dst, int proc, armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_NbPutValueFloat - not implemented yet");
}

extern "C" float
ARMCI_GetValueFloat(void *src, int proc)
{
  sst_throw(unimplementederror, "ARMCI_GetValueFloat - not implemented yet");
}

extern "C" void
ARMCI_Fence(int proc)
{
  sst_throw(unimplementederror, "ARMCI_Fence - not implemented yet");
}

extern "C" void
ARMCI_AllFence()
{
  sst_throw(unimplementederror, "ARMCI_AllFence - not implemented yet");
}

extern "C" int
ARMCI_Wait(armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_Wait - not implemented yet");
}

extern "C" int
ARMCI_Test(armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_Test - not implemented yet");
}

extern "C" int
ARMCI_WaitProc(int proc)
{
  sst_throw(unimplementederror, "ARMCI_WaitProc - not implemented yet");
}

extern "C" int
ARMCI_WaitAll()
{
  sst_throw(unimplementederror, "ARMCI_WaitAll - not implemented yet");
}

extern "C" int
ARMCI_Barrier()
{
  sst_throw(unimplementederror, "ARMCI_Barrier - not implemented yet");
}

extern "C" void
ARMCI_SET_AGGREGATE_HANDLE(armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_SET_AGGREGATE_HANDLE - not implemented yet");
}

extern "C" void
ARMCI_UNSET_AGGREGATE_HANDLE(armci_hdl_t* handle)
{
  sst_throw(unimplementederror, "ARMCI_UNSET_AGGREGATE_HANDLE - not implemented yet");
}

extern "C" int
ARMCI_Rmw(int op, void *ploc, void *prem, int value, int proc)
{
  sst_throw(unimplementederror, "ARMCI_Rmv - not implemented yet");
}

extern "C" int
ARMCI_Create_mutexes(int count)
{
  sst_throw(unimplementederror, "ARMCI_Create_mutexes - not implemented yet");
}

extern "C" int
ARMCI_Destroy_mutexes(void)
{
  sst_throw(unimplementederror, "ARMCI_Destroy_mutexes - not implemented yet");
}

extern "C" void
ARMCI_Lock(int mutex, int proc)
{
  sst_throw(unimplementederror, "ARMCI_Lock - not implemented yet");
}

extern "C" void
ARMCI_Unlock(int mutex, int proc)
{
  sst_throw(unimplementederror, "ARMCI_Unlock - not implemented yet");
}

extern "C" int
ARMCI_Malloc(void* ptr[], armci_size_t bytes)
{
  sst_throw(unimplementederror, "ARMCI_Malloc - not implemented yet");
}

extern "C" void*
ARMCI_Malloc_local(armci_size_t bytes)
{
  return malloc(bytes);
}

extern "C" int
ARMCI_Free(void *address)
{
  shfree(address);
  return ARMCI_SUCCESS;
}

extern "C" int
ARMCI_Free_local(void *address)
{
  free(address);
  return ARMCI_SUCCESS;
}

extern "C" void
armci_msg_brdcst(void* buffer, int len, int root)
{
  sst_throw(unimplementederror, "armci_msg_brdcst - not implemented yet");
}

extern "C" void
armci_msg_lgop(void *x, int n, int type, char* op)
{
  sst_throw(unimplementederror, "armci_msg_lgop - not implemented yet");
}

extern "C" void
armci_msg_igop(void *x, int n, int type, char* op)
{
  sst_throw(unimplementederror, "armci_msg_igop - not implemented yet");
}

extern "C" void
armci_msg_dgop(void *x, int n, int type, char* op)
{
  sst_throw(unimplementederror, "armci_msg_dgop - not implemented yet");
}

extern "C" void
armci_msg_fgop(void *x, int n, int type, char* op)
{
  sst_throw(unimplementederror, "armci_msg_fgop - not implemented yet");
}

extern "C" void
armci_msg_barrier(void)
{
  sst_throw(unimplementederror, "armci_msg_barrier - not implemented yet");
}

extern "C" void
armci_msg_reduce(void *x, int n, char* op, int type, int root)
{
  sst_throw(unimplementederror, "armci_msg_reduce - not implemented yet");
}

extern "C" int
armci_domain_nprocs(armci_domain_t domain, int id)
{
  sst_throw(unimplementederror, "armci_domain_nprocs - not implemented yet");
}

extern "C" int
armci_domain_count(armci_domain_t domain)
{
  sst_throw(unimplementederror, "armci_domain_count - not implemented yet");
}

extern "C" int
armci_domain_id(armci_domain_t domain, int glob_proc_id)
{
  sst_throw(unimplementederror, "armc_domain_id - not implemented yet");
}

extern "C" int
armci_domain_glob_proc_id(armci_domain_t domain, int id, int loc_proc_id)
{
  sst_throw(unimplementederror, "armci_domain_glob_proc_id - not implemented yet");
}

