/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/hpx/agas/agas_ideal.h>

namespace hpx
{

  boost::unordered_map<unsigned long long, lva> agas_ideal::lvamap_;
  unsigned long long agas_ideal::gid_count_ = 0;

  void
  agas_ideal::handle(const sstmac::sst_message::ptr &msg)
  {
    //once component migration is enabled, need to be able to  handle it here?
  }

  bool
  agas_ideal::new_gid(lva::ENTITY_TYPE t, globalid &gid,
      unsigned long long &waitid)
  {
    globalid ret(gid_count_++);
    lva l(ret, locality_, t);
    lvamap_[ret.gid_] = l;
    gid = ret;
    return true;
  }

  bool
  agas_ideal::resolve_locality(const globalid &gid,
      sstmac::nodeaddress::ptr &loc, unsigned long long &waitid)
  {
    if (lvamap_.find(gid.gid_) != lvamap_.end())
      {
        loc = lvamap_[gid.gid_].addr();
        return true;
      }
    else
      {
        sst_throw(sstmac::ssterror, "agas_ideal::resolve_locality - gid %lu not found", gid.gid_);
      }
  }

  void
  agas_ideal::gid_move_complete(globalid gid)
  {
    lva l = lvamap_[gid.gid_];
    lva newl(gid.gid_, locality_, l.etype());
    lvamap_[gid.gid_] = newl;
  }

  void
  agas_ideal::gid_moving(globalid gid, sstmac::nodeaddress::ptr to)
  {

  }

}
