/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/hpx/agas/agas_server.h>
#include <sstmac/software/api/hpx/runtime/hpx_thread_manager.h>
namespace hpx
{

  sstmac::debug agas_server::dbg_;

  static sstmac::sw::key::category agas_key_cat("HPX::agas");

  agas_server::agas_server()
  {

    if (!dbg_.is_real())
      {
        dbg_ = sstmac::debug("<debug> hpx | hpx_agas_server",
            sstmac::logger::blue);
        sstmac::logger::register_logger(&dbg_);
      }

  }

  globalid
  agas_server::new_gid(lva::ENTITY_TYPE t)
  {
    unsigned long long waitid;
    globalid ret;
    bool found = new_gid(t, ret, waitid);

    if (!found)
      {
        sstmac::sw::key::ptr key = sstmac::sw::key::construct(agas_key_cat);
        newgid_waiting_[waitid] = key;
        os_->block(key);

        ret = newgid_value_[waitid];
        newgid_waiting_.erase(waitid);
        newgid_value_.erase(waitid);
      }

    return ret;
  }

  agas_server::~agas_server()
  {
  }

  void
  agas_server::set_threadman(boost::intrusive_ptr<hpx_thread_manager> a)
  {
    threadman_ = a;
  }

}
