/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_SINGLE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_AGAS_AGAS_SINGLE_H_INCLUDED

#include <sstmac/software/api/hpx/agas/agas_server.h>
#include <sstmac/software/api/hpx/agas/globalid.h>
#include <sstmac/software/api/hpx/agas/lva.h>
#include <sstmac/software/api/hpx/parcels/parcel.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>
#include <map>
#include <queue>
#include <sstmac/common/sstmac_env.h>
namespace hpx
{

  namespace parcels
  {
    class update_locality : public hpx::parcels::parcel
    {
    protected:

      update_locality(globalid gid, sstmac::nodeaddress::ptr up) :
        parcel(UPDATE_LOCALITY), gid_(gid), up_(up)
      {

      }

      globalid gid_;
      sstmac::nodeaddress::ptr up_;

    public:
      typedef boost::intrusive_ptr<update_locality> ptr;

      static ptr
      construct(globalid gid, sstmac::nodeaddress::ptr up,
          sstmac::timestamp now)
      {
        if (stats_[UPDATE_LOCALITY] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[UPDATE_LOCALITY] = sstmac::stat_time_window_sum::construct(
                "<stats> hpx_parcels | hpx", "hpx_parcels_update_locality.csv");
          }
        stats_[UPDATE_LOCALITY]->record(now, 1);
        return ptr(new update_locality(gid, up));
      }

      virtual
      ~update_locality()
      {

      }

      virtual long
      get_byte_length() const
      {
        // which + trans_id + gid + params
        return 12;
      }

      globalid
      gid() const
      {
        return gid_;
      }

      sstmac::nodeaddress::ptr
      upaddr()
      {
        return up_;
      }

      virtual std::string
      to_string() const
      {
        return "hpx::parcels::update_locality";
      }
    };
  }

  class agas_single : public agas_server
  {

#define MAX_OPS 1
#define OP_SIZE 64

  protected:
    unsigned long long gid_count_;
    sstmac::nodeaddress::ptr locality_;

    long WAITING_ID;

    boost::unordered_map<unsigned long long, lva> lvamap_;

    boost::unordered_map<unsigned long long, sstmac::nodeaddress::ptr> cache_;
    //boost::unordered_map<long, sstmac::sw::key::ptr> waiting_;

    //hpx::parcels::parcel::ptr rx_;

    //std::queue<hpx::parcels::parcel::ptr> pending_;

    // sstmac::sw::lib_compute_memmove::ptr lib_mem_;

    bool server_here_;
    sstmac::nodeaddress::ptr server_node_;

  public:
    typedef boost::intrusive_ptr<agas_single> ptr;

    static ptr
    construct()
    {
      return ptr(new agas_single());
    }

    virtual
    ~agas_single()
    {
    }

    virtual void
    handle(const sstmac::sst_message::ptr &msg);

    virtual service::ptr
    clone()
    {
      return construct();
    }

    virtual void
    start()
    {
      service::start();

      //lib_mem_ = sstmac::sw::lib_compute_memmove::construct(
      //     "agas-server-lib-mem");
      // lib_mem_->init_handler(ptr(this));
      // lib_mem_->init_waiter(wait_);
      // register_lib(lib_mem_);

      locality_ = os_->get_my_addr();
      gid_count_ = 0;

      long s = sstmac::sstmac_env::params->get_optional_long_param(
          "hpx_agas_server_node", 0);
      if (s == locality_->unique_id())
        {
          server_here_ = true;
        }
      else
        {
          server_here_ = false;
        }
      server_node_ = sstmac::nodeaddress::convert_from_uniqueid(s);
    }

    static std::string
    lib_name()
    {
      return "hpx_agas_single";
    }

    // ------ these are the useful functions ------ //

    virtual bool
    new_gid(lva::ENTITY_TYPE t, globalid &gid, unsigned long long &waitid);

    virtual bool
    new_gid(lva::ENTITY_TYPE t, globalid &gid, unsigned long long &waitid,
        sstmac::nodeaddress::ptr loc);

    virtual bool
    resolve_locality(const globalid &gid, sstmac::nodeaddress::ptr &loc,
        unsigned long long &waitid);

    void
    gid_move_complete(globalid gid);

    void
    gid_moving(globalid gid, sstmac::nodeaddress::ptr to);

    // -------------------------------------- //

  protected:
    agas_single() :
      WAITING_ID(0)
    {

      libname_ = lib_name();
    }

    // void
    //  service_one();

  };

}

#endif
