/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_DISTRIBUTING_FACTORY_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_DISTRIBUTING_FACTORY_H_INCLUDED

#include <sstmac/software/api/hpx/components/hpx_component.h>
//#include <sstmac/software/api/hpx/parcels/create_component.h>
#include <sstmac/software/api/hpx/components/hpx_stubs.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action_params.h>
#include <sstmac/software/api/hpx/lcos/eager_future.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action.h>
#include <sstmac/software/api/hpx/runtime/hpx_thread_manager.h>
#include <sstmac/common/messages/valuepayload.h>

namespace hpx
{
  namespace components
  {

    unsigned long long
    create_component(std::string name);

    class hpx_distributing_factory : public hpx_component
    {

    protected:
      hpx_distributing_factory(globalid gid) :
        hpx_component(gid)
      {
      }

    public:
      typedef boost::intrusive_ptr<hpx_distributing_factory> ptr;

      static ptr
      construct(globalid gid)
      {
        return ptr(new hpx_distributing_factory(gid));
      }

      virtual hpx_component::ptr
      clone(globalid gid)
      {
        return construct(gid);
      }

      virtual
      ~hpx_distributing_factory()
      {
      }

      virtual bool
      movable() const
      {
        return false;
      }

      static std::string
      component_name()
      {
        return "hpx::components::hpx_distributing_factory";
      }

      static std::string
      create_action_name()
      {
        return component_name() + "::create";
      }

      virtual void
      register_actions(boost::intrusive_ptr<hpx_thread_manager> threadman);

      typedef hpx::lcos::eager_future<unsigned long long> ef_type;

      ef_type::ptr
      get_ef(sstmac::nodeaddress::ptr addr, hpx_action_params::ptr params);

      template<typename StubType>
        std::vector<boost::intrusive_ptr<StubType> >
        create_components(std::string which, long num)
        {

          std::vector<ef_type::ptr> efs;

          sstmac::sw::environment::const_ptr env = hpx_app::get_env();
          size_t appcnt = env->get_app_count();
          size_t tasknum = 0;
          for (long i = 0; i < num; i++)
            {
              hpx_action_params1<std::string>::ptr params = hpx_action_params1<
                  std::string>::construct(which);
              sstmac::sw::taskid tid(tasknum);
              ef_type::ptr ef = get_ef(env->get_node_address(tid), params);
              efs.push_back(ef);
              tasknum = (tasknum + 1) % appcnt;
            }

          std::vector<boost::intrusive_ptr<StubType> > ret;
          for (long i = 0; i < num; i++)
            {
              unsigned long long id = efs[i]->get();
              SSTMAC_DEBUG
                    << "hpx_distribution_factory: got a response back from component "
                    << i << ": " << id << "\n";
              globalid gid(id);
              boost::intrusive_ptr<StubType> stub = StubType::construct(gid);
              ret.push_back(stub);
            }
          return ret;
        }
    };

  } /* namespace hpx */
}
#endif /* HPX_DISTRIBUTING_FACTORY_H_ */
