/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_DISTRIBUTING_FACTORY_BLOCK_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_DISTRIBUTING_FACTORY_BLOCK_H_INCLUDED

#include <sstmac/software/api/hpx/components/hpx_distributing_factory.h>

namespace hpx
{
  namespace components
  {

    class hpx_distributing_factory_block : public hpx_distributing_factory
    {

    protected:
      hpx_distributing_factory_block(globalid gid) :
        hpx_distributing_factory(gid)
      {
      }

    public:
      typedef boost::intrusive_ptr<hpx_distributing_factory_block> ptr;

      static ptr
      construct(globalid gid)
      {
        return ptr(new hpx_distributing_factory_block(gid));
      }

      virtual hpx_component::ptr
      clone(globalid gid)
      {
        return construct(gid);
      }

      virtual
      ~hpx_distributing_factory_block()
      {
      }

      static std::string
      component_name()
      {
        return "hpx::components::hpx_distributing_factory_block";
      }

      static std::string
      create_action_name()
      {
        return component_name() + "::create";
      }

      template<typename StubType>
        std::vector<boost::intrusive_ptr<StubType> >
        create_components(std::string which, long num)
        {

          std::vector<ef_type::ptr> efs;

          sstmac::sw::environment::const_ptr env = hpx_app::get_env();
          size_t appcnt = env->get_app_count();
          size_t tasknum = 0;

          size_t per = num / appcnt;
          size_t extra = num - (per * appcnt);
          size_t allocated = 0;
          size_t thres = per;
          for (long i = 0; i < num; i++)
          {
            if (tasknum < extra)
              thres = per + 1;
            else
              thres = per;
            hpx_action_params1<std::string>::ptr params = hpx_action_params1<
                std::string>::construct(which);
            sstmac::sw::taskid tid(tasknum);
            ef_type::ptr ef = get_ef(env->get_node_address(tid), params);
            efs.push_back(ef);
            allocated++;
            if (allocated == thres)
            {
              allocated = 0;
              tasknum = (tasknum + 1) % appcnt;
            }

          }

          std::vector<boost::intrusive_ptr<StubType> > ret;
          for (long i = 0; i < num; i++)
          {
            unsigned long long id = efs[i]->get();
            SSTMAC_DEBUG
                  << "hpx_distribution_factory: got a response back from component "
                  << i << ": " << id << "\n";
            globalid gid(id);
            boost::intrusive_ptr<StubType> stub = StubType::construct(gid);
            ret.push_back(stub);
          }
          return ret;
        }
    };

  } /* namespace hpx */
}
#endif /* HPX_DISTRIBUTING_FACTORY_H_ */
