/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_PROMISE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_PROMISE_H_INCLUDED

#include <sstmac/software/api/hpx/lcos/base_lco_result.h>
#include <sstmac/software/api/hpx/hpx_app.h>
#include <sstmac/common/messages/valuepayload.h>
#include <sstmac/software/api/hpx/parcels/payloadization.h>
namespace hpx
{
  namespace lcos
  {

    /**
     * A promise is a general kind of lco that allows us to start something, either locally
     * or remotely, and be able to wait on the result at our convenience
     */
      class promise_key
      {
        public:
            static sstmac::sw::key::category category;
      };

    template<typename Result>
      class promise : public base_lco_result
      {

      protected:
        sstmac::payload::const_ptr res_;
        bool done_;
        sstmac::sw::key::ptr blocker_;
        boost::intrusive_ptr<hpx::parcels::depayloadizer<Result> > depay_;

        promise(boost::intrusive_ptr<hpx::parcels::depayloadizer<Result> > de) :
            done_(false), depay_(de)
        {
          blocker_ = sstmac::sw::key::construct(promise_key::category);
        }

      public:
        typedef boost::intrusive_ptr<promise<Result> > ptr;

        virtual
        ~promise()
        {
        }

        virtual void
        set_value(sstmac::payload::const_ptr res, sstmac::sw::OS_API::ptr os)
        {
          res_ = res;

          done_ = true;

          SSTMAC_DEBUG <<
            "promise::set_value() - about to unblock with blocker " << "\n";

          hpx_app::hpx_unblock(blocker_);

        }

        virtual sstmac::payload::const_ptr
        get_value()
        {
          if (done_)
            {
              SSTMAC_DEBUG <<
                "promise::get() -- Action has already finished \n";
              return res_;
            }
          else
            {
              SSTMAC_DEBUG
                    <<
                    "promise::get() -- Action has not finished yet, blocking with blocker "
                    << "\n";

              hpx_app::hpx_block(blocker_);
            //  hpx_app::hpx_thread_swap();
              if (!done_)
                {
                  throw sstmac::ssterror(
                      "hpx::lcos::eager_future::get() - not done after coming out of block");
                }
              SSTMAC_DEBUG
                    <<
                    "promise::get() -- Action has just finished, returning result \n";
              return res_;
            }
        }

        Result
        get()
        {
          SSTMAC_DEBUG << "hpx::lcos::promise::get() \n";
          sstmac::payload::const_ptr val = get_value();

          return depay_->depayloadize(val);
        }

      };

  }
}

#endif
