/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_NEW_GID_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_NEW_GID_H_INCLUDED

#include <sstmac/software/api/hpx/parcels/parcel.h>
#include <sstmac/software/api/hpx/hpx_app.h>

namespace hpx
{

  namespace parcels
  {

    class new_gid : public parcel
    {

    public:
      typedef boost::intrusive_ptr<new_gid> ptr;

      static ptr
      construct(lva::ENTITY_TYPE t, sstmac::nodeaddress::ptr ori, long wait, sstmac::timestamp now)
      {
        if (stats_[NEW_GID] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[NEW_GID] = sstmac::stat_time_window_sum::construct(
                "<stats> hpx_parcels | hpx", "hpx_parcels_new_gid.csv");
          }
        stats_[NEW_GID]->record(now, 1);

        return ptr(new new_gid(t, ori, wait));
      }

      lva::ENTITY_TYPE
      new_gid_type() const {
        return new_gid_type_;
      }

      sstmac::nodeaddress::ptr
      origin() const
      {
        return origin_;
      }

      long
      wait_id() const
      {
        return waiting_;
      }

      virtual long
      get_byte_length() const
      {
        return 16 + 8 + 8;
      }

    protected:
      new_gid(lva::ENTITY_TYPE t,  sstmac::nodeaddress::ptr ori, long waitid) :
          sst_message(HPX),
          parcel(NEW_GID), origin_(ori), waiting_(waitid), new_gid_type_(t)
      {

      }


      sstmac::nodeaddress::ptr origin_;
      long waiting_;
      lva::ENTITY_TYPE new_gid_type_;
    };

  }
}
#endif
