/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_PARCEL_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_PARCEL_H_INCLUDED

#include <sstmac/hardware/network/network_message.h>
#include <sstmac/common/stats/stat_time_window.h>

namespace hpx
{
  namespace parcels
  {

    class parcel :
        public sstmac::library_message,
        public sstmac::hw::network_message
    {

    public:

      typedef boost::intrusive_ptr<parcel> ptr;

      enum PARCEL_TYPE
      {
        REMOTE_ACTION,
        REMOTE_ACTION_RESULT,
        RESOLVE_GID,
        RESOLVE_GID_RESPONSE,
        NEW_GID,
        NEW_GID_RESPONSE,
        MOVE_COMPONENT,
        KILL,
        UPDATE_LOCALITY
      };

      static ptr
      construct(PARCEL_TYPE t)
      {
        return ptr(new parcel(t));
      }

      PARCEL_TYPE
      type() const
      {
        return type_;
      }

      virtual
      ~parcel()
      {
      }

      virtual bool
      get_needs_ack() const
      {
        return false;
      }

      virtual const sstmac::sw::taskid&
      source_task() const
      {
        return dummytask_;
      }

      virtual const sstmac::sw::taskid&
      dest_task() const
      {
        return dummytask_;
      }

    protected:
      parcel(PARCEL_TYPE t) :
        sst_message(sst_message::HPX), library_message(""), type_(t) //overwritten later by set_lib_name
      {
        dummytask_ = sstmac::sw::taskid(sstmac::sw::taskid::unbound_task);
        needs_ack_ = false;
      }

      PARCEL_TYPE type_;

      static std::map<PARCEL_TYPE, sstmac::stat_time_window_sum::ptr> stats_;
      sstmac::sw::taskid dummytask_;
    };

    class kill_parcel : public parcel
    {

    public:
      typedef boost::intrusive_ptr<kill_parcel> ptr;

      static ptr
      construct()
      {
        return ptr(new kill_parcel());
      }

      virtual long
      get_byte_length() const
      {
        // which + trans_id + gid + params
        return 4;
      }

      virtual
      ~kill_parcel()
      {
      }

      virtual std::string
      to_string() const
      {
        return "hpx::parcels::kill_parcel";
      }

    protected:
      kill_parcel() :
        sst_message(HPX), parcel(KILL)
      {

      }

    };

  }
}
#endif
