/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_RESOLVE_GID_RESPONSE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_RESOLVE_GID_RESPONSE_H_INCLUDED

#include <sstmac/software/api/hpx/parcels/parcel.h>
#include <sstmac/software/api/hpx/hpx_app.h>
namespace hpx
{

  namespace parcels
  {

    class resolve_gid_response : public parcel
    {

    public:
      typedef boost::intrusive_ptr<resolve_gid_response> ptr;

      static ptr
      construct(globalid gid, sstmac::nodeaddress::ptr ori, long wait,
          sstmac::timestamp now)
      {
        if (stats_[RESOLVE_GID_RESPONSE] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[RESOLVE_GID_RESPONSE] =
                sstmac::stat_time_window_sum::construct(
                    "<stats> hpx_parcels | hpx",
                    "hpx_parcels_resolve_gid_response.csv");
          }
        stats_[RESOLVE_GID_RESPONSE]->record(now, 1);
        return ptr(new resolve_gid_response(gid, ori, wait));
      }

      globalid
      gid() const
      {
        return gid_;
      }

      sstmac::nodeaddress::ptr
      where() const
      {
        return where_;
      }

      long
      wait_id() const
      {
        return waiting_;
      }

      virtual long
      get_byte_length() const
      {
        return 8 + 8 + 16;
      }

    protected:
      resolve_gid_response(globalid gid, sstmac::nodeaddress::ptr ori,
          long wait) :
          sst_message(HPX),
          parcel(RESOLVE_GID_RESPONSE),
          gid_(gid), where_(ori), waiting_(wait)
      {

      }

      globalid gid_;
      sstmac::nodeaddress::ptr where_;
      long waiting_;
    };

  }
}
#endif
