/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_ACTIONS_HPX_ACTION_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_ACTIONS_HPX_ACTION_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action_params.h>
#include <sstmac/software/api/hpx/components/hpx_component.h>
#include <sstmac/software/api/hpx/runtime/actions/hpx_action_params.h>
#include <sstmac/common/messages/valuepayload.h>
#include <sstmac/common/messages/vectorpayload.h>
#include <sstmac/software/api/hpx/parcels/payloadization.h>
namespace hpx
{
  namespace actions
  {

#define HPX_SUCCESS 0

    long
    get_next_act_lib();

    class hpx_action : public sstmac::ptr_type
    {
    public:
      typedef boost::intrusive_ptr<hpx_action> ptr;

      virtual std::string
      to_string() const
      {
        return "hpx_action";
      }

      virtual
      ~hpx_action()
      {
      }

      virtual sstmac::payload::const_ptr
      act(hpx_action_params::ptr params)
      {
        throw sstmac::unimplementederror(
            "your action does not implement act(params)");
      }

      virtual sstmac::payload::const_ptr
      act(hpx::components::hpx_component::ptr comp,
          hpx_action_params::ptr params)
      {
        throw sstmac::unimplementederror(
            "your action does not implement act(comp, params)");
      }

      static sstmac::debug&
      dbg();

    protected:

      hpx_action()
      {

        if (!dbg_.is_real())
          {
            dbg_ = sstmac::debug("<debug> hpx | hpx_actions",
                sstmac::logger::blue);
            sstmac::logger::register_logger(&dbg_);
          }
      }

      static sstmac::debug dbg_;

    };

    template<typename ResultType, typename Arg1, ResultType
    (*F)(Arg1 a), sstmac::payload::const_ptr
    (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class plain_result_action1 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<plain_result_action1> ptr;

        static ptr
        construct()
        {
          return ptr(new plain_result_action1<ResultType, Arg1, F, P> ());
        }

        virtual
        ~plain_result_action1()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "plain_result_action1: acting \n";
          typename hpx::hpx_action_params1<Arg1>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params1<Arg1> >(
                  params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          Arg1 arg0 = pcast->get_arg0();
          ResultType res = (*F)(arg0);
          return P(res);
        }

      };

    template<typename ResultType, typename Arg1, typename Arg2, ResultType
    (*F)(Arg1 a, Arg2 b), sstmac::payload::const_ptr
    (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class plain_result_action2 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<plain_result_action2> ptr;

        static ptr
        construct()
        {
          return ptr(new plain_result_action2<ResultType, Arg1, Arg2, F, P> ());
        }

        virtual
        ~plain_result_action2()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "plain_result_action2: acting \n";
          typename hpx::hpx_action_params2<Arg1, Arg2>::ptr
              pcast = boost::dynamic_pointer_cast<hpx::hpx_action_params2<Arg1,
                  Arg2> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          ResultType res = (*F)(arg0, arg1);
          return P(res);
        }

      };

    template<typename ResultType, typename Arg1, typename Arg2, typename Arg3,
        ResultType
        (*F)(Arg1 a, Arg2 b, Arg3 c), sstmac::payload::const_ptr
        (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class plain_result_action3 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<plain_result_action3> ptr;

        static ptr
        construct()
        {
          return ptr(
              new plain_result_action3<ResultType, Arg1, Arg2, Arg3, F, P> ());
        }

        virtual
        ~plain_result_action3()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "plain_result_action3: acting \n";
          typename hpx::hpx_action_params3<Arg1, Arg2, Arg3>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params3<Arg1, Arg2,
                  Arg3> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          Arg3 arg2 = pcast->get_arg2();
          ResultType res = (*F)(arg0, arg1, arg2);
          return P(res);
        }

      };

    template<typename ResultType, typename Arg1, typename Arg2, typename Arg3,
        typename Arg4, ResultType
        (*F)(Arg1 a, Arg2 b, Arg3 c, Arg4 d), sstmac::payload::const_ptr
        (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class plain_result_action4 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<plain_result_action4> ptr;

        static ptr
        construct()
        {
          return ptr(
              new plain_result_action4<ResultType, Arg1, Arg2, Arg3, Arg4, F, P> ());
        }

        virtual
        ~plain_result_action4()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "plain_result_action4: acting \n";
          typename hpx::hpx_action_params4<Arg1, Arg2, Arg3, Arg4>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params4<Arg1, Arg2,
                  Arg3, Arg4> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          Arg3 arg2 = pcast->get_arg2();
          Arg4 arg3 = pcast->get_arg3();
          ResultType res = (*F)(arg0, arg1, arg2, arg3);
          return P(res);
        }

      };

    template<typename ResultType, typename CompType, ResultType
    (CompType::*F)(), sstmac::payload::const_ptr
    (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class component_result_action0 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<component_result_action0> ptr;

        static ptr
        construct()
        {
          return ptr(
              new component_result_action0<ResultType, CompType, F, P> ());
        }

        virtual
        ~component_result_action0()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx::components::hpx_component::ptr comp,
            hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "component_result_action0: acting \n";

          boost::intrusive_ptr<CompType> ccast = boost::dynamic_pointer_cast<
              CompType>(comp);
          ResultType res = (ccast.get()->*F)();
          return P(res);
        }

      };

    template<typename ResultType, typename CompType, typename Arg1, ResultType
    (CompType::*F)(Arg1 a), sstmac::payload::const_ptr
    (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class component_result_action1 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<component_result_action1> ptr;

        static ptr
        construct()
        {
          return ptr(
              new component_result_action1<ResultType, CompType, Arg1, F, P> ());
        }

        virtual
        ~component_result_action1()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx::components::hpx_component::ptr comp,
            hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "component_result_action1: acting \n";
          typename hpx::hpx_action_params1<Arg1>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params1<Arg1> >(
                  params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          boost::intrusive_ptr<CompType> ccast = boost::dynamic_pointer_cast<
              CompType>(comp);
          Arg1 arg0 = pcast->get_arg0();
          ResultType res = (ccast.get()->*F)(arg0);
          return P(res);

        }

      };

    template<typename ResultType, typename CompType, typename Arg1,
        typename Arg2, ResultType
        (CompType::*F)(Arg1 a, Arg2 b), sstmac::payload::const_ptr
        (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class component_result_action2 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<component_result_action2> ptr;

        static ptr
        construct()
        {
          return ptr(
              new component_result_action2<ResultType, CompType, Arg1, Arg2, F,
                  P> ());
        }

        virtual
        ~component_result_action2()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx::components::hpx_component::ptr comp,
            hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "component_result_action2: acting \n";
          typename hpx::hpx_action_params2<Arg1, Arg2>::ptr
              pcast = boost::dynamic_pointer_cast<hpx::hpx_action_params2<Arg1,
                  Arg2> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          boost::intrusive_ptr<CompType> ccast = boost::dynamic_pointer_cast<
              CompType>(comp);
          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          ResultType res = (ccast.get()->*F)(arg0, arg1);
          return P(res);
        }

      };

    template<typename ResultType, typename CompType, typename Arg1,
        typename Arg2, typename Arg3, ResultType
        (CompType::*F)(Arg1 a, Arg2 b, Arg3 c), sstmac::payload::const_ptr
        (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class component_result_action3 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<component_result_action3> ptr;

        static ptr
        construct()
        {
          return ptr(
              new component_result_action3<ResultType, CompType, Arg1, Arg2,
                  Arg3, F, P> ());
        }

        virtual
        ~component_result_action3()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx::components::hpx_component::ptr comp,
            hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "component_result_action3: acting \n";
          typename hpx::hpx_action_params3<Arg1, Arg2, Arg3>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params3<Arg1, Arg2,
                  Arg3> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          boost::intrusive_ptr<CompType> ccast = boost::dynamic_pointer_cast<
              CompType>(comp);

          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: component didn't cast");
            }

          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          Arg3 arg2 = pcast->get_arg2();
          ResultType res = (ccast.get()->*F)(arg0, arg1, arg2);
          return P(res);
        }

      };

    template<typename ResultType, typename CompType, typename Arg1,
        typename Arg2, typename Arg3, typename Arg4, ResultType
        (CompType::*F)(Arg1 a, Arg2 b, Arg3 c, Arg4 d),
        sstmac::payload::const_ptr
        (*P)(ResultType r) = hpx::parcels::payloadizer<ResultType> >
      class component_result_action4 : public hpx_action
      {

      public:
        typedef boost::intrusive_ptr<component_result_action4> ptr;

        static ptr
        construct()
        {
          return ptr(
              new component_result_action4<ResultType, CompType, Arg1, Arg2,
                  Arg3, Arg4, F, P> ());
        }

        virtual
        ~component_result_action4()
        {
        }

        virtual sstmac::payload::const_ptr
        act(hpx::components::hpx_component::ptr comp,
            hpx_action_params::ptr params)
        {
          SSTMAC_DEBUG << "component_result_action4: acting \n";
          typename hpx::hpx_action_params4<Arg1, Arg2, Arg3, Arg4>::ptr pcast =
              boost::dynamic_pointer_cast<hpx::hpx_action_params4<Arg1, Arg2,
                  Arg3, Arg4> >(params);
          if (!pcast)
            {
              sst_throw(sstmac::ssterror, "hpx::component_result_action3: params didn't cast");
            }
          boost::intrusive_ptr<CompType> ccast = boost::dynamic_pointer_cast<
              CompType>(comp);
          Arg1 arg0 = pcast->get_arg0();
          Arg2 arg1 = pcast->get_arg1();
          Arg3 arg2 = pcast->get_arg2();
          Arg4 arg3 = pcast->get_arg3();
          ResultType res = (ccast.get()->*F)(arg0, arg1, arg2, arg3);
          return P(res);
        }

      };

    void
    compute(const sstmac::timestamp &t);

    sstmac::nodeaddress::ptr
    find_here();

#define HPX_DBG if(hpx::actions::hpx_action::dbg().is_active()) hpx::actions::hpx_action::dbg() << hpx::actions::hpx_action::dbg().time()
  }
}

#endif
