/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include <sstmac/software/api/hpx/runtime/hpx_random_balancer.h>

namespace hpx
{

  using namespace sstmac;
  using namespace std;
  using namespace RNG;

  rngint_t RMAX = 100000;

  RNG::Combo::ptr hpx_random_balancer::rng_;

  void
  hpx_random_balancer::balance(
      boost::unordered_map<long, boost::intrusive_ptr<
          hpx::components::hpx_component> > comps,
      sstmac::sw::environment::const_ptr env,
      boost::unordered_map<hpx::components::hpx_component::ptr,
          nodeaddress::ptr> &change)
  {
    boost::unordered_map<long, boost::intrusive_ptr<
        hpx::components::hpx_component> >::iterator it, end = comps.end();
    for (it = comps.begin(); it != end; it++)
      {
        if (it->second->movable())
          {
            rngint_t r = rng_->value() % RMAX;
            double v = (double) r / (double) RMAX;

            if (v < thres_)
              {
                long i = rng_->value() % env->get_app_count();
                nodeaddress::ptr goingto = env->get_node_address(i);
                change[it->second] = goingto;
              }
          }
      }

  }

}
