/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_KEYVAL_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_KEYVAL_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/software/api/mpi/sstmac_mpi.h>

namespace sstmac
{
  namespace sw
  {

    class keyval : public ptr_type
    {
    protected:
      keyval(int k, MPI_Copy_function* c, MPI_Delete_function* d, void* e) :
        key_(k), copy_(c), del_(d), extra_(e)
      {

      }

      int key_;
      MPI_Copy_function* copy_;
      MPI_Delete_function* del_;
      void* extra_;


      void* val_;

      static int keycount;

    public:
      typedef boost::intrusive_ptr<keyval> ptr;

      static ptr
      construct(int k, MPI_Copy_function* c, MPI_Delete_function* d, void* e)
      {
        return ptr(new keyval(k, c, d, e));
      }

      static int
      get_new_key() {
        return keycount++;
      }

      virtual
      ~keyval()
      {
      }

      virtual std::string
      to_string() const {
        return "keyval";
      }

      keyval::ptr
      clone(int k) const {
        keyval::ptr ret = construct(k, copy_, del_, extra_);
        int flag = 0;
        (copy_)(-1, key_, extra_, val_, ret->val_, &flag);
        if(flag != 0){
            sst_throw(ssterror, "mpi_comm::keyval - copy function returned with flag %d", flag);
        }
        return ret;
      }

      int
      key() const {
        return key_;
      }

      void
      set_val(void* val){
        val_ = val;
      }

      void*
      val() const {
        return val_;
      }
    };

  }
}

#endif /* KEYVAL_H_ */
