#include <sstmac/software/api/mpi/mpi_comm/mpicomm_cart.h>

namespace sstmac {
    namespace sw {
  /// Hello.
  mpicomm_cart::mpicomm_cart(
      const mpicommid &id, //const appid &aid,
      const mpiid &rank, mpigroup::ptr peers,
      const environment::const_ptr &env, const appid &aid, int ndims,
      int *dims, int *periods, int reorder) :
    mpicomm(id, rank, peers, env, aid), ndims_(ndims), reorder_(reorder)
  {

    for (int i = 0; i < ndims; i++)
      {
        dims_.push_back(dims[i]);
        periods_.push_back(periods[i]);
      }

    topotype_ = TOPO_CART;
  }

  mpicomm_cart::ptr
  mpicomm_cart::construct(
      const mpicommid &id, //const appid &aid,
      const mpiid &rank, mpigroup::ptr peers,
      const environment::const_ptr &env, const appid &aid, int ndims,
      int *dims, int *periods, int reorder)
  {
    return ptr(
        new mpicomm_cart(id, rank, peers, env, aid, ndims, dims, periods,
            reorder));
  }

  void
  mpicomm_cart::set_coords(int rank, int* coords)
  {
    int prev = 1;
    for (uint i = 0; i < dims_.size(); i++)
      {
        int co = (rank / prev) % dims_[i];

        coords[i] = co;
        prev = prev * dims_[i];
      }
  }

  int
  mpicomm_cart::get_rank(int* coords)
  {
    int rank = 0;
    int prev = 1;
    for (uint i = 0; i < dims_.size(); i++)
      {
        rank += coords[i] * prev;
        prev *= dims_[i];
      }
    return rank;
  }

  int
  mpicomm_cart::shift(int dir, int dis)
  {

    if (dir >= (int) dims_.size())
      {
        sst_throw(ssterror, "mpicomm_cart::shift: dir %d is too big for dims %d",
                       dir, dims_.size());
      }
    int coords[dims_.size()];
    set_coords(rank_.id, coords);
    coords[dir] += dis;

    if (coords[dir] >= dims_[dir])
      {
        if (periods_[dir])
          {
            coords[dir] = coords[dir] % dims_[dir];
            return get_rank(coords);
          }
        else
          {
            return mpicomm::proc_null;
          }

      }
    else if (coords[dir] < 0)
      {
        if (periods_[dir])
          {
            coords[dir] = (dims_[dir] + coords[dir]) % dims_[dir];
            return get_rank(coords);
          }
        else
          {
            return mpicomm::proc_null;
          }
      }
    else
      {
        return get_rank(coords);
      }

  }

 }
}
