/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_MPICOMM_CART_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_COMM_MPICOMM_CART_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_comm/mpicomm.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * An MPI_Cart communicator handle.
     */
    class mpicomm_cart : public mpicomm
    {
    protected:
      int ndims_;
      std::vector<int> dims_;
      std::vector<int> periods_;
      int reorder_;

      /// Hello.
      mpicomm_cart(
          const mpicommid &id, //const appid &aid,
          const mpiid &rank, mpigroup::ptr peers,
          const environment::const_ptr &env, const appid &aid, int ndims,
          int *dims, int *periods, int reorder);

    public:
      typedef boost::intrusive_ptr<mpicomm_cart> ptr;

      static ptr
      construct(
          const mpicommid &id, //const appid &aid,
          const mpiid &rank, mpigroup::ptr peers,
          const environment::const_ptr &env, const appid &aid, int ndims,
          int *dims, int *periods, int reorder);

      /// Goodbye.
      virtual
      ~mpicomm_cart()
      {
      }

      int
      get_dim(uint i) const
      {
        if (i > dims_.size())
          return -1;
        return dims_[i];
      }

      void set_coords(int rank, int* coords);

      int
      get_period(uint i) const
      {
        if (i > periods_.size())
          return -1;
        return periods_[i];
      }

      int get_rank(int* coords);


      int shift(int dir, int dis);

      int
      get_ndims() const {
        return ndims_;
      }

    };
  } //end of namespace sstmac
}
#endif
