
#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


void
eager_ssend::recv_header(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    sst_throw(illformederror, "eager_ssend protocol should never have recv-request handle a header");
}

void
eager_ssend::recv_payload(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    queue->send_completion_ack(msg);
}

bool
eager_ssend::send_needs_completion_ack() const
{
    return true;
}

bool
eager_ssend::send_needs_nic_ack() const
{
    return false;
}

bool
eager_ssend::send_needs_eager_ack() const
{
    return false;
}

bool
eager_ssend::send_needs_rendezvous_ack() const
{
    return false;
}

void
eager_ssend::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    sst_throw(illformederror, "eage_ssend protocol handling payload, but only headers should arrive");
}

bool
eager_ssend::handshake_only() const
{
    return false;
}

void
eager_ssend::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    queue->post_header(msg);
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: eager_ssend::ssend_header done \n";
}

void
eager_ssend::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req)
        req->handle(msg);
    queue->notify_probes(msg);
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: eager_ssend::handle_header done \n";
}

    }
}
