#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


void
rendezvous_rdma::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: RDMA Send Header");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: start rendezvous rdma send of message " << msg->to_string()
          << "\n";
    //msg->set_needs_ack(false);  //we don't care about the nic ack for this

    queue->post_header(msg);
}

void
rendezvous_rdma::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: RDMA Handle Header");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: handle rendezvous rdma header on msg "
          << msg->to_string() << "\n";

    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req){
        if (req->open_source()){
            //we have to initialize the recv
            //event::ptr ev = new_event(req, &mpiqueue::recvrequest::handle, msg);
            //init_system_recv(msg->payload_bytes(), queue, ev);
            req->handle(msg);
        }
        else {
            req->handle(msg);
        }
    }
    queue->notify_probes(msg);
}

void
rendezvous_rdma::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: RDMA Handle Payload");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: handle rendezvous rdma payload on msg "
          << msg->to_string() << "\n";

    mpiqueue::pending_req_map::iterator it = queue->recv_needs_payload_.find(msg->id());
    if (it == queue->recv_needs_payload_.end())
    {
        sst_throw(illformederror,
                  "mpiqueue[%s]: rendezvous_rdma::handle_payload: "
                  "data message without a matching ack on %s",
                  queue->id_string().c_str(),
                  msg->to_string().c_str());
    }
    mpiqueue::recvrequest::ptr recver = it->second;
    queue->recv_needs_payload_.erase(it);
    recver->handle(msg);
}

bool
rendezvous_rdma::send_needs_completion_ack() const
{
    return true;
}

bool
rendezvous_rdma::send_needs_rendezvous_ack() const
{
    return false;
}

void
rendezvous_rdma::recv_header(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: RDMA Recv Header");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: recv rendezvous rdma header on msg "
          << msg->to_string() << "\n";

    msg->mutate(sst_message::GNI_HEADER_TO_RDMA_GET_REQ);
    msg->set_needs_ack(false);
    queue->recv_needs_payload_[msg->id()] = req;
    queue->post_rdma(msg);
}

void
rendezvous_rdma::recv_payload(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: RDMA Recv Payload");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: recv rendezvous rdma payload on msg "
          << msg->to_string() << "\n";
    queue->send_completion_ack(msg);
}


    }
}
