#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpiserver.h>

namespace sstmac {
    namespace sw {


bool
rendezvous_socket::send_needs_completion_ack() const
{
    return false;
}

bool
rendezvous_socket::send_needs_rendezvous_ack() const
{
    return true;
}

void
rendezvous_socket::send_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: Socket Send Header");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: start rendezvous socket send\n";
    queue->post_header(msg);
}

void
rendezvous_socket::handle_header(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: Socket Handle Header");
    mpiqueue::recvrequest::ptr req = queue->find_pending_request(msg);
    if (req)
      req->handle(msg);

    queue->notify_probes(msg);

    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]:: handle new rendezvous socket header done \n";
}

void
rendezvous_socket::handle_payload(const mpiqueue::ptr &queue, const mpimessage::ptr &msg)
{
    SSTMACBacktrace("MPI Rendezvous Protocol: Socket Handle Payload");
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]::incoming rendezvous payload "
          << msg->to_string() << "\n";
    mpiqueue::pending_req_map::iterator it = queue->recv_needs_payload_.find(msg->id());
    if (it == queue->recv_needs_payload_.end())
    {
        sst_throw(illformederror,
                  "mpiqueue[%s]: rendezvous_socket::handle_payload: "
                  "data message without a matching ack on %s",
                  queue->id_string().c_str(),
                  msg->to_string().c_str());
    }
    mpiqueue::recvrequest::ptr recver = it->second;
    queue->recv_needs_payload_.erase(it);
    recver->handle(msg);
}

void
rendezvous_socket::recv_payload(
    const mpiqueue::ptr &queue,
    const mpimessage::ptr &msg,
    const mpiqueue::recvrequest::ptr& req
)
{
    SSTMAC_DEBUG << "mpiqueue[" << queue->id_string()
          << "]: recv rendezvous payload "
          << msg->to_string() << "\n";
}

    }
}

