/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_protocol/mpiprotocol.h>
#include <sstmac/software/api/mpi/mpi_queue/mpiqueue-sendrequest.h>
#include <sstmac/software/api/mpi/mpiserver.h>
#include <sstmac/software/api/mpi/mpimessage.h>
#include <sstmac/common/messages/payload.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  
  

  //
  // Hello.
  //
  mpiqueue::sendrequest::sendrequest(const mpimessage::ptr& mess,
                                     const mpirequest_t key,
                                     const mpiqueue::ptr& queue,
                                     const eventhandler::ptr& completion) :
      queue_(queue), completion_(completion), key_(key), mess_(mess)
  {
  }

  //
  // Hi there.
  //
  mpiqueue::sendrequest::ptr
  mpiqueue::sendrequest::construct(const mpimessage::ptr& mess, const mpirequest_t key, const mpiqueue::ptr& queue,
                                   const eventhandler::ptr& completion)
  {
    sendrequest::ptr retval(new sendrequest(mess, key, queue, completion));
    return retval;
  }

  //
  // Goodbye.
  //
  mpiqueue::sendrequest::~sendrequest() throw ()
  {
  }

  //
  // Test whether we match a given message, for nic send acks.
  //
  bool
  mpiqueue::sendrequest::matches(const mpimessage::ptr& message) const
  {
    bool sameid = (mess_->seqnum() == message->seqnum());
    bool destmatches = mess_->dest().matches(message->dest());
    bool catmatches = mess_->cat() == message->cat();
    return (sameid && destmatches && catmatches
            && mess_->source().matches(message->source())
            && mess_->tag().matches(message->tag()));
  }

  void
  mpiqueue::sendrequest::wait_for_buffer()
  {
      /**
        CHANGE! At some point, we may want to track buffer
        resources for MPI. For now, let's not worry about buffers.
        Just clear the completion handlers and use this as a
        placeholder for clearing nic acks
      */
    key_ = 0;
    completion_ = 0;
  }

  void
  mpiqueue::sendrequest::complete(const mpimessage::ptr &msg)
  {

      SSTMAC_DEBUG << "completing sendrequest"
           << ", dst=" << mess_->dest().id
           << ", tag=" << mess_->tag().id
           << ", cat=" << mpimessage::str(msg->cat())
           << ", count=" << mess_->count()
           << ", type=" << mess_->type().label
           << ", seqnum=" << mess_->seqnum()
           << "\n";

    if (key_)
        key_->complete(msg);

    if (completion_)
      completion_->handle(msg);

  }

} } // end of namespace sstmac.
