/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPILOGTWOVIRT_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPILOGTWOVIRT_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/software/api/mpi/mpi_types/mpiop.h>
#include <deque>

namespace sstmac { namespace sw
{

  class mpilogtwovirtallreduce : public mpicollective
  {
    private:
      int nproc_;

      int my_rank_;

      /// Number of elements in our complete data array.
      int count_;
      /// The datatype we're sending and receiving.
      mpitype type_;
      /// The operation we're applying to our data.
      mpiop op_;
      /// The root node -- only used if allreduce_ is false.
      mpiid root_;

      /// We can have any number of pending sends, but the collective op
      /// is not over until all the sends have completed.
      int pending_sends_;
      /// We will not dispatch the next operation (send or receive) until
      /// a pending receive is retired.  There should only be zero or one.
      int pending_recvs_;

      int num_rounds_;

      int round_;

      /// We have a one-deep stack telling us whether we need to reduce
      /// the results from the next receive or not.
      bool reducing_round_;

      bool complete_lock_;

      std::set<int> my_roles_;

      static int two_to_power(int n);

      static int log_base_two(int n);

      int virtual_rank_to_actual_rank(int me);

      struct exchange {
          int partner;
          int count;
          exchange(int p, int c) :
              partner(p), count(c)
          {
          }
      };

      void do_run_round(int round, int me, int virtual_me,
                        std::list<exchange>& to_do);

      void configure_round(int me, int round,
                           int& count, int& partner);

      void run_next_round();

    protected:
      mpilogtwovirtallreduce(const mpirequest::ptr& thekey,
          const mpiqueue::ptr& queue, int count, const mpitype &datatype,
          const mpiop &op, const mpiid &root, const mpitag &tag,
          const mpicomm::ptr &comm, const payload::const_ptr& content,
          const eventhandler::ptr& completion);

      /// Callback method to indicate that a send operation has completed.
      virtual void
      send_complete(int count, const mpitype &type, const mpitag &tag, const mpiid &dest);

      /// Callback method to indicate that a receive operation has completed.
      virtual void
      recv_complete(int count, const mpistatus_t& stat);

      /// Callback method to indicate that a copy operation has completed.
      virtual void
      copy_complete(int count, const mpitype &type, const mpitag &tag,
          const mpiid &source);

  public:
      typedef boost::intrusive_ptr<mpilogtwovirtallreduce> ptr;

      /// Hi.
      static ptr
      construct(const mpirequest::ptr& thekey,
          const mpiqueue::ptr& queue, int count,
          const mpitype &datatype, const mpiop &op, const mpiid &root,
          const mpitag &tag, const mpicomm::ptr &comm,
          const payload::const_ptr& content,
          const eventhandler::ptr& completion);

      void start();

      virtual ~mpilogtwovirtallreduce() throw () {};

      std::string to_string() const {
          return "mpilogtwovirtallreduce";
      }

  };
 }
}
#endif // MPILOGTWOVIRT_H
