/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPISIMULATANEOUSBTREESCAN_H
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_ENGINES_MPISIMULATANEOUSBTREESCAN_H


#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollpayload.h>
#include <sstmac/software/api/mpi/mpi_types/mpitype.h>
#include <sstmac/software/api/mpi/mpi_types/mpitag.h>
#include <sstmac/software/api/mpi/mpi_comm/mpicomm.h>
#include <sstmac/software/api/mpi/mpi_types/mpiop.h>

namespace sstmac { namespace sw {

  /// Important types.
  class mpiqueue;
  class mpistatus;
  class nodeinterface;
  class topologyiterator;

  /**
   * An mpikernel to performs an mpi_scan.  This uses the simultaneous binary tree algorithm
   * from "Parallel Prefix (Scan) Algorithms for MPI" in
   * Recent Advances in Parallel Virtual Machine and Message Passing Interface: 13th edition
   */

  class mpi_simultaneous_btree_scan_engine : public mpicollective {
    /// The lenght of our data array.
    int count_;

    /// The type we are sending and receiving.
    mpitype type_;

    /// The operation we are applying to our data.
    mpiop op_;


    /// The data we are collecting.
    payload::const_ptr content_;

    int send_round_;

    int send_two_to_the_k_;

    int recv_round_;

    int recv_two_to_the_k_;

    int me_;

    int nproc_;

    /// The number of pending sends.
    int pending_sends_;

    /// The number of pending receives.
    int pending_recvs_;

    bool send_done_;

    bool recv_done_;

    /// Fire off the next set of send and receive operations.
    void sendrecv();

    void next_send();

    void next_recv();

  protected:
    /// Hi.
    mpi_simultaneous_btree_scan_engine(const mpirequest::ptr& thekey,
                  const mpiqueue::ptr& queue,
                  int count, const mpitype &type,
                  const mpiop &op,
                  const mpitag &tag, const mpicomm::ptr &comm,
                  const payload::const_ptr& content,
                  const eventhandler::ptr& completion);

    /// Callback method to indicate that a send operation has completed.
    virtual void send_complete(int count,
                               const mpitype &type,
                               const mpitag &tag,
                               const mpiid &dest);

    /// Callback method to indicate that a receive operation has completed.
    virtual void recv_complete(int count, const mpistatus_t& stat);

    /// Callback to to signal copy completion.
    void copy_complete(int count,
                       const mpitype &type,
                       const mpitag &tag,
                       const mpiid &source);

  public:
    typedef boost::intrusive_ptr<mpi_simultaneous_btree_scan_engine> ptr;

    /// Make a new engine.
    static mpi_simultaneous_btree_scan_engine::ptr
    construct(const mpirequest::ptr& thekey,
              const mpiqueue::ptr& queue,
              int count, const mpitype &type,
              const mpiop &op, const mpitag &tag, const mpicomm::ptr &comm,
              const payload::const_ptr& content,
              const eventhandler::ptr& completion);

    /// Goodbye.
    virtual ~mpi_simultaneous_btree_scan_engine() throw();

    /// Capture the 'start' method so we can set up our reducer.
    virtual void start();

    std::string to_string() const;

  };

} } // end of namespace sstmac

#endif // MPISIMULATANEOUSBTREESCAN_H
