/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLPAYLOAD_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPI_COLLECTIVE_MPICOLLPAYLOAD_H_INCLUDED

#include <sstmac/common/messages/payload.h>

namespace sstmac
{
  namespace sw
  {

    class mpicollpayload : public payload, public serializable_type<
        mpicollpayload>
    {
    ImplementSerializable(mpicollpayload)

    public:
      typedef boost::intrusive_ptr<mpicollpayload> ptr;
      typedef boost::intrusive_ptr<const mpicollpayload> const_ptr;

      static mpicollpayload::ptr
      construct(const std::vector<payload::const_ptr> & l)
      {
        mpicollpayload::ptr ret(new mpicollpayload(l));
        return ret;
      }

      static mpicollpayload::ptr
      construct(const const_ptr &p)
      {
        mpicollpayload::ptr ret(new mpicollpayload(p->get_content()));
        return ret;
      }

      virtual serializable::ptr
      serialization_clone() const
      {
        return ptr(new mpicollpayload(contents_));
      }

      static ptr
      construct()
      {
        return ptr(new mpicollpayload());
      }

      virtual
      ~mpicollpayload() throw ()
      {
      }

      std::vector<payload::const_ptr>
      get_content() const
      {
        return contents_;
      }

      void
      set_content(const payload::const_ptr &p, int index)
      {
        contents_[index] = p;
      }

      virtual std::string
      to_string() const
      {
        return "mpicollpayload";
      }

      virtual payload::const_ptr
      clone() const
      {

        payload::const_ptr ret(new mpicollpayload(contents_));
        return ret;
      }

      virtual long
      get_byte_length() const
      {
        long ret = 0;
        for (uint i = 0; i < contents_.size(); i++)
          ret += contents_[i]->get_byte_length();
        return ret;
      }

      virtual const payload::const_ptr
      add(const payload::const_ptr &other) const
      {

        throw unimplementederror(
            "mpicollpayload:add doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      prod(const payload::const_ptr &other) const
      {

        throw unimplementederror(
            "mpicollpayload:prod doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      min(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator< doesn't make a whole lot of sense.");

      }

      virtual const payload::const_ptr
      max(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator> doesn't make a whole lot of sense.");

      }

      virtual bool
      equals(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator== doesn't make a whole lot of sense.");

      }

      virtual const payload::const_ptr
      logical_or(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator|| doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      logical_xor(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator xor doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      logical_and(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator&& doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      bitwise_or(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator| doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      bitwise_xor(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator^ doesn't make a whole lot of sense.");
      }

      virtual const payload::const_ptr
      bitwise_and(const payload::const_ptr &other) const
      {
        throw unimplementederror(
            "mpicollpayload:operator& doesn't make a whole lot of sense.");
      }

      virtual void
      serialize_order(sst_serializer* ser);

    protected:
      std::vector<payload::const_ptr> contents_;

      mpicollpayload(const std::vector<payload::const_ptr> & l) :
        contents_(l)
      {

      }

      mpicollpayload()
      {

      }
    };

  }
}

#endif
