/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREALLTOALLV_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREALLTOALLV_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  //
  // Default strategy for alltoallv operations with variable data lengths.
  //
  class mpicorealltoallv : public mpialltoallvstrategy {
    /// Hi.
    mpicorealltoallv();

  public:
    typedef boost::intrusive_ptr<mpicorealltoallv> ptr;
    /// Hello.
    static mpicorealltoallv::ptr construct();

    /// Must be virtual.
    virtual ~mpicorealltoallv() throw();

    /// Build a kernel for a fused reduce-alltoallv operation
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          const std::vector<int> &sendcnts, const mpitype &sendtype,
          const std::vector<int> &recvcnts, const mpitype &recvtype,
          const mpitag &tag, const mpicomm::ptr &comm,
          const std::vector<payload::const_ptr >& content,
          OS_API::ptr &os
          ) const;
  };

} } // end of namespace sstmac

#endif
