/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREBCAST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREBCAST_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

#include <sstmac/software/api/mpi/mpi_strategies/mpi_iterator/topologyiterator.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/mpicollective.h>

namespace sstmac { namespace sw
{

  /**
   * Broadcast between nodes.
   */
  class mpicorebcast : public mpibcaststrategy
  {



    /// Fire off the next set of send and receive operations.
    void
    sendrecv();

  protected:
    /// Hello.
    mpicorebcast();


  public:
    typedef boost::intrusive_ptr<mpicorebcast> ptr;

    /// Hi.
    static mpicorebcast::ptr
    construct();

    /// Goodbye.
    virtual
    ~mpicorebcast() throw ();

    /// Build a kernel to perform a broadcast.
    virtual void
    execute(const mpirequest_t& thekey,
        const mpiqueue::ptr& queue, int count,
        const mpitype &type, const mpiid &root, const mpitag &tag,
        const mpicomm::ptr &comm, const payload::const_ptr& content,
        OS_API::ptr &os
        ) const;
  };

} } // end of namespace sstmac

#endif
