/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORERECV_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORERECV_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  /// Basic receive strategy.  Upon receipt, the receiver will test
  /// whether the sender wanted a return receipt etc. and respond accordingly.
  class mpicorerecv : public mpirecvstrategy {
  protected:
    /// Hello.
    mpicorerecv();
  public:
    typedef boost::intrusive_ptr<mpicorerecv> ptr;

    /// Hello.
    static mpicorerecv::ptr construct();
    /// Goodbye.
    virtual ~mpicorerecv() throw();
    /// Post a receive request from the given node to be matched by the
    /// given request handle.  All receives are non-blocking; it is up
    /// to the strategy to handle return receipts or other synchronization.
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          int count, const mpitype &type, const mpiid &source,
          const mpitag &tag, const mpicomm::ptr &comm,
          OS_API::ptr &os
          ) const;
  };



} } // end of namespace sstmac.

#endif
