/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorereduce.h>
#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpirabenseifner.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hi.
  //
  mpicorereduce::mpicorereduce()
  {
  }

  //
  // Hello.
  //
  mpicorereduce::ptr
  mpicorereduce::construct()
  {
    return mpicorereduce::ptr (new mpicorereduce);
  }

  //
  // Perform a fused reduce-reduce operation
  //
  void
  mpicorereduce::execute(const mpirequest_t& req,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiop &op, const mpiid &root, const mpitag &tag,
      const mpicomm::ptr &comm, const payload::const_ptr& content,
      OS_API::ptr &os) const
  {
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpirabenseifner::ptr engine = mpirabenseifner::construct(
        false, req, queue, count, type, op, root, tag, comm, content,
        completion);

    engine->start();
  }

} } // end of namespace sstmac
