/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorereduce_scatter.h>
#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpihalvingreducescatter.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw {

  // Pointer types.


  //
  // Hi.
  //
  mpicorereduce_scatter::mpicorereduce_scatter() {
  }

  //
  // Hello.
  //
  mpicorereduce_scatter::ptr mpicorereduce_scatter::construct() {
    return mpicorereduce_scatter::ptr(new mpicorereduce_scatter);
  }

  //
  // Goodbye
  //
  mpicorereduce_scatter::~mpicorereduce_scatter() throw() {
  }

  //
  // Perform a fused reduce-scatter operation
  //
  void
  mpicorereduce_scatter::execute(const mpirequest_t& req,
      const mpiqueue::ptr& queue,
      const std::vector<int> &recvcnts,
      const mpitype &type, const mpiop &op,
      const mpitag &tag, const mpicomm::ptr &comm,
      const payload::const_ptr& content, OS_API::ptr &os
  ) const
  {
    eventhandler::ptr completion = queue->progress_done_handler(os, req);
    mpihalvingreducescatter::ptr engine = mpihalvingreducescatter::construct(
        req, queue, recvcnts, type, op, tag, comm, content, completion);

    engine->start();
  }

} } // end of namespace sstmac
