/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREREDUCE_SCATTER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREREDUCE_SCATTER_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  //
  /// Base strategy for fused reduce-scatter operations.
  //
  class mpicorereduce_scatter : public mpireduce_scatterstrategy {
  protected:
    /// Hi.
    mpicorereduce_scatter();
  public:
    typedef boost::intrusive_ptr<mpicorereduce_scatter> ptr;

    /// Hello.
    static mpicorereduce_scatter::ptr construct();

    /// Goodbye
    virtual ~mpicorereduce_scatter() throw();

    /// Build a kernel to perform a fused reduce-scatter operation
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          const std::vector<int> &recvcnts, const mpitype &type,
          const mpiop &op, const mpitag &tag, const mpicomm::ptr &comm,
          const payload::const_ptr& content, OS_API::ptr &os

          ) const;
  };

} } //end of namespace sstmac

#endif
