/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORERSEND_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORERSEND_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  /**
   * Basic strategy for an "I'm sending and you better be ready" operation.
   */
  class mpicorersend : public mpisendstrategy {
    /// Private mpikernel type to handle the actual communications.
    class engine;
  protected:
    mpicorersend();
  public:
    typedef boost::intrusive_ptr<mpicorersend> ptr;

    /// Hello.
    static mpicorersend::ptr construct();
    /// Goodbye.
    virtual ~mpicorersend() throw();
    /// Basic nonblocking send.  A simulator may opt to delay completion of
    /// this operation depending on message size.
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          int count, const mpitype &type, const mpiid &dest,
          const mpitag &tag, const mpicomm::ptr &comm,
          const payload::const_ptr& content,
          OS_API::ptr &os
          ) const;
  };

} } // end of namespace sstmac

#endif
