/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_strategies/mpicorescan.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpi_collective/engines/mpisimultaneousbtreescan.h>



namespace sstmac { namespace sw {

  // Pointer types.
  

  // 
  // Hello.
  //
  mpicorescan::mpicorescan() {
  }

  //
  // Build a new object.
  //
  mpicorescan::ptr mpicorescan::construct() {
    return mpicorescan::ptr(new mpicorescan);
  }

  //
  // Destructor.
  //
  mpicorescan::~mpicorescan() throw() {
  }

  //
  // Build a kernel to perform a scan operation
  //
  void
  mpicorescan::execute(const mpirequest_t& req,
                     const mpiqueue::ptr& queue,
                     int count, const mpitype &type, const mpiop &op,
                     const mpitag &tag, const mpicomm::ptr &comm,
                     const payload::const_ptr& content,
                     OS_API::ptr &os
                     ) const
  {
      eventhandler::ptr completion = queue->progress_done_handler(os, req);
      mpi_simultaneous_btree_scan_engine::ptr engine =
            mpi_simultaneous_btree_scan_engine::construct(
                req, queue, count, type, op, tag, comm,
                content, completion);

      engine->start();
  }

} } // end of namespace sstmac

