/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORESCATTERV_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORESCATTERV_H_INCLUDED

#include <sstmac/software/api/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  //
  // Default strategy for scatterv operations.
  //
  class mpicorescatterv : public mpiscattervstrategy {
    /// Hi.
    mpicorescatterv();
  public:
    typedef boost::intrusive_ptr<mpicorescatterv> ptr;

    /// Hello.
    static mpicorescatterv::ptr construct();
    /// Must be virtual.
    virtual ~mpicorescatterv() throw() {}
    /// Perform a fused reduce-scatterv operation
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          const std::vector<int> &sendcounts, const mpitype &sendtype,
          int recvcount, const mpitype &recvtype,
          const mpiid &root, const mpitag &tag,
          const mpicomm::ptr &comm,
          const std::vector< payload::const_ptr > &cc, OS_API::ptr &os
          ) const;
  };

} } // end of namespace sstmac

#endif
