/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_TYPES_MPIOP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_TYPES_MPIOP_H_INCLUDED

#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/types.h>

namespace sstmac
{
  namespace sw
  {

    /*
     * Operations for simulating data reduction.
     */
    struct mpiop
    {
      std::string label;
      // timestamp cost_per_entry;
      // id gets assigned automatically by the constructor.
      int id;

      explicit
      mpiop(const std::string &labelit = ""); //, const timestamp &costit = time_t(0));

      mpiop(void
      (*func)(void *, void *, int *, long *), int commute);

      mpiop(const mpiop &other);

      mpiop&
      operator=(const mpiop& other);

      void
      deep_copy(const mpiop& other);

      void
      (*userfunc_)(void *, void *, int *, long *);
      int commute_;

      // Common reduction operations.
      static const mpiop max;
      static const mpiop min;
      static const mpiop sum;
      static const mpiop prod;
      static const mpiop land;
      static const mpiop band;
      static const mpiop lor;
      static const mpiop bor;
      static const mpiop lxor;
      static const mpiop bxor;
      static const mpiop minloc;
      static const mpiop maxloc;
      static const mpiop replace;
      static const mpiop op_null;
    };

    /// Allow mpiop in sorted containers.
    inline bool
    operator<(const mpiop &a, const mpiop &b)
    {
      return a.id < b.id;
    }

    inline bool
    operator==(const mpiop &a, const mpiop &b)
    {
      return a.id == b.id;
    }

  }
} // end of namespace sstmac.

#endif
