/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpi_types/mpitag.h>
#include <sstmac/common/errors.h>
#include <iostream>
#include <sstream>
#include <sstmac/software/api/mpi/mpitypes.h>

namespace sstmac { namespace sw
{

  //
  // The static "any tag" tag.
  //
  const mpitag mpitag::any_tag(MPI_ANY_TAG);

  //
  // Tag MPI requests (this is just an int in the MPI spec).
  //
  mpitag::mpitag(int idit) :
    id(idit)
  {
  }

  //
  // Test whether two mpi tags match.
  //
  bool
  mpitag::matches(const mpitag &other) const
  {
    return ((this->id == any_tag.id) || (this->id == other.id));
  }

  //
  // (In)equality comparison (stricter than matches).
  //
  bool
  operator==(const mpitag &a, const mpitag &b)
  {
    return (a.id == b.id);
  }
  bool
  operator!=(const mpitag &a, const mpitag &b)
  {
    return !(a == b);
  }
  bool
  operator<(const mpitag &a, const mpitag &b)
  {
    return (a.id < b.id);
  }

  std::string
  mpitag::to_string() const
  {
    std::stringstream ss(std::stringstream::in | std::stringstream::out);
    ss << "mpitag(id=" << id << ")";
    return ss.str();
  }

  //
  // Fairly self-explanatory.
  //
  std::ostream&
  operator<<(std::ostream &os, const mpitag &tag)
  {
    os << tag.to_string();
    return os;
  }

} } // end of namespace sstmac
