/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIAPP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIAPP_H_INCLUDED

#include <sstmac/software/process/app.h>

#include <sstmac/software/api/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/software/api/mpi/mpi_strategies/mpistrategy.h>
#include <sstmac/software/api/mpi/mpi_comm/mpicommfactory.h>
#include <sstmac/software/api/mpi/mpiapi.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * A base type for applications that can run via mpi.
     * Intended to be safe for multiple inheritance (for applications
     * that use MPI and some other form of communication).
     */
    class mpiapp : public app
    {
      mpiapi::ptr mpi_;



    protected:

      mpiapp(softwareid id, std::string a) :
          app(id)
      { }

      mpiapp(const sim_parameters::ptr& params) : app(params) {}

      mpiapp(softwareid id) :
          app(id)
      { }

    public:

      /// Access the MPI API.
      const mpiapi::ptr&
      mpi() const
      {
        return mpi_;
      }

      typedef boost::intrusive_ptr<mpiapp> ptr;

      /// Goodbye.
      virtual ~mpiapp();

      virtual void kill();

      virtual std::string
      to_string() const
      {
        return "mpiapp";
      }

      virtual void
      init_libs(sim_parameters::ptr params);


    };

  }
} // end of namespace sstmac.

#endif
