/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIREQUEST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIREQUEST_H_INCLUDED

#include <sstmac/software/process/key.h>
#include <sstmac/software/api/mpi/mpistatus.h>
#include <sstmac/software/api/mpi/mpimessage.h>

#include <sstmac/common/ptr_type.h>

namespace sstmac { namespace sw
{

  class mpirequest : public ptr_type
  {
    // ------- constructor / boost stuff -------------//


  protected:
    mpirequest(const key::ptr &k);

    static debug dbg_;

  public:

    virtual
    std::string to_string() const
    {
        return "mpirequest";
    }

    typedef boost::intrusive_ptr<mpirequest> ptr;

    virtual
    ~mpirequest();

    static mpirequest::ptr
    construct(const key::ptr &k);
    // --------------------------------------//


  public:
    void
    complete(const mpimessage::ptr &msg);

    bool
    is_complete()
    {
      return complete_;
    }

    void
    cancel(){
      cancelled_ = true;
      complete(mpimessage::ptr());
    }

    const_mpistatus_t
    get_status()
    {
      return stat_;
    }

    key::ptr
    get_key()
    {
      return key_;
    }

    bool
    is_cancelled() const {
      return cancelled_;
    }

    virtual bool is_persistent() const {
        return false;
    }

  protected:
    const_mpistatus_t stat_;
    key::ptr key_;

    bool complete_;
    bool cancelled_;
  };

  typedef mpirequest::ptr const_mpirequest_t;
  typedef mpirequest::ptr mpirequest_t;

} } //end of namespace sstmac

#endif
