/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/api/mpi/mpistatus.h>
#include <sstmac/common/messages/payload.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw {

  // Pointer types.
  
  

  //
  // Hello.
  //
  mpistatus::mpistatus() :
    commid_(-1), source_(mpiid::any_source),
    dest_(mpiid::any_source), tag_(mpitag::any_tag),
    send_type_(mpitype::mpi_byte), recv_type_(mpitype::mpi_byte), sent_(0),
    recvd_(0)
  {}

  //
  // Copy.
  //
  mpistatus::mpistatus(const mpistatus &m) :
    commid_(m.commid_), source_(m.source_),
    dest_(m.dest_), tag_(m.tag_), send_type_(m.send_type_),
    recv_type_(m.recv_type_), sent_(m.sent_), recvd_(m.recvd_)
  {}

  //
  // Make me an object.
  //
  mpistatus::ptr mpistatus::construct() {
    mpistatus::ptr retval(new mpistatus);
    //retval->set_self(retval);
    return retval;
  }

  //
  // Goodbye.
  //
  mpistatus::~mpistatus() throw() {
  }

  //
  // Copy.
  //
  mpistatus::ptr mpistatus::clone() const {
    mpistatus::ptr retval(new mpistatus(*this));
   // retval->set_self(retval);
    return retval;
  }

  //
  // The communicator that was used.
  //
  const mpicommid& mpistatus::commid() const {
    return commid_;
  }

  //
  // Register the communicator used.
  //
  void mpistatus::set_commid(const mpicommid &c) {
    commid_ = c;
  }

  //
  // Transmissions originated from here:
  //
  const mpiid& mpistatus::source() const {
    return source_;
  }

  //
  // Define the source.
  //
  void mpistatus::set_source(const mpiid &s) {
    source_ = s;
  }

  //
  // Transmissions ended up here:
  //
  const mpiid& mpistatus::dest() const {
    return dest_;
  }

  //
  // Define the destination.
  //
  void mpistatus::set_dest(const mpiid &d) {
    dest_ = d;
  }

  //
  // This was the tag used:
  //
  const mpitag& mpistatus::tag() const {
    return tag_;
  }

  //
  // Define the tag.
  //
  void mpistatus::set_tag(const mpitag &t) {
    tag_ = t;
  }

  //
  // This is the type that was sent:
  //
  const mpitype& mpistatus::send_type() const {
    return send_type_;
  }

  //
  // Define the send type.
  //
  void mpistatus::set_send_type(const mpitype &t) {
    send_type_ = t;
  }

  //
  // This is the type that was received.
  //
  const mpitype& mpistatus::recv_type() const {
    return recv_type_;
  }

  //
  // Define the receive type.
  //
  void mpistatus::set_recv_type(const mpitype &t) const {
    recv_type_ = t;
  }

  //  
  // This is the content.
  payload::const_ptr mpistatus::content() const {
    return content_;
  }

  //
  // Set the content.
  //
  void mpistatus::set_content(const payload::const_ptr& cc) {
    content_ = cc;
  }

  //
   // Access bytes sent.
   //
   int64_t
   mpistatus::bytes_sent() const
   {
     return sent_;
   }

   //
   // Increment bytes set.
   //
   void
   mpistatus::add_bytes_sent(int64_t bytes)
   {
     sent_ += bytes;
   }

   //
   // Access bytes received.
   //
   int64_t
   mpistatus::bytes_received() const
   {
     return recvd_;
   }

   //
   // Increment bytes received.
   //
   void
   mpistatus::add_bytes_received(int64_t bytes)
   {
     recvd_ += bytes;
   }

} } // end of namespace sstmac.
