/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_RMA_MESSAGE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_RMA_MESSAGE_H_INCLUDED

#include <sstmac/software/api/mpi/mpimessage.h>

namespace sstmac
{
  namespace sw
  {

    class mpi_rma_message : public mpimessage
    {
    public:
      typedef boost::intrusive_ptr<mpi_rma_message> ptr;

      typedef enum
      {
        get, put
      } rmatype_t;

      struct op_info
      {
        int count_; //number of bytes
        long long disp_;
        rmatype_t type_;
        long epoch_;
        mpiid target_;
      };

    protected:
      rmatype_t rmatype_;
      int win_;
      op_info info_;


      /// Hello.
      mpi_rma_message(const std::string& libn, const timestamp& time,
          int64_t envelope, int64_t mintrans, int count, const mpitype &type,
          const mpiid &source, const mpiid &dest, const mpicommid &commid,
          int seqnum, const messageid &msgid, category cat,
          const taskid &source_task, const taskid &dest_task, const appid &aid,
          content_type_t content_type, const payload::const_ptr& content,
          const mpiprotocolPtr& protocol, MESSAGE_TYPES ty, rmatype_t rmat,
          int win, const op_info &info);

      mpi_rma_message(content_type_t content_type, rmatype_t rmat, int win,
          const op_info &info);

      mpi_rma_message(const payload::const_ptr& content,
          content_type_t content_type, rmatype_t rmat, int win,
          const op_info &info);

    public:
      /// Construct a new message.
      static mpi_rma_message::ptr
      construct(const std::string& libn, const timestamp& time,
          int64_t envelope, int64_t mintrans, int count, const mpitype &type,
          const mpiid &source, const mpiid &dest, const mpicommid &commid,
          int seqnum, const messageid &msgid, category cat,
          const taskid &source_task, const taskid &dest_task, const appid &aid,
          content_type_t content_type, const payload::const_ptr& content,
          const mpiprotocolPtr& protocol, rmatype_t rmat, int win,
          const op_info &info, MESSAGE_TYPES ty = MPI);

      static mpi_rma_message::ptr
      construct(content_type_t content_type, rmatype_t rmat, int win,
          const op_info &info)
      {
        mpi_rma_message::ptr ret(
            new mpi_rma_message(content_type, rmat, win, info));
        return ret;
      }

      static mpi_rma_message::ptr
      construct(const payload::const_ptr& content, content_type_t content_type,
          rmatype_t rmat, int win, const op_info &info)
      {
        mpi_rma_message::ptr ret(
            new mpi_rma_message(content, content_type, rmat, win, info));
        return ret;
      }

      /// Goodbye.
      virtual
      ~mpi_rma_message() throw ()
      {
      }

      virtual mpimessage::ptr
      deep_clone() const;

      virtual std::string
      to_string() const;

      int
      winid() const
      {
        return win_;
      }

      rmatype_t
      get_rmatype() const
      {
        return rmatype_;
      }
      op_info
      get_opinfo() const
      {
        return info_;
      }

      long
      epoch() const{
        return info_.epoch_;
      }

      virtual mpistatus_t
      build_status() const;
    };

  }
} // end of namespace sstmac.

#endif
