/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_global_H_
#define SSTMAC_SHMEM_global_H_

#include <sstmac/common/ptr_type.h>

#include <sstmac/common/debug.h>
#include <sstmac/software/process/sstmac_global.h>

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {

      template<typename T>
        class global : public sstmac_global_builtin<T>
        {

        protected:

        public:

          typedef std::vector<global<T>*> global_map;

          static global_map* constructed_;

          explicit
          global() :
            sstmac_global_builtin<T> ()
          {
            if (!constructed_)
            {
              constructed_ = new global_map();
            }
            constructed_->push_back(this);
          }

          explicit
          global(T init) :
            sstmac_global_builtin<T> (init)
          {
            if (!constructed_)
            {
              constructed_ = new global_map();
            }
            constructed_->push_back(this);
          }

          global(const sstmac_global_builtin<T>& other)
          {
            sst_throw(illformederror, "copy constructor should never be called for primitive global");
          }

          template<typename U>
            T&
            operator =(const U& b)
            {
              T& myval = sstmac_global_builtin<T>::get_val();
              myval = (T) b;
              return myval;
            }

          T*
          operator&() const
          {
            return &sstmac_global_builtin<T>::get_val();
          }

        };

      template<typename T>
        std::vector<global<T>*>* global<T>::constructed_ = NULL;

    }
  }
}

typedef sstmac::sw::shmem::global<double*> shmem_double_ptr;
typedef sstmac::sw::shmem::global<double> shmem_double;
typedef sstmac::sw::shmem::global<long double*> shmem_long_double_ptr;
typedef sstmac::sw::shmem::global<long double> shmem_long_double;
typedef sstmac::sw::shmem::global<char*> shmem_char_ptr;
typedef sstmac::sw::shmem::global<char> shmem_char;
typedef sstmac::sw::shmem::global<long*> shmem_long_ptr;
typedef sstmac::sw::shmem::global<long> shmem_long;
typedef sstmac::sw::shmem::global<int*> shmem_int_ptr;
typedef sstmac::sw::shmem::global<int> shmem_int;
typedef sstmac::sw::shmem::global<long long> shmem_long_long;
typedef sstmac::sw::shmem::global<long long*> shmem_long_long_ptr;
typedef sstmac::sw::shmem::global<bool*> shmem_bool_ptr;
typedef sstmac::sw::shmem::global<bool> shmem_bool;
typedef sstmac::sw::shmem::global<short*> shmem_short_ptr;
typedef sstmac::sw::shmem::global<short> shmem_short;
typedef sstmac::sw::shmem::global<float*> shmem_float_ptr;
typedef sstmac::sw::shmem::global<float> shmem_float;

#endif
