/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include "shmem/memory/heap.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {

      debug heap::dbg_;

      heap::heap()
      {
        addr_count_ = 0;
        if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> shmem | heap", logger::red);
          logger::register_logger(&dbg_);
        }
      }

      symmetric::ptr
      heap::shmalloc(size_t size, bool real)
      {
        symmetric::ptr ret = symmetric::construct(size, addr_count_, real);

        if (real)
          mems_[ret->get_raw()] = ret;

        addrmap_[addr_count_] = ret;

        SSTMAC_DEBUG << "shmem::heap::shmalloc(" << addr_count_ << ", size="
              << size << ", real = " << real << ")\n";

        addr_count_++;

        return ret;
      }



      void
      heap::shfree(void* addr)
      {
        SSTMAC_DEBUG << "shmem::heap::shfree(" << (long) addr << ")\n";

        if (mems_.find(addr) == mems_.end())
        {
          sst_throw(ssterror, "shmem::heap::shfree - couldn't find object at addr %l", (long)addr);
        }
        symmetric::ptr s = mems_[addr];
        mems_.erase(addr);
        addrmap_.erase(s->get_shmem_addr());
      }

      symmetric::ptr
      heap::get_symmetric_by_raw(void* addr)
      {
        if (addr == NULL)
          return symmetric::ptr();

        if (mems_.find(addr) == mems_.end())
        {
          sst_throw(ssterror, "shmem::heap::get_symmetric_by_raw - can't find addr %l", addr);
        }

        return mems_[addr];
      }

      symmetric::ptr
      heap::get_symmetric_by_addr(long addr)
      {
        SSTMAC_DEBUG << "shmem::heap::get_symmetric_by_addr(" << addr << ")\n";
        if (addrmap_.find(addr) == addrmap_.end())
        {
          sst_throw(ssterror, "shmem::heap::get_symmetric_by_addr - can't find addr %l", addr);
        }

        return addrmap_[addr];
      }

      void
      heap::free_by_addr(long addr)
      {
        if (addrmap_.find(addr) == addrmap_.end())
        {
          sst_throw(ssterror, "shmem::heap::free_by_addr - can't find addr %l", addr);
        }

        addrmap_.erase(addr);
      }

    }
  }
}
