/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SHMEM_shared_heap_info_H_
#define SSTMAC_SHMEM_shared_heap_info_H_

#include "shmem/memory/heap.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {
      class shared_heap_info : public ptr_type
      {

      protected:
        shared_heap_info(heap::ptr h) :
          heap_(h)
        {

        }

        heap::ptr heap_;

      public:
        typedef boost::intrusive_ptr<shared_heap_info> ptr;

        static ptr
        construct(heap::ptr h)
        {
          return ptr(new shared_heap_info(h));
        }

        virtual
        ~shared_heap_info()
        {

        }

        /**
         * This can be determined, because in reality we would have pointers
         * to everyone else's heap address
         */
        bool
        addr_accessible(long addr){
          return heap_->find_symmetric_by_addr(addr);
        }

        virtual std::string
        to_string() const
        {
          return "shared_info_heap";
        }
      };
    }
  }
}

#endif
