/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SHMEM_server_H_
#define SSTMAC_SHMEM_server_H_

#include <sstmac/software/services/service.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>
#include "shmem/messages/transfer.h"
#include "shmem/messages/message.h"

namespace sstmac
{
  namespace sw
  {
    namespace shmem
    {
      class api;

      class server : public service, virtual public nocopy, public eventmodule
      {
        static debug dbg_;
        int aid_;

        lib_compute_memmove::ptr lib_mem_;

        // pe number, pointer to api
        boost::unordered_map<long, boost::intrusive_ptr<api> > apis_;

        std::queue<transfer::const_ptr> transfer_q_;
        bool q_server_busy_;

        server(int aid);

        // --- callbacks ----
        void
        tran_copy_complete(transfer::const_ptr tran, key::ptr k);

        void
        incoming_mem_done(transfer::const_ptr t);

        // ------------------

      public:
        typedef boost::intrusive_ptr<server> ptr;

        static ptr
        construct(int aid)
        {
          return ptr(new server(aid));
        }

        virtual
        ~server();

        virtual std::string
        to_string() const
        {
          return get_lib_name();
        }

        virtual void
        handle(const sst_message::ptr &msg);

        virtual service::ptr
        clone();

        virtual void
        start();

        virtual void
        add_pe(boost::intrusive_ptr<api> api);

        virtual void
        unblock_other_server(ptr serv, key::ptr k);

        virtual void
        unblock_api(key::ptr);

        void
        start_transfer(transfer::ptr tran, key::ptr k);

        void
        check_q();

        void
        send_tran_response(transfer::const_ptr t, payload::const_ptr pay,
            message::SHMEM_MSG_TYPE type);

      };
    }
  }
}
#endif /* server_H_ */
