/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_PTHREAD_IMPL_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_PTHREAD_IMPL_H_INCLUDED

#include <stdlib.h>

typedef long sstmac_pthread_t;
typedef long sstmac_pthread_attr_t;
typedef long sstmac_pthread_key_t;
typedef long sstmac_pthread_cond_t;
typedef long sstmac_pthread_mutex_t;
typedef long sstmac_pthread_once_t;
typedef long sstmac_pthread_cond_attr;
typedef long sstmac_pthread_mutex_attr;

#define SSTMAC_PTHREAD_THREADS_MAX 1000
#define SSTMAC_PTHREAD_KEYS_MAX 10000
#define SSTMAC_PTHREAD_STACK_MIN 16384
#define SSTAMC_PTHREAD_CREATE_DETACHED 0
#define SSTMAC_PTHREAD_CREATE_JOINABLE 1

#ifdef __cplusplus
extern "C"
{
#endif
  int
  SSTMAC_pthread_create(sstmac_pthread_t * thread,
      const sstmac_pthread_attr_t * attr, void *
      (*start_routine)(void *), void * arg);

  void
  SSTMAC_pthread_exit(void *retval);

  int
  SSTMAC_pthread_join(sstmac_pthread_t thread, void ** status);

  sstmac_pthread_t
  SSTMAC_pthread_self();

  int
  SSTMAC_pthread_equal(sstmac_pthread_t thread_1, sstmac_pthread_t thread_2);

  int
  SSTMAC_pthread_mutex_init(sstmac_pthread_mutex_t * mutex,
      const sstmac_pthread_mutex_attr *attr);

  int
  SSTMAC_pthread_mutex_destroy(sstmac_pthread_mutex_t * mutex);

  int
  SSTMAC_pthread_mutex_lock(sstmac_pthread_mutex_t * mutex);

  int
  SSTMAC_pthread_mutex_trylock(sstmac_pthread_mutex_t * mutex);

  int
  SSTMAC_pthread_mutex_unlock(sstmac_pthread_mutex_t * mutex);

  int
  SSTMAC_pthread_cond_init(sstmac_pthread_cond_t * cond,
      const sstmac_pthread_cond_attr *attr);

  int
  SSTMAC_pthread_cond_destroy(sstmac_pthread_cond_t * cond);

  int
  SSTMAC_pthread_cond_wait(sstmac_pthread_cond_t * cond,
      sstmac_pthread_mutex_t * mutex);

  int
  SSTMAC_pthread_cond_timedwait(sstmac_pthread_cond_t * cond,
      sstmac_pthread_mutex_t * mutex, const struct timespec * abstime);

  int
  SSTMAC_pthread_cond_signal(sstmac_pthread_cond_t * cond);

  int
  SSTMAC_pthread_cond_broadcast(sstmac_pthread_cond_t * cond);

  int
  SSTMAC_pthread_once(sstmac_pthread_once_t * once_init, void
  (*init_routine)(void));

  int
  SSTMAC_pthread_key_create(sstmac_pthread_key_t * key, void
  (* dest_routine)(void *));

  int
  SSTMAC_pthread_key_delete(sstmac_pthread_key_t key);

  int
  SSTMAC_pthread_setspecific(sstmac_pthread_key_t key, const void * pointer);

  void *
  SSTMAC_pthread_getspecific(sstmac_pthread_key_t key);

  void
  SSTMAC_pthread_cleanup_push(void
  (*routine)(void *), void *routine_arg);

  void
  SSTMAC_pthread_cleanup_pop(int execute);

  int
  SSTMAC_pthread_attr_init(sstmac_pthread_attr_t *attr);

  int
  SSTMAC_pthread_attr_destroy(sstmac_pthread_attr_t *attr);

  int
  SSTMAC_pthread_attr_getstacksize(sstmac_pthread_attr_t *attr, size_t * stacksize);

  int
  SSTMAC_pthread_attr_setstacksize(sstmac_pthread_attr_t *attr, size_t stacksize);

  int
  SSTMAC_pthread_attr_getdetachstate(const sstmac_pthread_attr_t *attr, int *state);

  int
  SSTMAC_pthread_attr_setdetachstate(sstmac_pthread_attr_t *attr, int state);

#ifdef __cplusplus
}
#endif

#endif
