/*  sstmac_upc.h                                                                    */

#ifndef SSTMAC_UPC_sstmac_upc_H_
#define SSTMAC_UPC_sstmac_upc_H_

#include <sstmac/sstmacro.h>

#include "sstmac_upc/api.h"

// Include the definition for the shared and shared_ptr classes.
#include <sstmac/software/api/upc/sstmac_upc/memory/shared.h>
#include <sstmac/software/api/upc/sstmac_upc/memory/shared_ptr.h>
#include <sstmac/software/api/upc/sstmac_upc/memory/lock.h>
#include <sstmac/software/api/upc/sstmac_upc/memory/global.h>

#include <string>

// Predefined Macros
#define __UPC__ 1
#define __UPC_VERSION__ 200505L

// Keeps the translator out of trouble...
struct upc_shared
{
  typedef sstmac::sw::upc::shared type;
};

template<typename T, size_t B>
  struct upc_shared_ptr
  {
    typedef sstmac::sw::upc::shared_ptr<T, B> type;
  };

sstmac::sw::upc::api::ptr
get_current_upc();

// Replaced built-in values with function calls:
size_t
sstmac_upc_threads(void);
size_t
sstmac_upc_mythread(void);
size_t
sstmac_upc_maxblocksize(void);

// For macro definitions that rely on the THREADS built-in
#define THREADS  sstmac_upc_threads()
#define MYTHREAD  sstmac_upc_mythread()

// Replaced UPC statements with function calls:
void
sstmac_upc_notify(int key);
void
sstmac_upc_notify(void);
void
sstmac_upc_wait(int key);
void
sstmac_upc_wait(void);
void
sstmac_upc_barrier(int key);
void
sstmac_upc_barrier(void);
void
sstmac_upc_fence(void);

// Replaced UPC expressions with function calls:
template<typename T, size_t B>
  size_t
  sstmac_upc_localsizeof(sstmac::sw::upc::shared_ptr<T, B> s_obj){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->localsizeof(s_obj);
}
template<typename T, size_t B>
  size_t
  sstmac_upc_blocksizeof(sstmac::sw::upc::shared_ptr<T, B> s_obj){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->blocksizeof(s_obj);
}
template<typename T, size_t B>
  size_t
  sstmac_upc_elemsizeof(sstmac::sw::upc::shared_ptr<T, B> s_obj){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->elemsizeof(s_obj);
}
size_t
sstmac_upc_localsizeof_s(std::string type_name);
size_t
sstmac_upc_blocksizeof_s(std::string type_name);
size_t
sstmac_upc_elemsizeof_s(std::string type_name);

// The remainder of the UPC runtime library.

void
upc_global_exit(int status);

// Shared array allocations.
sstmac::sw::upc::shared::ptr
upc_global_alloc(size_t nblocks, size_t nbytes);
sstmac::sw::upc::shared::ptr
upc_all_alloc(size_t nblocks, size_t nbytes);
sstmac::sw::upc::shared::ptr
upc_alloc(size_t nbytes);
sstmac::sw::upc::shared::ptr
upc_local_alloc(size_t nblocks, size_t nbytes);
template<typename T, size_t B>
  void
  upc_free(sstmac::sw::upc::shared_ptr<T, B>& ptr){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  upc->free<T, B>(ptr);
}

// Shared pointer properties.
template<typename T, size_t B>
  size_t
  upc_threadof(sstmac::sw::upc::shared_ptr<T, B> &ptr){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->threadof(ptr);
}
template<typename T, size_t B>
  size_t
  upc_phaseof(sstmac::sw::upc::shared_ptr<T, B> &ptr){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->phaseof(ptr);
}
template<typename T, size_t B>
  sstmac::sw::upc::shared_ptr<T, B>&
  upc_resetphase(sstmac::sw::upc::shared_ptr<T, B> &ptr){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->resetphase<T, B>(ptr);
}
template<typename T, size_t B>
  size_t
  upc_addrfield(sstmac::sw::upc::shared_ptr<T, B> &ptr){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  return upc->addrfield(ptr);
}

// ----- dumb global variable versions

template<typename T, size_t B>
  void
  upc_free(sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >& ptr){
  upc_free(ptr.get_val());
}

template<typename T, size_t B>
  size_t
  upc_threadof(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> > &ptr)
  {
    return upc_threadof(ptr.get_val());
  }
template<typename T, size_t B>
  size_t
  upc_phaseof(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> > &ptr)
  {
    return upc_phaseof(ptr.get_val());
  }
template<typename T, size_t B>
  sstmac::sw::upc::shared_ptr<T, B>&
  upc_resetphase(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> > &ptr)
  {
    return upc_resetphase(ptr.get_val());
  }
template<typename T, size_t B>
  size_t
  upc_addrfield(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >&ptr)
  {
    return upc_addrfield(ptr.get_val());
  }

size_t
upc_affinitysize(size_t totalsize, size_t nbytes, size_t threadid);

// Locking type defined in lock.h
typedef sstmac::sw::upc::lock upc_lock_t;

// Lock allocations.
upc_lock_t *
upc_global_lock_alloc();
upc_lock_t *
upc_all_lock_alloc();
void
upc_lock_free(upc_lock_t *ptr);

// Locking/unlocking.
void
upc_lock(upc_lock_t *ptr);
int
upc_lock_attempt(upc_lock_t *ptr);
void
upc_unlock(upc_lock_t *ptr);

// Shared memory copies.
template<typename T, size_t B>
  void
  upc_memcpy(sstmac::sw::upc::shared_ptr<T, B>& dst,
      sstmac::sw::upc::shared_ptr<T, B>& src, size_t n){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  upc->memcpy<T, B>(dst, src, n);
}
template<typename T, size_t B>
  void
  upc_memget(void *dst, sstmac::sw::upc::shared_ptr<T, B>& src, size_t n){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  upc->memget<T, B>(dst, src, n);
}
template<typename T, size_t B>
  void
  upc_memput(sstmac::sw::upc::shared_ptr<T, B>& dst, const void *src, size_t n){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  upc->memput<T, B>(dst, src, n);
}
template<typename T, size_t B>
  void
  upc_memset(sstmac::sw::upc::shared_ptr<T, B>& dst, int c, size_t n){
  sstmac::sw::upc::api::ptr upc = get_current_upc();
  upc->memset<T, B>(dst, c, n);
}

//-----dumb global variable version
template<typename T, size_t B>
  void
  upc_memcpy(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >& dst,
      sstmac::sw::upc::shared_ptr<T, B>& src, size_t n)
  {
    upc_memcpy(dst.get_val(), src, n);
  }
template<typename T, size_t B>
  void
  upc_memget(
      void *dst,
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >& src,
      size_t n)
  {
    upc_memget(dst, src.get_val(), n);
  }
template<typename T, size_t B>
  void
  upc_memput(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >& dst,
      const void *src, size_t n)
  {
    upc_memput(dst.get_val(), src, n);
  }
template<typename T, size_t B>
  void
  upc_memset(
      sstmac::sw::sstmac_global_builtin<sstmac::sw::upc::shared_ptr<T, B> >& dst,
      int c, size_t n)
  {
    upc_memset(dst.get_val(), c, n);
  }

// UPC flag type (upc_flag_t) moved to upc_collective.h

//#include "sstmac_upc/sstmac_upc.cc"

#endif
