/*  shared_heap.h                                                            */

#ifndef SSTMAC_UPC_shared_heap_H_
#define SSTMAC_UPC_shared_heap_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>

#include "sstmac_upc/memory/shared.h"
#include "sstmac_upc/memory/lock.h"

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            class shared_heap : public ptr_type
            {
                private:
                    typedef std::vector<shared::ptr> sharedlisttype;
                    sharedlisttype shared_list_;
                    
                    typedef std::vector<lock*> locklisttype;
                    locklisttype lock_list_;
                    
                    long shared_count_;
                    long lock_count_;
                    
                    static debug dbg_;
                    
                    shared_heap();
                    
                public:
                    typedef boost::intrusive_ptr<shared_heap> ptr;
                    
                    static ptr
                    construct()
                    {
                        return ptr(new shared_heap());
                    }
                    
                    virtual
                    ~shared_heap()
                    {
                    }
                    
                    virtual std::string
                    to_string() const
                    {
                        return "shared_heap";
                    }
                    
                    void
                    add_shared(shared::ptr sobj);
                    
                    void
                    delete_shared(shared::ptr sobj);
                    
                    bool
                    contains_shared(shared::ptr sobj);
                    
                    void
                    add_lock(lock *lck);
                    
                    void
                    delete_lock(lock *lck);
                    
            };
        }
    }
}

#endif
