/*  put_transfer.h                                                           */

#ifndef SSTMAC_UPC_put_transfer_H_
#define SSTMAC_UPC_put_transfer_H_

#include "sstmac_upc/messages/transfer.h"
#include "sstmac_upc/memory/shared.h"

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            template <typename T>
            class put_transfer : public transfer
            {

                protected:

                    const T* val_;
                    shared::ptr shobj_;
                    size_t baseaddr_;
                    size_t phase_;
                    size_t size_;
                    long from_thread_;
                    long to_thread_;
                    bool real_;

                    T * alloc_;
                    
                    put_transfer(shared::ptr shobj, T* val, size_t size, size_t baseaddr, size_t phase, long fromthread, long tothread, bool real) :
                        shobj_(shobj), size_(size), baseaddr_(baseaddr), phase_(phase), from_thread_(fromthread), to_thread_(tothread), real_(real)
                    {
                        if (real_ && size_ > 0)
                        {
                            alloc_ = (T*)malloc(size_ * sizeof(T));
                            memcpy(alloc_, val, size_ * sizeof(T));
                        }
                        val_ = alloc_;
                    }

                public:
                    typedef boost::intrusive_ptr<put_transfer> ptr;

                    static ptr
                    construct(shared::ptr shobj, T val, size_t baseaddr, size_t phase, long fromthread, long tothread)
                    {
                        return ptr(new put_transfer(shobj, &val, 1, baseaddr, phase, fromthread, tothread, true));
                    }
                    
                    static ptr
                    construct(shared::ptr shobj, T val, size_t baseaddr, size_t phase, long fromthread, long tothread, bool real)
                    {
                        return ptr(new put_transfer(shobj, &val, 1, baseaddr, phase, fromthread, tothread, real));
                    }

                    static ptr
                    construct(shared::ptr shobj, T *val, size_t size, size_t baseaddr, size_t phase, long fromthread, long tothread)
                    {
                        return ptr(new put_transfer(shobj, val, size, baseaddr, phase, fromthread, tothread, true));
                    }
                    
                    static ptr
                    construct(shared::ptr shobj, T *val, size_t size, size_t baseaddr, size_t phase, long fromthread, long tothread, bool real)
                    {
                        return ptr(new put_transfer(shobj, val, size, baseaddr, phase, fromthread, tothread, real));
                    }

                    virtual std::string
                    to_string() const
                    {
                        return "put_transfer";
                    }

                    virtual
                    ~put_transfer()
                    {
                        if (real_) free(alloc_);
                    }

                    virtual long
                    size_bytes() const
                    {
                        return sizeof(T) * size_;
                    }

                    virtual long
                    get_remote_thread() const
                    {
                        return to_thread_;
                    }

                    virtual long
                    get_local_thread() const
                    {
                        return from_thread_;
                    }

                    virtual TRANSFER_TYPES
                    get_trans_type() const
                    {
                        return PUT;
                    }

                    virtual void
                    execute(transfer_api::ptr a) const
                    {
                        if (real_ && a->check_shared_heap(shobj_))
                        {
                            void* r = shobj_->get_offset_raw(baseaddr_, to_thread_, phase_, sizeof(T));

                            SSTMAC_DEBUG << "put_transfer: executing on thread " << to_thread_
                                    << " with raw " << (long) r << " with base address "
                                    << baseaddr_ << " and phase " << phase_ << "\n";
                            T* casted = (T*) r;

                            for (int i = 0; i < size_; i++)
                            {
                                SSTMAC_DEBUG << " setting shared object " << casted[i] << " to " << val_[i] << "\n";
                                casted[i] = val_[i];
                            }
                        }

                        a->send_put_complete(const_ptr(this));
                    }
                
            };
            
        }
    }
}

#endif
