/*  transfer.h                                                               */

#ifndef SSTMAC_UPC_transfer_H_
#define SSTMAC_UPC_transfer_H_

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>

#include "sstmac_upc/transfer_api.h"

namespace sstmac
{
    namespace sw
    {
        namespace upc
        {
            //  class api;
            
            class transfer : public ptr_type
            {
                protected:
                    static long tcount_;
                    long tid_;

                    static debug dbg_;

                    transfer()
                    {
                        tid_ = tcount_++;

                        if (!dbg_.is_real())
                        {
                            dbg_ = debug("<debug> upc | transfer", logger::magenta);
                            logger::register_logger(&dbg_);
                        }
                    }
                    
                public:
                    typedef boost::intrusive_ptr<transfer> ptr;
                    typedef boost::intrusive_ptr<const transfer> const_ptr;

                    enum TRANSFER_TYPES
                    {
                        GET, PUT
                    };

                    virtual
                    ~transfer()
                    {
                    }

                    virtual long
                    get_tid() const
                    {
                        return tid_;
                    }

                    virtual long
                    size_bytes() const = 0;

                    virtual long
                    get_remote_thread() const = 0;

                    virtual long
                    get_local_thread() const = 0;

                    virtual void
                    execute(transfer_api::ptr a) const = 0;

                    virtual TRANSFER_TYPES
                    get_trans_type() const = 0;
                    
            };
         
        }
   }
}

#endif
