/*  sstmac_upc.cc                                                            */

#include "sstmac_upc.h"

#include <sstmac/software/process/os_api.h>
//#include "sstmac_upc/memory/shared_ptr.h"
//#include "sstmac_upc/memory/lock.h"
#include "sstmac_upc/app.h"
#include "sstmac_upc/api.h"

using namespace sstmac;
using namespace sstmac::sw;
/*
 * ---------------------------------------------------------------------------
 * --------------- Utility functions to convert to sstmac --------------------
 * ---------------------------------------------------------------------------
 */
upc::api::ptr
get_current_upc()
{
    thread::ptr t = OS_API::get_current_thread();
    upc::api::ptr a = t->get_api<upc::api> ();
    a->setup();
    return a;
}

/*
 * ---------------------------------------------------------------------------
 * --------------- Now for UPC implementation functions ----------------------
 * ---------------------------------------------------------------------------
 */

size_t
sstmac_upc_threads(void)
{
    upc::api::ptr upc = get_current_upc();
    return upc->threads();
}

size_t
sstmac_upc_mythread(void)
{
    upc::api::ptr upc = get_current_upc();
    return upc->mythread();
}

size_t
sstmac_upc_maxblocksize(void)
{
    upc::api::ptr upc = get_current_upc();
    return upc->maxblocksize();
}

/*
 * ---------------------------------------------------------------------------
 * Synchronization
 * ---------------------------------------------------------------------------
 */

void
sstmac_upc_notify(int key)
{
    upc::api::ptr upc = get_current_upc();
    upc->notify(key);
}

void
sstmac_upc_notify(void)
{
    upc::api::ptr upc = get_current_upc();
    upc->notify();
}

void
sstmac_upc_wait(int key)
{
    upc::api::ptr upc = get_current_upc();
    upc->wait(key);
}

void
sstmac_upc_wait(void)
{
    upc::api::ptr upc = get_current_upc();
    upc->wait();
}

void
sstmac_upc_barrier(int key)
{
    upc::api::ptr upc = get_current_upc();
    upc->barrier(key);
}

void
sstmac_upc_barrier(void)
{
    upc::api::ptr upc = get_current_upc();
    upc->barrier();
}

void
sstmac_upc_fence(void)
{
    upc::api::ptr upc = get_current_upc();
    upc->fence();
}

/*
 * ---------------------------------------------------------------------------
 * Shared 'sizeof' style operators
 * ---------------------------------------------------------------------------
 */



size_t
sstmac_upc_localsizeof_s(std::string type_name)
{
    upc::api::ptr upc = get_current_upc();
    return upc->localsizeof(type_name);
}

size_t
sstmac_upc_blocksizeof_s(std::string type_name)
{
    upc::api::ptr upc = get_current_upc();
    return upc->blocksizeof(type_name);
}

size_t
sstmac_upc_elemsizeof_s (std::string type_name)
{
    upc::api::ptr upc = get_current_upc();
    return upc->elemsizeof(type_name);
}

/*
 * ---------------------------------------------------------------------------
 * End UPC execution
 * ---------------------------------------------------------------------------
 */

void
upc_global_exit(int status)
{
    upc::api::ptr upc = get_current_upc();
    upc->global_exit(status);
}

/*
 * ---------------------------------------------------------------------------
 * Shared memory allocation
 * ---------------------------------------------------------------------------
 */

upc::shared::ptr
upc_global_alloc(size_t nblocks, size_t nbytes)
{
    upc::api::ptr upc = get_current_upc();
    return upc->global_alloc(nblocks, nbytes);
}

upc::shared::ptr
upc_all_alloc(size_t nblocks, size_t nbytes)
{
    upc::api::ptr upc = get_current_upc();
    return upc->all_alloc(nblocks, nbytes);
}

upc::shared::ptr
upc_alloc(size_t nbytes)
{
    upc::api::ptr upc = get_current_upc();
    return upc->alloc(nbytes);
}

// Deprecated...
upc::shared::ptr
upc_local_alloc(size_t nblocks, size_t nbytes)
{
    upc::api::ptr upc = get_current_upc();
    return upc->alloc(nblocks * nbytes);
}


/*
 * ---------------------------------------------------------------------------
 * Shared pointer dissection
 * ---------------------------------------------------------------------------
 */



size_t
upc_affinitysize(size_t totalsize, size_t nbytes, size_t threadid)
{
    upc::api::ptr upc = get_current_upc();
    return upc->affinitysize(totalsize, nbytes, threadid);
}

/*
 * ---------------------------------------------------------------------------
 * Shared lock functions
 * ---------------------------------------------------------------------------
 */

upc_lock_t *
upc_global_lock_alloc()
{
    upc::api::ptr upc = get_current_upc();
    return upc->global_lock_alloc();
}

upc_lock_t *
upc_all_lock_alloc()
{
    upc::api::ptr upc = get_current_upc();
    return upc->all_lock_alloc();
}

void
upc_lock_free(upc_lock_t *ptr)
{
    upc::api::ptr upc = get_current_upc();
    upc->lock_free(ptr);
}

void
upc_lock(upc_lock_t *ptr)
{
    upc::api::ptr upc = get_current_upc();
    upc->lock_wait(ptr);
}

int
upc_lock_attempt(upc_lock_t *ptr)
{
    upc::api::ptr upc = get_current_upc();
    return upc->lock_attempt(ptr);
}

void
upc_unlock(upc_lock_t *ptr)
{
    upc::api::ptr upc = get_current_upc();
    upc->unlock(ptr);
}

/*
 * ---------------------------------------------------------------------------
 * Explicit shared memory operations
 * ---------------------------------------------------------------------------
 */



