/*  sstmac_upc_collective.h                                                         */

#ifndef SSTMAC_UPC_sstmac_upc_collective_H_
#define SSTMAC_UPC_sstmac_upc_collective_H_

#include "sstmac_upc/memory/shared_ptr.h"

#define __UPC_COLLECTIVE__ 1

enum upc_flag
{
    UPC_IN_NOSYNC = 0x02,
    UPC_IN_MYSYNC = 0x01,
    UPC_IN_ALLSYNC = 0x00,
    UPC_OUT_NOSYNC = 0x20,
    UPC_OUT_MYSYNC = 0x10,
    UPC_OUT_ALLSYNC = 0x00
};
typedef int upc_flag_t;

enum upc_op
{
    UPC_ADD,
    UPC_MULT,
    UPC_AND,
    UPC_OR,
    UPC_XOR,
    UPC_LOGAND,
    UPC_LOGOR,
    UPC_MIN,
    UPC_MAX,
    UPC_FUNC,
    UPC_NONCOMM_FUNC
};
typedef int upc_op_t;

// Relocalization
template <typename T, size_t B, typename U, size_t C>
void upc_all_broadcast(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<U, C>& src, size_t nbytes, upc_flag_t flags);
template <typename T, size_t B>
void upc_all_scatter(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, size_t nbytes, upc_flag_t flags);
template <typename T, size_t B>
void upc_all_gather(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, size_t nbytes, upc_flag_t flags);
template <typename T, size_t B>
void upc_all_gather_all(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, size_t nbytes, upc_flag_t flags);
template <typename T, size_t B>
void upc_all_exchange(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, size_t nbytes, upc_flag_t flags);
template <typename T, size_t B>
void upc_all_permute(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, sstmac::sw::upc::shared_ptr<int, 1>& perm, size_t nbytes, upc_flag_t flags);

// Computation
template <typename T, size_t B>
void upc_all_reduce(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, upc_op_t op, size_t nelems, size_t blk_size, T (*func)(T, T), upc_flag_t flags);
template <typename T, size_t B>
void upc_all_prefix_reduce(sstmac::sw::upc::shared_ptr<T, B>& dst, sstmac::sw::upc::shared_ptr<T, B>& src, upc_op_t op, size_t nelems, size_t blk_size, T (*func)(T, T), upc_flag_t flags);

//#include "sstmac_upc/sstmac_upc_collective.cc"

#endif
