/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORMSG_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORMSG_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>

#include <sstmac/common/ptr_type.h>

namespace sstmac { namespace sw {

  /**
   * Base type for messages that get sent between peers in a
   * distributed actor-based application.
   */
  class actormsg : public ptr_type { //: public sst_message {
  public:

    virtual std::string to_string() const{
            return "actormsg";
          }

    typedef boost::intrusive_ptr<actormsg> ptr;

    /// Goodbye.
    virtual ~actormsg() {}

    /// Create a copy of this object.  Required because we don't want
    /// people accidentally creating shared objects (which aren't supposed
    /// to exist in ACTORs).
    virtual actormsg::ptr actorclone() const = 0;
    virtual int get_kind() const = 0;
  };

} } // end of namespace sstmac

#endif
