/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_ACTOR_SIMPLEMAILSERVER_DATA_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_ACTOR_SIMPLEMAILSERVER_DATA_H_INCLUDED

#include <sstmac/software/libraries/actor/simplemailserver.h>
#include <sstmac/hardware/network/network_message.h>

namespace sstmac { namespace sw
{

  /**
   * Payload sent by a simplemailserver.
   */
  class simplemailserver::data : 
    public library_message,
    public hw::network_message
  {

    /// WHAT -- is your name?
    actorid source_;
    /// WHAT -- is your quest?
    actorid dest_;
    /// WHAT -- is your weight?
    int64_t bytes_;
    /// Carry on.
    actormsg::ptr payload_;

    /// Creationism.
    data(const std::string& libn, const actorid src,
        const actorid dst, int64_t mlen,
        const actormsg::ptr& pload) :
        sst_message(sst_message::ACTOR),
        library_message(libn),
        source_(src), dest_(dst), bytes_(mlen), payload_(pload)
    {
    }

    /// Creationism.
    data(const std::string& libn, 
        const actorid &src,
        const actorid &dst, int64_t mlen,
        const actormsg::ptr& pload, MESSAGE_TYPES ty) :
        sst_message(ty),
        library_message(libn),
        source_(src), dest_(dst), bytes_(mlen), payload_(pload)
    {
    }

  public:

    typedef boost::intrusive_ptr<data> ptr;

    /// And off we go.
    static data::ptr
    construct(const std::string& libn, 
        const actorid &src,
        const actorid &dst, int64_t mlen,
        const actormsg::ptr& pload)
    {
      data::ptr rv(new data(libn, src, dst, mlen, pload));
      return rv;
    }

    /// And off we go.
    static data::ptr
    construct(const std::string& libn, 
        const actorid &src,
        const actorid &dst, int64_t mlen,
        const actormsg::ptr& pload, MESSAGE_TYPES ty)
    {
      data::ptr rv(new data(libn, src, dst, mlen, pload, ty));

      return rv;
    }

    /// Goodbye.
    virtual
    ~data() throw ()
    {
    }

    const actorid&
    source() const
    {
      return source_;
    }
    const actorid&
    dest() const
    {
      return dest_;
    }
    virtual long
    get_byte_length() const
    {
      return bytes_;
    }
    actormsg::ptr
    payload() const
    {
      return payload_;
    }

    /// required by networkdata.
    const taskid&
    source_task() const
    {
      static const taskid unbound(taskid::unbound_task);
      return unbound;
    }

    /// required by networkdata.
    const taskid&
    dest_task() const
    {
      static const taskid unbound(taskid::unbound_task);
      return unbound;
    }

    virtual sst_message::ptr
    clone() const
    {
      return data::construct(lib_name_, source_, dest_, bytes_, payload_);
    }

    virtual sst_message::ptr
    clone(MESSAGE_TYPES ty) const
    {
      return data::construct(lib_name_, source_, dest_, bytes_, payload_, ty);
    }

    /// This method is required by the networkdata.
    /// We know that we don't reuse data, so we just return this object.
    /* boost::shared_ptr<networkdata> complement() const {
     return self_.lock();
     }*/
  };

} } // end of namespace sstmac

#endif
