/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_ACTOR_SIMPLEMBOXAPP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_ACTOR_SIMPLEMBOXAPP_H_INCLUDED

#include <sstmac/software/process/app.h>
#include <sstmac/software/libraries/actor/simplembox.h>
#include <sstmac/software/libraries/actor/simplemailserver-mbox.h>

namespace sstmac { namespace sw
{


  /**
   * Base type for applications that do simple mailbox-based messaging.
   */
  class simplemboxapp : public app
  {
  public:
    typedef boost::intrusive_ptr<simplemboxapp> ptr;

  private:
    /// Keep the mailbox here.
    boost::intrusive_ptr<simplembox> mbox_;

    bool initialized_;

  protected:

    simplemboxapp(softwareid id);

    simplemboxapp(const sim_parameters::ptr& params) : app(params) {}

    /// Access our mailbox.
    const simplembox::ptr
    mbox() const;

    virtual void
    compute(timestamp &t, int threads);

public:
  /// Access our mailbox.
  simplembox::ptr
  mbox();

  /// Goodbye.
  virtual
  ~simplemboxapp();

  virtual void
  kill(){}

  };

} } // end of namespace sstmac

#endif
