/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_INST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_INST_H_INCLUDED

#include <sstmac/software/libraries/compute/libcompute.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/messages/eventdata.h>

//these are the default instruction labels

namespace sstmac
{
  namespace sw
  {

    class lib_compute_inst :
        virtual public libcompute
    {
    public:
      typedef boost::intrusive_ptr<lib_compute_inst> ptr;
      // ------- constructor / boost stuff -------------//

    protected:
      lib_compute_inst(softwareid id);

      lib_compute_inst(const std::string& id);

    public:

      virtual
      ~lib_compute_inst(){ }

      // --------------------------------------//

    public:

      void
      compute_inst(const eventdata::ptr &msg);

      static lib_compute_inst::ptr
      construct(softwareid id)
      {
        return ptr(new lib_compute_inst(id));
      }

      static lib_compute_inst::ptr
      construct(std::string id)
      {
        return ptr(new lib_compute_inst(id));
      }

      virtual bool
      supported() const
      {
        return os_->kernel_supported(AMI::AMI_COMP_INSTR);
      }

    protected:

      static debug dbg_;
    };

  }
} //end of namespace sstmac

#endif
