/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED

#include <sstmac/software/libraries/compute/libcompute.h>
#include <sstmac/common/eventcallback.h>

namespace sstmac
{
  namespace sw
  {

    class lib_compute_memmove :
        virtual public libcompute
    {
      // ------- constructor / boost stuff -------------//

    private:
        void init();

    protected:
      lib_compute_memmove(softwareid id);

      lib_compute_memmove(const std::string& libname);

      void do_access(
              long bytes,
              AMI::AMI_COMP_FUNC func,
              long srcthread,
              long dstthread,
              const eventcallback::ptr& cb
      );

      static const long unlimited_page_size = -1;
      static const long default_page_size = unlimited_page_size;

      eventcallback::ptr compute_cb_;

      eventcallback::ptr nic_cb_;

      eventcallback::ptr finalize_cb_;

      eventcallback::ptr rdma_cb_;

      void compute(const sst_message::ptr& msg);

      void finish(const sst_message::ptr& msg);

    public:
      typedef boost::intrusive_ptr<lib_compute_memmove> ptr;

      static key::category key_category;

      virtual
      ~lib_compute_memmove()
      {
      }

      static ptr
      construct(softwareid id)
      {
        return ptr(new lib_compute_memmove(id));
      }

      static ptr
      construct(const std::string& libname)
      {
        return ptr(new lib_compute_memmove(libname));
      }
      // --------------------------------------//

      virtual void
      unregister_all_libs();

      static std::string
      get_name(const std::string& n)  {
        return "computelibmemmove" + n;
      }

      virtual bool supported() const;

      void reserve_cpu(const sst_message::ptr& msg);

      void finish_rdma(const sst_message::ptr& msg);

      void consume_params(const sim_parameters::ptr &params);

    public:

      void read(long bytes,
                const eventcallback::ptr& cb = eventcallback::ptr());

      void read(long bytes, long destthread,
                const eventcallback::ptr& cb = eventcallback::ptr());

      void read(long bytes, long srcthread, long destthread,
                const eventcallback::ptr& cb = eventcallback::ptr());

      void write(long bytes,
                 const eventcallback::ptr& cb = eventcallback::ptr());

      void write(long bytes, long srcthread, long destthread,
                 const eventcallback::ptr& cb = eventcallback::ptr());

      void write(long bytes, long destthread,
                 const eventcallback::ptr& cb = eventcallback::ptr());

      void copy(long bytes,
                const eventcallback::ptr& cb = eventcallback::ptr());

      void copy(long bytes, long srcthread, long destthread,
                const eventcallback::ptr& cb = eventcallback::ptr());

      void read(long bytes, const taskid& srctask,
                     const eventcallback::ptr& cb = eventcallback::ptr());

      void write(long bytes, const taskid& dsttask,
                     const eventcallback::ptr& cb = eventcallback::ptr());

    protected:

      static debug dbg_;

      double loop_unroll_;

      bool prefetch_;

      double pipeline_efficiency_;

    };

  }
} //end of namespace sstmac

#endif
