/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_sleep.h>
#include <sstmac/software/process/graphviz.h>
#include <sstmac/software/process/key.h>

namespace sstmac { namespace sw
{

  debug lib_sleep::dbg_;
  key::category lib_sleep::key_category("Sleep");

  lib_sleep::lib_sleep(softwareid id)
      : library()
  {
    if (!dbg_.is_real())
      {
        dbg_ = debug("<debug> lib_sleep");
        logger::register_logger(&dbg_);
      }

    libname_ = "sleeplib" + id.to_string();
    key_cat_ = lib_sleep::key_category;
  }

  lib_sleep::lib_sleep(const std::string& id)
      : library()
  {
    if (!dbg_.is_real())
      {
        dbg_ = debug("<debug> lib_sleep");
        logger::register_logger(&dbg_);
      }

    libname_ = id;
    key_cat_ = lib_sleep::key_category;
  }


  void
  lib_sleep::sleep(timestamp time)
  {
      SSTMACBacktrace("Sleep");
      if (supported())
      {
        sst_message::ptr cmsg = sleep_message::construct(get_lib_name(), time);

        SSTMAC_DEBUG << "lib_sleep: executing compute kernel \n";
        os_->execute_kernel(AMI::AMI_COMP_SLEEP, cmsg);

        SSTMAC_DEBUG << "lib_sleep: blocking \n";
        wait_this(cmsg);
      }
    else
      {
        throw valueerror(
            "lib_sleep requires the presence of a processor model which can sleep");
      }
  }

} } //end of namespace sstmac
