/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_SLEEP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_SLEEP_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>

namespace sstmac
{
  namespace sw
  {

    class lib_sleep : public library
    {
      // ------- constructor / boost stuff -------------//

    protected:

      lib_sleep(softwareid id);

      lib_sleep(const std::string& id);

    public:
      static key::category key_category;

      typedef boost::intrusive_ptr<lib_sleep> ptr;

      virtual
      ~lib_sleep()
      {
      }

      static ptr
      construct(softwareid id)
      {
        return ptr(new lib_sleep(id));
      }

      static ptr
      construct(const std::string& id)
      {
        return ptr(new lib_sleep(id));
      }
      // --------------------------------------//

    public:

      void
      sleep(timestamp time);

      virtual bool
      supported() const
      {
        return os_->kernel_supported(AMI::AMI_COMP_SLEEP);
      }

    protected:

      static debug dbg_;
    };

  }
} //end of namespace sstmac

#endif
