#include <sstmac/software/libraries/compute/mem_access.h>

DeclareSerializable(sstmac::sw::mem_access);

namespace sstmac {
    namespace sw {

      static nodeaddress::ptr null_addr = 0;

      mem_access::mem_access(long bytes,
                 const std::string& libn,
                 AMI::AMI_COMP_FUNC access_type,
                 long srcth, long destth) :
          sst_message(COMPUTE),
          eventdata(libn),
          bytes_(bytes),
          func_(access_type),
          srcthread_(srcth),
          dstthread_(destth)
      {
      }

      mem_access::ptr
      mem_access::construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long thr)
      {
        return ptr(new mem_access(bytes, libn, access_type, thr, thr));
      }

      mem_access::ptr
      mem_access::construct(long bytes,
                const std::string& libn,
                AMI::AMI_COMP_FUNC access_type,
                long srcth, long dstth)
      {
        return ptr(new mem_access(bytes, libn, access_type, srcth, dstth));
      }


      AMI::AMI_COMP_FUNC
      mem_access::get_function() const
      {
          return func_;
      }

      long
      mem_access::get_byte_length() const
      {
        return bytes_;
      }

      long
      mem_access::source_thread(const sw::operatingsystem::ptr& os) const
      {
          return srcthread_;
      }

      long
      mem_access::dest_thread(const sw::operatingsystem::ptr& os) const
      {
          return dstthread_;
      }

      void
      mem_access::set_page_event_callback(const event_step_callback::ptr &cb)
      {
          if (page_event_cb_){
              sst_throw(valueerror, "ERROR: mem_access::set_page_event_callback: mem_access already has page callback");
          }
          page_event_cb_ = cb;
      }

      event_step_callback::ptr
      mem_access::pop_page_event_callback()
      {
          event_step_callback::ptr tmp = page_event_cb_;
          page_event_cb_ = 0;
          return tmp;
      }

      nodeaddress::ptr
      mem_access::toaddr() const {
        return nodeaddress::ptr();
      }

      nodeaddress::ptr
      mem_access::fromaddr() const {
        return nodeaddress::ptr();
      }

    }
}
