/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <iostream>
#include <sstmac/software/process/app.h>
#include <sstmac/common/messages/eventdata.h>

using namespace sstmac;
using namespace sstmac::sw;

app::ptr
get_current_app()
{
  thread::ptr t = OS_API::get_current_thread();
  app::ptr a = t->get_parent_app();
  if (!a){
      throw sstmac::ssterror("sstmac_compute::get_current_app: no parent application");
  }
  return a;
}

extern "C" void
SSTMAC_compute(double seconds)
{
  app::ptr app = get_current_app();
  app->compute(timestamp(seconds));
}

extern "C" void
SSTMAC_compute_block_read(long long bytes)
{
  app::ptr app = get_current_app();
  app->compute_block_read(bytes);
}

extern "C" void
SSTMAC_compute_loop(long long from, long long to, long long numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop(from, to, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop2(long long from1, long long to1, long long from2, long long to2, long long numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop2(from1, to1, from2, to2, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop3(long long from1, long long to1, long long from2, int to2, long long from3,
    long long to3, long long numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop3(from1, to1, from2, to2, from3, to3, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop4(long long from1, long long to1, long long from2, long long to2, long long from3,
    long long to3, int from4, long long to4, long long numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop4(from1, to1, from2, to2, from3, to3, from4, to4, numlines, app->tid().id);
}
