/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_NPPFS_SERVER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_NPPFS_SERVER_H_INCLUDED

#include <sstmac/software/services/service.h>
#include <sstmac/software/libraries/filesystem/file_descriptor.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>
#include <sstmac/software/libraries/filesystem/openfile.h>
#include <sstmac/software/libraries/filesystem/disk_message.h>
namespace sstmac
{
  namespace sw
  {

    class nppfs_server : public service
    {
    protected:
      nppfs_server(std::string fname, bool ismeta) :
          fname_(fname), is_meta_(ismeta), ACCESS_ID(0)
      {
        if(is_meta_)
          read_disk_file();

        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> nppfs | nppfs_server");
            logger::register_logger(&dbg_);
          }

        libname_ = "nppfs_server";
       // compute_counter_ = 0;
      }

      std::string fname_;
      static debug dbg_;
      bool is_meta_;  //am i the metadata server, or just a regular disk server

      void
      read_disk_file();

      long ACCESS_ID;
     // long compute_counter_;
      
      boost::unordered_map<long, openfile::ptr> pending_;
      boost::unordered_map<long, nodeaddress::ptr> sources_;
      boost::unordered_map<long, std::string> fromlibs_;
      boost::unordered_map<long, bool> needs_resp_;
      boost::unordered_map<std::string, file_descriptor::ptr> files_;
     // boost::unordered_map<long, disk_message::ptr> computing_;

    public:
      typedef boost::intrusive_ptr<nppfs_server> ptr;

      static ptr
      construct(std::string fname, bool ismeta)
      {
        return ptr(new nppfs_server(fname, ismeta));
      }

      virtual void
      start();

      virtual service::ptr
      clone();

      virtual void
      handle(const sst_message::ptr &msg);

      virtual
      ~nppfs_server()
      {
      }

      virtual std::string
      to_string() const {
        return "nppfs_server";
      }
    };

  }
} /* namespace sstmac */
#endif /* NPPFS_SERVER_H_ */
