/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_OPENFILE_H_
#define SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_OPENFILE_H_

#include <sstmac/common/ptr_type.h>

#include <sstmac/hardware/disk/vesper/disk.h>

namespace sstmac
{
  namespace sw
  {

    class openfile : public ptr_type
    {
    protected:
      openfile(std::string name, long s) : name_(name), sector_(s), offset_(0), open_(true)
        {

        }
    public:
      typedef boost::intrusive_ptr<openfile> ptr;

      static ptr
      construct(std::string name, long s)
      {
        return ptr(new openfile(name, s));
      }

      virtual std::string
      to_string() const
      {
        return "openfile";
      }

      virtual
      ~openfile()
      {
      }

      long
      get_sector() const
      {
        return sector_;
      }

      void
      close()
      {
        open_ = false;
      }

      long
      get_offset() const
      {
        return offset_;
      }

      long
      get_offset_sectors() const
      {
        return offset_ / (sstmac::hw::Disk::blockSize * 1000000);
      }

      void
      seek(long bytes)
      {
        offset_ += bytes;
        update_max_access(offset_);
      }

      virtual void
      update_max_access(long max){

      }

      void
      clear_offset()
      {
        offset_ = 0;
      }

      bool
      is_open() const
      {
        return open_;
      }

      std::string
      name() const {
        return name_;
      }



    protected:
      std::string name_;
      long sector_;
      long offset_; // in bytes
      bool open_;

    };

  }
} /* namespace sstmac */
#endif /* OPENFILE_H_ */
