#include <sstmac/software/libraries/libwaiters.h>

namespace sstmac {
    namespace sw {

void
callback_lib_waiter::wait_msg(const sst_message::ptr &msg)
{
    if (!msg->has_callback())
    {
      sst_throw(illformederror,
          "Callback waiter for library received msg %s without callback",
          msg->to_string().c_str());
    }
}

void
callback_lib_waiter::incoming_msg(const sst_message::ptr &msg)
{
    eventhandler::ptr cb = msg->pop_callback();
    cb->handle(msg);
}

serial_single_msg_lib_waiter::serial_single_msg_lib_waiter(key::category cat)
    : serial_blocking_lib_waiter(cat)
{
}

void
serial_single_msg_lib_waiter::wait_msg(const sst_message::ptr &msg)
{
    if (pending_msg_){
        sst_throw(illformederror,
           "serial_single_msg_lib_waiter::wait_msg: already have pending msg");
    }
    pending_msg_ = msg;
    os_->block(key_);
}

void
serial_single_msg_lib_waiter::incoming_msg(const sst_message::ptr &msg)
{
    if (pending_msg_ != msg){
        sst_throw(illformederror,
          "serial_single_msg_lib_waiter::wait_msg: pending msg does not match incoming msg");
    }
    pending_msg_ = 0;
    os_->unblock(key_);
}

serial_generic_lib_waiter::serial_generic_lib_waiter(key::category cat)
  : serial_blocking_lib_waiter(cat)
{
}

void
serial_generic_lib_waiter::wait_msg(const sst_message::ptr &msg)
{
    if (wait_req_){
        sst_throw(illformederror,
            "serial_generic_lib_waiter::wait_msg: already waiting");
    }
    wait_req_ = wait_this::construct(msg);
}

sst_message::ptr
serial_generic_lib_waiter::wait_msg_type(sst_message::MESSAGE_TYPES ty, int count)
{
    if (wait_req_){
        sst_throw(illformederror,
            "serial_generic_lib_waiter::wait_msg: already waiting");
    }
    wait_req_ = wait_num_type::construct(ty, count);
    os_->block(key_);
    sst_message::ptr msg = wait_req_->get_msg();
    wait_req_ = 0;
    return msg;
}

void
serial_generic_lib_waiter::incoming_msg(const sst_message::ptr &msg)
{
    bool done = wait_req_->incoming_message(msg);
    if (done){
        wait_req_ = 0;
        os_->unblock(key_);
    }
}

multithreaded_blocking_lib_waiter::multithreaded_blocking_lib_waiter(key::category cat)
 : cat_(cat)
{
}

void
multithreaded_blocking_lib_waiter::incoming_msg(const sst_message::ptr &msg)
{
    std::map<sst_message::ptr,key::ptr>::iterator it = keys_.find(msg);
    if (it == keys_.end()){
        sst_throw(illformederror, "multithreaded_blocking_lib_waiter::incoming_msg\n"
            "no key for message %s", msg->to_string().c_str());
    }
    key::ptr the_key = it->second;
    keys_.erase(it);
    os_->unblock(the_key);
}

void
multithreaded_blocking_lib_waiter::wait_msg(const sst_message::ptr &msg)
{
    key::ptr the_key = key::construct(cat_);
    keys_[msg] = the_key;
    os_->block(the_key);
}

serial_blocking_lib_waiter::serial_blocking_lib_waiter(key::category cat)
    : key_cat_(cat)
{
}

    }
}
