#include "libtopomap.hpp"
#include <metis.h>

/* this is the recursive mapping approach similar to SCOTCH */
int TPM_Map_recursive(int r, TPM_Graph *ptg_ref, TPM_Graph_ewgts *ptgc_ref,
    TPM_Graph_vwgts *nprocs_ref, TPM_Graph *ltg_ref, TPM_Graph_ewgts *weights_ref,
    TPM_Mapping *mapping_ref, TPM_Mapping ltg_map /* indices in logical topology */, TPM_Mapping *ptg_map_ref /* map from passed ptg (subgraph) into global ptg */) {

  // turn refs into normal objects
  TPM_Graph& ptg = *ptg_ref;
  TPM_Graph& ptgc = *ptgc_ref; // ptg edge capacities!
  TPM_Graph& ltg = *ltg_ref;
  TPM_Graph& ltgw = *weights_ref; // ltg edge weights!
  TPM_Graph_vwgts& nprocs = *nprocs_ref;
  TPM_Mapping& mapping = *mapping_ref;
  TPM_Mapping& pmap= *ptg_map_ref;

  //if(!r) printf("[%i] TPM_MAP_RECURSIVE: nprocs: %i, ptg: %i ltg: %i\n", r, std::accumulate(nprocs.begin(), nprocs.end(), 0), ptg.size(), ltg_map.size());

  if(ltg_map.size() != 1) {

    //if(r) MPI_Barrier(MPI_COMM_WORLD);

    idx_t n = ptg.size();
    idx_t ncon=1;
    int nedges=0; for(int i=0; i<ptg.size(); ++i) nedges += ptg[i].size();

    std::vector<idx_t>  xadj(n+1),
                          adjncy(nedges+1),
                          part(n),
                          vwgt(n+1),
                          adjwgt(nedges+1);

    int nparts = 2;
    int edgecut = 0;

    int offset = 0;
    int adjoff = 0;
    for(int i=0; i<n; ++i) {
      xadj[i] = offset;
      offset += ptg[i].size();

      vwgt[i] = nprocs[i];

      //if(!r) { if(i==0) printf("nprocs: "); if(vwgt[i] != 0) printf("%i ", i); if(i==n-1) printf("\n"); }

      //if(!r) printf("|(off: %i, wgt: %i) ", xadj[i], vwgt[i]);
      for(int j=0; j<ptg[i].size(); ++j) {
        adjncy[adjoff] = ptg[i][j];
        adjwgt[adjoff] = ptgc[i][j];
        //if(!r) printf("%i ", adjncy[adjoff]);
        adjoff++;
      }
    }
    xadj[n] = offset;
    //if(!r) printf("|(%i)\n", xadj[n]);

    METIS_PartGraphRecursive(&n, &ncon, &xadj[0], &adjncy[0], &vwgt[0], NULL, &adjwgt[0], &nparts, NULL, NULL, NULL, &edgecut, &part[0]);

    int group1=0, group0=0, diff=2;

    while(diff > 1) {
      group0=0;
      group1=0;
      for(int i=0; i<n; ++i) {
        if(part[i] == 0) group0 += nprocs[i];
        if(part[i] == 1) group1 += nprocs[i];
      }
      // printf("[%i] pmap group0: %i, group1: %i\n", r, group0, group1);
      // check if it's balanced or not -- correct if necessary
      diff = group0-group1;
      if(diff < 0) diff *= -1;
      if(diff > 1) { //it's not balanced!
        int min, min_vert;
        if(group0 > group1) { // reset one 0 to a 1 in part: pick the least connected one!
          min=std::numeric_limits<int>::max();
          min_vert=-1;
          for(int i=0; i<part.size(); ++i) {
            if(part[i] == 0 && nprocs[i] <= diff) {
              int outweight = std::accumulate(ptgc[i].begin(), ptgc[i].end(), 0);
              if(min > outweight) {
                min_vert = i;
                min = outweight;
              }
            }
          }
          assert(min_vert != -1);
          part[min_vert] = 1;
        }
        if(group1 > group0) { // reset one 1 to a 0 in part: pick the least connected one!
          min=std::numeric_limits<int>::max();
          min_vert=-1;
          for(int i=0; i<part.size(); ++i) {
            if(part[i] == 1 && nprocs[i] <= diff) {
              int outweight = std::accumulate(ptgc[i].begin(), ptgc[i].end(), 0);
              if(min > outweight) {
                min_vert = i;
                min = outweight;
              }
            }
          }
          assert(min_vert != -1);
          part[min_vert] = 0;
        }
        if(!r) printf("correcting pmap partition ... moving vert %i with minimal weight %i\n", min_vert, min);
      }
    }

    TPM_Graph_vwgts nprocs0, nprocs1;
    //TPM_Graph_vwgts nprocs0(n,0), nprocs1(n,0);
    // fill new Mapping vectors
    /*for(int i=0; i<n; ++i) {
      // fill only nprocs in the right partitions!
      if(part[i] == 0) {
        nprocs0[i] = nprocs[i];
      } else {
        nprocs1[i] = nprocs[i];
      }
    }*/

    // remove all cut edges from ptg
    /*for(int i=0; i<ptg.size(); ++i) {
      for(int j=0; j<ptg[i].size(); ) {
        if(part[i] != part[ ptg[i][j] ]) {
          if(!r) printf("removed edge %i->%i (%i != %i)\n", i, ptg[i][j], part[i], part[ ptg[i][j] ]);
          ptg[i].erase(ptg[i].begin()+j);
          ptgc[i].erase(ptgc[i].begin()+j);
        } else j++;
      }
    }*/

    // build two ptgs for the two partitions
    TPM_Graph ptg0, ptg1;
    TPM_Graph_ewgts ptgc0, ptgc1;
    TPM_Mapping pmap0, pmap1; // needed to translate ranks back to original graph

    TPM_Mapping newverts(ptg.size()); // translation table from old vertices to index in two new graphs
    int idx0=0, idx1=0;
    for(int i=0; i<ptg.size(); ++i) {
      if(part[i] == 0) { newverts[i] = idx0++; }
      if(part[i] == 1) { newverts[i] = idx1++; }
    }

    for(int i=0; i<part.size(); ++i) {
      if(part[i] == 0) {
        ptg0.resize(ptg0.size()+1);
        ptgc0.resize(ptgc0.size()+1);
        nprocs0.push_back(nprocs[i]);
        pmap0.push_back(pmap[i]);
        for(int j=0; j<ptg[i].size(); ++j) {
          // only add edges that are leading to a vertex in the same partition
          if(part[i] == part[ ptg[i][j] ]) {
            ptg0[ptg0.size()-1].push_back(newverts[ptg[i][j]]);
            ptgc0[ptgc0.size()-1].push_back(ptgc[i][j]);
          }
        }
      }
      if(part[i] == 1) {
        ptg1.resize(ptg1.size()+1);
        ptgc1.resize(ptgc1.size()+1);
        nprocs1.push_back(nprocs[i]);
        pmap1.push_back(pmap[i]);
        for(int j=0; j<ptg[i].size(); ++j) {
          // only add edges that are leading to a vertex in the same partition
          if(part[i] == part[ ptg[i][j] ]) {
            ptg1[ptg1.size()-1].push_back(newverts[ptg[i][j]]);
            ptgc1[ptgc1.size()-1].push_back(ptgc[i][j]);
          }
        }
      }
    }

    /*if(!r) {
      printf("ptg group0 (edgecut: %i): ", edgecut);
      for(int i=0; i<nprocs0.size(); ++i) if(nprocs0[i] != 0) printf(" %i ", pmap[i]);
      printf("\n");
    }*/

    /******* partition logical topology graph ********/
    n = ltg_map.size();
    nedges=0; for(int i=0; i<ltg_map.size(); ++i) nedges += ltg[ ltg_map[i] ].size();

    // invert ptg_map (ptg graph) to translate adjacency lists to new graph
    int max = *std::max_element(ltg_map.begin(),ltg_map.end());
    std::vector<idx_t> invmap(max+1,-1);
    for(int i=0; i<n; ++i) invmap[ ltg_map[i] ] = i;
    // end inversion

    xadj.resize(n+1);
    vwgt.resize(n+1);
    adjncy.resize(nedges+1);
    adjwgt.resize(nedges+1);
    part.resize(n);

    nparts = 2;
    edgecut = 0;

    offset = 0;
    adjoff = 0;
    for(int i=0; i<n; ++i) {
      xadj[i] = offset;
      // compute the right offset (only including edges in our group)
      for(int j=0; j<ltg[ ltg_map[i] ].size(); ++j) if(ltg[ ltg_map[i] ][j] <= max && invmap[ ltg[ ltg_map[i] ][j] ] != -1) offset++;

      vwgt[i] = 1;

      for(int j=0; j<ltg[ ltg_map[i] ].size(); ++j) {
        // see ptg for comments about datastructures
        int dst = -1;
        if(ltg[ ltg_map[i] ][j] <= max) { // if the destination is > max then it's not in our set!
          dst = invmap[ ltg[ ltg_map[i] ][j] ];
        }
        // -1 are the destinations outside our group! Exclude them!
        if(dst != -1) {
          adjncy[adjoff] = dst;
          adjwgt[adjoff] = ltgw[ ltg_map[i] ][j];
          adjoff++;
          //printf("i: %i, ptg_map[i]: %i, j: %i, ptg[ ptg_map[i] ][j]: %i, invmap[ ptg[ ptg_map[i] ][j] ]: %i\n", i, ptg_map[i], j, ptg[ ptg_map[i] ][j], invmap[ ptg[ ptg_map[i] ][j] ]);
        }
      }
    }
    xadj[n] = offset;

    //if(!r) printf("calling second Metis\n");
    METIS_PartGraphRecursive(&n, &ncon, &xadj[0], &adjncy[0], NULL, NULL, &adjwgt[0], &nparts, NULL, NULL, NULL, &edgecut, &part[0]);

    //if(!r) printf("[%i] lmap group0: %i, group1: %i\n", r, group0, group1);

    // check if it's balanced or not -- correct if necessary
    diff = 2; // enter at least once :)
    while(diff > 1) {
      group1 = std::accumulate(part.begin(), part.end(), 0); // number of vertices in the second group
      group0 = part.size()-group1;
      diff = group0-group1;
      if(diff < 0) diff *= -1;
      if(diff > 1) { //it's not balanced!
        int min_vert, min;
        if(group0 > group1) { // reset one 0 to a 1 in part: pick the least connected one!
          min=std::numeric_limits<int>::max();
          min_vert=-1;
          for(int i=0; i<part.size(); ++i) {
            if(part[i] == 0) {
              int outweight = std::accumulate(ltgw[ ltg_map[i] ].begin(), ltgw[ ltg_map[i] ].end(), 0);
              if(min > outweight) {
                min_vert = i;
                min = outweight;
              }
            }
          }
          assert(min_vert != -1);
          part[min_vert] = 1;
        }
        if(group1 > group0) { // reset one 0 to a 1 in part: pick the least connected one!
          min=std::numeric_limits<int>::max();
          min_vert=-1;
          for(int i=0; i<part.size(); ++i) {
            if(part[i] == 1) {
              int outweight = std::accumulate(ltgw[ ltg_map[i] ].begin(), ltgw[ ltg_map[i] ].end(), 0);
              if(min > outweight) {
                min_vert = i;
                min = outweight;
              }
            }
          }
          assert(min_vert != -1);
          part[min_vert] = 0;
        }
        if(!r) printf("correcting lmap partition ... moving vert %i with minimal weight %i (group0: %i, group1: %i)\n", min_vert, min, group0, group1);
      }
    }

    TPM_Mapping lmap1, lmap2;
    // fill new Mapping vectors
    for(int i=0; i<part.size(); ++i) {
      //if(!r) printf("%i\n", part[i]);

      // the i-th entry in part is the i-th entry in ptg_map
      if(part[i] == 0) {
        lmap1.push_back( ltg_map[i] );
      } else {
        lmap2.push_back( ltg_map[i] );
      }
    }

    /*if(!r) {
      printf("ltg group0 (edgecut: %i): ", edgecut);
      for(int i=0; i<lmap1.size(); ++i) printf(" %i ", lmap1[i]);
      printf("\n");
    }*/

    // save memory in recursion!
    xadj.resize(0);
    vwgt.resize(0);
    adjncy.resize(0);
    adjwgt.resize(0);
    part.resize(0);
    invmap.resize(0);

    //if(!r) printf("calling recursively %i %i, %i %i\n", lmap1.size(), pmap1.size(), lmap2.size(), pmap2.size());
    // call recursively (two subtrees)
   // check which pmap fits which lmap size (might be two different if uneven group size) :-)
   int nprocs0_size = std::accumulate(nprocs0.begin(), nprocs0.end(), 0);
   int nprocs1_size = std::accumulate(nprocs1.begin(), nprocs1.end(), 0);

   if(lmap1.size() == nprocs0_size && lmap2.size() == nprocs1_size) {
      TPM_Map_recursive(r, &ptg0, &ptgc0, &nprocs0, ltg_ref, weights_ref, mapping_ref, lmap1, &pmap0);
      TPM_Map_recursive(r, &ptg1, &ptgc1, &nprocs1, ltg_ref, weights_ref, mapping_ref, lmap2, &pmap1);
   } else if(lmap1.size() == nprocs1_size && lmap2.size() == nprocs0_size) {
      TPM_Map_recursive(r, &ptg1, &ptgc1, &nprocs1, ltg_ref, weights_ref, mapping_ref, lmap1, &pmap1);
      TPM_Map_recursive(r, &ptg0, &ptgc0, &nprocs0, ltg_ref, weights_ref, mapping_ref, lmap2, &pmap0);
   } else {
     if(!r) printf("[%i] mapping sizes don't match (lmap1.size(): %lu, nprocs1_size: %i, lmap2.size(): %lu, nprocs0_size: %i) -- this should never happen - abort!\n", r, lmap1.size(), nprocs1_size, lmap2.size(), nprocs0_size);
     MPI_Barrier(MPI_COMM_WORLD);
     MPI_Abort(MPI_COMM_WORLD, 4);
   }

  } else {
    // end of recursion -- finalize mapping
    assert(std::accumulate(nprocs.begin(), nprocs.end(), 0) == 1); // only one nproc in ptg left
    int index = 0; while(nprocs[index] == 0) index++;

    assert(ltg_map.size() == 1);
    mapping[ ltg_map[0] ] = pmap[index];

    //if(!r) printf("mapping: %i to %i (index: %i)\n", pmap[index], ltg_map[0], index);
  }
  return 0;
}
