/*
 * routing.hpp
 *
 *  Created on: Aug 18, 2012
 *      Author: htor
 */

#ifndef ROUTING_HPP_
#define ROUTING_HPP_

#include <map>
#include <vector>
#include <iostream>
#include <string>

enum routing_type { COMPXYZ, SSSP, COMPCRAYXE6, UNCOMPRESSED };
typedef struct {int x,y,z; } xyzcoords_t;

class comparexyz { // simple comparison function
   public:
      bool operator()(const xyzcoords_t a,const xyzcoords_t b) {
        // Calculate the 1-D order of a and b
        // TODO: make this less hacky - this assumes 100x100x100 is max torus size
        int a_order = a.x + a.y * 100 + a.z * 100 * 100;
        int b_order = b.x + b.y * 100 + b.z * 100 * 100;
        if (a_order < b_order)
            return true;
        return false;
      }
};


typedef struct {
  enum routing_type type;

  std::map<std::pair<int,int>, std::vector<int>* > routes; // this map finds a route (an array of ints) identified by a <src,dst> pair

  /* for XYZ routing only ... */
  std::map<xyzcoords_t, int, comparexyz> xyz_to_rank; // resolve coordinates to rank
  std::vector<xyzcoords_t> rank_to_xyz; // resolve rank to coordinates
  std::vector<int> xe6gemini; // which gemini is this node attached to! (also index into h)
  xyzcoords_t xyzmaxdims; // local rank's XYZ coordinates

} TPM_Routes;

void TPM_RoutesInit(int print, TPM_Routes *r);
int TPM_RoutesRead(TPM_Routes *r, char *file);
void TPM_RouteGet(TPM_Graph *g_ref, TPM_Graph_dewgts *w_ref, TPM_Routes *routes_ref, int src, int dst, std::vector<int> *path_ref);
void TPM_SSSP(TPM_Graph *g_ref, TPM_Graph_dewgts *w_ref, int src, TPM_Graph_dists *dist_ref, TPM_Graph_preds *pred_ref);

#endif /* ROUTING_HPP_ */
