/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_TOPOMAP_TOPOMAP_API_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_TOPOMAP_TOPOMAP_API_H_INCLUDED

#include <sstmac/software/api/api.h>
#include <sstmac/software/process/pmi.h>

#include <sstmac/software/api/mpi/sstmac_mpi.h>

namespace sstmac
{
  namespace sw
  {

    class topomap_api : public sstmac::sw::api,
        public sstmac::sw::process_manager
    {

    protected:
      topomap_api(std::string f);

      MPI_Comm comm_;
      int rank_;
      std::string app_fname_;

      void
      make_system_graph(const char *sys_graph_file);

      void
      read_app_graph(std::string app_graph_file, std::vector<int> &sources,
          std::vector<int> &degrees, std::vector<int> &destinations,
          std::vector<int> &weights, int &nranks);

      void
      make_allocation_file(const char* fakename);

      static debug dbg_;

    public:
      typedef boost::intrusive_ptr<topomap_api> ptr;

      static ptr
      construct(const sstmac::sim_parameters::ptr& params)
      {
        std::string f =
            params->get_optional_param("topomap_app_graph_file", "");
        return ptr(new topomap_api(f));
      }

      virtual
      ~topomap_api()
      {
      }

      virtual std::string
      to_string() const
      {
        return "topomap_api";
      }

      virtual void
      init(const sstmac::sw::softwareid& sid)
      {
        libname_ = sstprintf("topomap_api_%d_%d",
                                sid.app_.id, sid.task_.id);
        process_manager::init(sid);
      }

      void
      init_os(const OS_API::ptr &os)
      {
        process_manager::init_os(os);
        api::init_os(os);
      }

      void
      tpm_init(bool usetopo);

      MPI_Comm
      tpm_comm()
      {
        return comm_;
      }

      int
      tpm_rank()
      {
        return rank_;
      }
    };

  }
}

#endif
