/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_UNBLOCKHANDLER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_UNBLOCKHANDLER_H_INCLUDED

#include <sstmac/common/eventhandler.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/software/process/os_api.h>
#include <sstmac/software/process/key.h>

#include <sstmac/common/ptr_type.h>

namespace sstmac
{
  namespace sw
  {

    class unblockhandler : public eventhandler
    {
    protected:
      OS_API::ptr os_;
      key::ptr key_;

    protected:
      unblockhandler(const OS_API::ptr &os, const key::ptr& k) :
          os_(os), key_(k)
      {

      }
    public:
      virtual std::string
      to_string() const
      {
        return "unblockhandler";
      }

      typedef boost::intrusive_ptr<unblockhandler> ptr;

      static unblockhandler::ptr
      construct(const OS_API::ptr &os, const key::ptr& k)
      {
        unblockhandler::ptr ret(new unblockhandler(os, k));
        return ret;
      }

      sst_message::ptr saved_msg_;

      virtual void
      handle(const sst_message::ptr &msg)
      {
        saved_msg_ = msg;
        os_->unblock(key_);
      }
    };

  }
} //end of namespace sstmac

#endif
