/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_APPID_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_APPID_H_INCLUDED

#include <iostream>
#include <sstream>
#include <sstmac/common/types.h>

namespace sstmac { namespace sw {

  /**
   * A ligthweight wrapper for an app index.
   * Done to avoid annoying mistakes where index values are mixed up.
   * 
   * I can't wait for C++ to add opaque typedefs.
   */
  struct appid {
    int id;
    explicit appid(int v = -1) : id(v) {}

    inline appid& operator++() {
      ++id;
      return *this;
    }
    inline appid operator++(int) {
      appid other(*this);
      ++id;
      return other;
    }

    std::string to_string() const {
      std::stringstream ss(std::stringstream::in | std::stringstream::out);
      ss << "appid(" << id << ")";
      return ss.str();
    }
  };

  inline bool operator==(const appid &a, const appid &b) {
    return (a.id == b.id);
  }
  inline bool operator!=(const appid &a, const appid &b) {
    return (a.id != b.id);
  }
  inline bool operator<(const appid &a, const appid &b) {
    return (a.id < b.id);
  }
  inline bool operator<=(const appid &a, const appid &b) {
    return (a.id <= b.id);
  }
  inline bool operator>(const appid &a, const appid &b) {
    return (a.id > b.id);
  }
  inline bool operator>=(const appid &a, const appid &b) {
    return (a.id >= b.id);
  }
  inline std::ostream& operator<<(std::ostream &os, const appid &n) {
    return (os << n.to_string());
  }

} } // end of namespace sstmac

#endif
