/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_GLOBAL_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_GLOBAL_H_INCLUDED

#include <sstream>

#include <boost/unordered_map.hpp>
#include <sstmac/common/logger.h>
#include <sstmac/software/process/process_context.h>

#include <sstmac/software/process/os_api.h>
#include <sstmac/software/process/thread.h>

namespace sstmac
{
  namespace sw
  {

    class sstmac_global
    {
    protected:
      process_context::ptr
      get_current_context() const
      {
        thread::ptr t = OS_API::get_current_thread();
        if (t)
          return t->get_process_context();
        else
          return NULL;
      }

    public:
      virtual std::string
      to_string() const
      {
        return "sstmac_global base class";
      }
    };

    inline std::ostream&
    operator<<(std::ostream &os, sstmac_global &g)
    {
      return (os << g.to_string());
    }

    inline logger&
    operator<<(logger &os, sstmac_global &g)
    {
      return (os << g.to_string());
    }

  }
}
#endif
