/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/process/key.h>
#include <sstmac/common/alloc.h>
#include <iostream>
#include <stdlib.h>

namespace sstmac { namespace sw {
  
  namespace keypool {
    static std::vector<key*>* chunks = 0;
  } // end of namespace keypool.

key::category key::general("General");
boost::unordered_map<std::string,int>* key::category_name_to_id_ = 0;
boost::unordered_map<int,std::string>* key::category_id_to_name_ = 0;

key::category::category(const std::string &name) :
    id_(-1)
{
    id_ = key::allocate_category_id(name);
}

key::category::category()
{
}

int
key::allocate_category_id(const std::string &name)
{
    if (!category_name_to_id_){
        category_name_to_id_ = new boost::unordered_map<std::string,int>;
        category_id_to_name_ = new boost::unordered_map<int,std::string>;
    }
    int id = category_name_to_id_->size();
    (*category_name_to_id_)[name] = id;
    (*category_id_to_name_)[id] = name;
    return id;
}

key::key() :
    keyname_id_(general.id())
{
}

key::key(const category& cat) :
    keyname_id_(cat.id())
{
}

key::ptr key::construct() {
      return new key;
}

key::ptr key::construct(const category &name) {
    return new key(name);
}


key::~key(){
}

void* key::operator new(size_t size) {
  if(! keypool::chunks) {
    keypool::chunks = new std::vector<key*>;
    keypool::chunks->reserve(1024);
  }

  void* memptr;
  if(!keypool::chunks->empty()) {
    memptr = keypool::chunks->back();
    keypool::chunks->pop_back();
  }
  else{
      memptr = ::malloc(size);
  }
  return memptr;
}

void key::add_blocker(thread_data_t t)
{
    blocking_t::iterator it = blocking_.find(t);
    if (it != blocking_.end())
    {
        std::cerr << "thread already there: " << it->second->to_string() << "\n";
        std::cerr << "me: " << (long)(this) << "\n";
        throw illformederror("key is blocking the same thread twice");
    }
    blocking_.insert(t);
}

void key::remove_blocker(thread_data_t t)
{
    blocking_.erase(t);
}

void key::operator delete(void* ptr){
  keypool::chunks->push_back(static_cast<key*>(ptr));
}

void key::clear_statics(){
    if (keypool::chunks){
        for (int i=0; i < keypool::chunks->size(); ++i){
            ::free(keypool::chunks->at(i));
        }
        keypool::chunks->clear();
        delete keypool::chunks;
    }
}

} } // end of namespace sstmac.
