#include <sstmac/software/process/pmi.h>
#include <sstmac/software/services/launch/launcher.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/node/node.h>

namespace sstmac {
    namespace sw {

process_manager::app_to_proc_to_node_map process_manager::node_map_;
process_manager::app_to_node_to_proc_map process_manager::proc_map_;

void
process_manager::init(const softwareid &sid)
{
    sid_ = sid;

    nodeaddress::ptr addr = launcher::get_node(sid.app_, sid.task_);
    node_map_[sid.app_.id][sid.task_.id] = addr;
    proc_map_[sid.app_.id][addr] = sid.task_.id;
    my_addr_ = addr;


}

void
process_manager::init_os(const OS_API::ptr &os)
{
    AMI* comp_ami = os->getAMI(AMI::AMI_COMM);
    hw::node* my_node = dynamic_cast<hw::node*>(comp_ami);
    if (!my_node){
        sst_throw(illformederror,
            "process manager AMI_COMM does not map to sstmac::hw::node");
    }
    hw::networkinterface::ptr nic = my_node->get_nic();
    interconn_ = nic->get_interconnect();
    top_ = interconn_->get_topology();
}

int
process_manager::get_partner(const nodeaddress::ptr &addr) const
{
    app_to_node_to_proc_map::const_iterator it1 =
        proc_map_.find(sid_.app_.id);

    if (it1 == proc_map_.end()){
        sst_throw(valueerror,
            "process_manager::get_partner: "
            "Could not find app %d. "
            "Are you sure process_manager::init was called?",
             sid_.app_.id);
    }

    const node_to_proc_map& node_map = it1->second;
    node_to_proc_map::const_iterator it2 =
        node_map.find(addr);

    if (it2 == node_map.end()){
        sst_throw(valueerror,
            "process_manager::get_partner: "
            "Could not find nodeaddress %ld for app %d",
            addr->unique_id(), sid_.app_.id);
    }

    return it2->second;
}

    }
}
