/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_PROCESS_CONTEXT_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_PROCESS_CONTEXT_H_INCLUDED

#include <sstmac/common/ptr_type.h>

namespace sstmac
{
  namespace sw
  {

    class process_context : public ptr_type
    {

    protected:
      process_context()
      {
        key_ = key_cnt_++;
      }

      long key_;
      static long key_cnt_;

    public:
      typedef boost::intrusive_ptr<process_context> ptr;

      static ptr
      construct()
      {
        return ptr(new process_context());
      }

      virtual ~process_context(){}

      virtual std::string to_string() const {
        return "process_context";
      }

      long
      get_key() const{
        return key_;
      }
    };

  }

}

#endif
