/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_SOFTWAREID_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_SOFTWAREID_H_INCLUDED

#include <iostream>
#include <sstream>
#include <sstmac/software/process/appid.h>
#include <sstmac/software/process/taskid.h>
#include <sstmac/software/process/threadid.h>

namespace sstmac { namespace sw {

  /**
   * A wrapper for an appid, taskid pair
   *
   */
  struct softwareid {
    appid app_;
    taskid task_;
    threadid thread_;

    explicit softwareid(appid a, taskid t, threadid th = threadid(0)) : app_(a), task_(t), thread_(th) {}

    softwareid() : app_(-1), task_(-1) { }

    /*inline softwareid& operator++() {
      ++task_;
      return *this;
    }
    inline softwareid operator++(int) {
      softwareid other(*this);
      ++task_;
      return other;
    }*/

    std::string to_string() const {
      std::stringstream ss(std::stringstream::in | std::stringstream::out);
      ss << "softwareid(" << app_ << ", " << task_ << "," << thread_ << ")";
      return ss.str();
    }
  };

  inline bool operator==(const softwareid &a, const softwareid &b) {
    return (a.app_.id == b.app_.id && a.task_.id == b.task_.id && a.thread_.id == b.thread_.id);
  }
  inline bool operator!=(const softwareid &a, const softwareid &b) {
    return (a.app_.id != b.app_.id || a.task_.id != b.task_.id || a.thread_.id != b.thread_.id);
  }
  inline bool operator<(const softwareid &a, const softwareid &b){
    return (a.task_.id < b.task_.id);
  }
  inline bool operator>(const softwareid &a, const softwareid &b){
    return (a.task_.id > b.task_.id);
  }


  inline std::ostream& operator<<(std::ostream &os, const softwareid &n) {
    return (os << n.to_string());
  }

} } // end of namespace sstmac

#endif
