/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_WAITER_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_WAITER_H_INCLUDED

#include <sstmac/common/ptr_type.h>

namespace sstmac { namespace sw
{

  /**
   * The main interface for something that can wait in a certain way for an event that is important to it.
   */
  class waiter : public virtual ptr_type
  {
  public:

    typedef boost::intrusive_ptr<waiter> ptr;

  protected:

    waiter()
    {

    }

  public:
    static ptr
    construct(){
      return ptr(new waiter());
    }

    virtual std::string
    to_string() const
    {
      return "waiter";
    }

    virtual bool
    is_blocking() const
    {
        return false;
    }

    virtual bool
    is_callback() const
    {
        return false;
    }

    virtual
    ~waiter()
    {

    }

    virtual void
    wait()
    {

    }


  };

  class callbackwaiter : public waiter
  {
    protected:
      callbackwaiter()
      {
      }

    public:
      typedef boost::intrusive_ptr<callbackwaiter> ptr;

      static ptr
      construct()
      {
          return new callbackwaiter();
      }

      bool is_callback() const
      {
          return true;
      }
  };
} } // end of namespace sstmac
#endif
