/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_hostname_allocation_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_hostname_allocation_H_INCLUDED

#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>

#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/hardware/network/topology/regular_topology.h>

namespace sstmac
{

  namespace sw
  {
    class hostname_allocation : public allocationstrategy
    {
    protected:
      hostname_allocation(const std::string& mapname);

      sstmac::hw::regular_topology::ptr regtop_;

    public:
      typedef boost::intrusive_ptr<hostname_allocation> ptr;

      virtual void init(const hw::topology::ptr &top);

      virtual void
      set_topology(const sstmac::hw::topology::ptr& top);

      static ptr
      construct(const std::string& mapname);

      static ptr
      construct(sim_parameters::ptr params);

      /**
       * This function reads in a map file for mapping host names to network coordinates.
       * @param here std::string used for debugging / exception throwing
       * @param mapfile name of the map file
       * @param hostmap map in which host mappings are placed
       */
      static void
      read_map_file(const std::string &here, const std::string &mapfile,
          std::map<std::string, std::vector<int> >& hostmap);

      virtual void
      allocate(int nnode_requested,
          node_set &allocation);

      virtual
      ~hostname_allocation() throw ()
      {
      }

      typedef boost::unordered_map<std::string, nodeaddress::ptr> nodemap_t;
      static nodemap_t hostnamemap_;

      static std::map<long, std::string> nodenum_to_host_map_;

    protected:
      std::string mapfile_;

    };

  }
}

#endif /* hostname_allocation_H_ */
