/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_INDEXSTRATEGY_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_INDEXSTRATEGY_H_INCLUDED

#include <sstmac/common/nodeaddress.h>
#include <sstmac/common/debug.h>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/software/process/appid.h>
#include <sstmac/common/factories/factory.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * Base class for strategies regarding how to sequentially number nodes
     * in a parallel simulation.
     */
    class indexstrategy :
            public factory_type
    {
    public:
    typedef std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp, std::allocator<nodeaddress::ptr> > node_set;

    protected:
      static debug dbg_;

      static hw::topology::ptr topology_;

      indexstrategy()
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> indexing | indexstrategy");
            logger::register_logger(&dbg_);
          }
      }

    public:
      static void clear_statics();

      virtual std::string
      to_string() const
      {
        return "indexstrat";
      }

      virtual void
      set_topology(const sstmac::hw::topology::ptr& top)
      {
        topology_ = top;
      }

      void init(const sstmac::hw::topology::ptr& top){
          set_topology(top);
      }

      typedef boost::intrusive_ptr<indexstrategy> ptr;

      virtual
      ~indexstrategy() throw ()
      {
      }

      /// Assign processes to nodes.
      /// \param nodes is the list of unique nodes to be used for the allocation
      /// \param ppn is the nominal number of processes allocated on each node.
      /// \param result is the resulting vector of length nodes (size nproc)
      /// \param nproc the total number of processes to allocate
      virtual void
      allocate(const appid& aid, const node_set &nodes, int ppn, std::vector<nodeaddress::ptr> &result, int nproc) = 0;
    };

    DeclareFactory1InitParam(indexstrategy, hw::topology::ptr);

  }
} // end of namespace sstmac

#endif
