/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/services/launch/indexing/roundrobinindexing.h>
#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace sw
  {
    SSTRegister("round | roundrobin", indexstrategy, roundrobinindexing,
"Tries to spread out ranks across the nodes. Ranks 0,1,2,3 are on different nodes. Ranks 0,4,8 are on the same node. Round robin and block indexing are equivalent if there is one process per node."
  );

    roundrobinindexing::roundrobinindexing()
    {
    }

    roundrobinindexing::ptr
    roundrobinindexing::construct(sim_parameters::ptr params)
    {
      return roundrobinindexing::ptr(new roundrobinindexing);
    }

    roundrobinindexing::~roundrobinindexing() throw ()
    {
    }

    void
    roundrobinindexing::allocate(
        const appid& aid,
        const node_set &nodes,
        int ppn,
        std::vector<nodeaddress::ptr> &result,
        int nproc)
    {

      if (nproc < 0)
        {
          nproc = nodes.size() * ppn;
        }

      if (nodes.empty())
        {
          throw valueerror("roundrobinindexing::allocate:  Empty node vector.");
        }
      if (ppn <= 0)
        {
          throw valueerror("roundrobinindexing::allocate:  ppn must be > 0.");
        }
      if ((int)(ppn * nodes.size()) != nproc)
        {
          throw valueerror(
              "roundrobingindexing::allocate: currently require ppn*nodes.size() == nproc");
        }

      node_set::iterator iter =
          nodes.begin();
      for (long i = 0; i < nproc / ppn; i++)
        {
          iter++;
        }

      for (int i = 0; i < ppn; ++i)
        {
          result.insert(result.end(), nodes.begin(), iter);
        }
    }

  }
} // end of namespace sstmac
