/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_complete_message_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_complete_message_H_INCLUDED

#include <sstmac/software/services/launch/messages/launch_message.h>

#include <sstmac/software/services/launch/launchinfo.h>

namespace sstmac
{
  namespace sw
  {

    class complete_message : public launch_message
    {

    public:
      typedef boost::intrusive_ptr<complete_message> ptr;

      static ptr
      construct(launchinfo::ptr &i, bool succ = true)
      {
        return ptr(new complete_message(i, succ));
      }

      /**
       * Stringifier
       * @return String description
       */
      virtual std::string
      to_string() const
      {
        return "complete_message";
      }

      bool
      success() const
      {
        return success_;
      }

      void
      set_local(bool l = false)  {
        local_ = l;
      }

      bool
      is_local() const {
        return local_;
      }

    protected:
      complete_message(launchinfo::ptr &i, bool succ) :
        sst_message(LAUNCH),
        launch_message(i, launch_message::COMPLETE), 
        success_(succ), local_(true)
      {

      }

      bool success_;
      bool local_;

    };

  }
}

#endif
