/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_start_message_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_MESSAGES_start_message_H_INCLUDED

#include <sstmac/software/services/launch/messages/launch_message.h>

#include <sstmac/software/services/launch/launchinfo.h>

namespace sstmac
{
  namespace sw
  {

    class start_message : public launch_message
    {

    public:
      typedef boost::intrusive_ptr<start_message> ptr;

      static ptr
      construct(launchinfo::ptr &i, nodeaddress::ptr to, taskid t,
          environment::ptr e)
      {
        return ptr(new start_message(i, to, t, e));
      }

      /**
       * Stringifier
       * @return String description
       */
      virtual std::string
      to_string() const
      {
        return "start_message";
      }

      taskid
      task() const
      {
        return tid_;
      }

      environment::ptr
      env() const
      {
        return env_;
      }



    protected:
      start_message(launchinfo::ptr &i, nodeaddress::ptr to, taskid t,
          environment::ptr e) :
        sst_message(LAUNCH), launch_message(i, launch_message::START), tid_(t),
            env_(e)
      {
        toaddr_ = to;

      }

      taskid tid_;
      environment::ptr env_;



    };

  }
}

#endif
