/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_reallaunch_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_reallaunch_H_INCLUDED

#include <sstmac/software/process/taskid.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/software/services/launch/launcher.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/software/process/app.h>

#include <sstmac/software/ami/environment.h>
#include <sstmac/common/debug.h>
#include <sstmac/software/services/launch/messages/launch_message.h>

#include <sstmac/software/services/launch/messages/arrive_message.h>
#include <sstmac/software/services/launch/messages/restart_message.h>
#include <sstmac/software/services/launch/messages/start_message.h>
#include <sstmac/software/services/launch/messages/complete_message.h>
#include <sstmac/software/services/launch/messages/kill_message.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * A launcher that does not model the time it takes to
     * send the application code or request to the nodes
     * across the network, the app just instantly comes up
     * at a given time
     */
    class reallaunch : public launcher, virtual public nocopy
    {
    public:
      typedef boost::intrusive_ptr<reallaunch> ptr;

    protected:

      reallaunch(const sim_parameters::ptr& params);

      key::ptr block_;



      enum APP_STATE
      {
        ARRIVING, WAITING, LAUNCHED, FINISHED, FAILED, NOT_RUNNING_HERE
      };

      //----- master node state ----- //


      std::map<appid, APP_STATE> states_;



      std::map<appid, long> complete_;

     // std::map<appid, environment::const_ptr> envmap_;

      std::queue<launchinfo::ptr> waiting_;
      // ---------------------------- //


      nodeaddress::ptr master_;

      sst_message::ptr visupdate_;

      long myid_;

      std::map<appid, int> intranode_rank_;

      std::map<appid, launchinfo::ptr> linfos_;

      sst_message::ptr clock_;

      void
      try_launch();


      virtual void
      msg_arrive(arrive_message::ptr msg);

      virtual void
      msg_start(start_message::ptr msg);

      virtual void
      msg_complete(complete_message::ptr msg);

      virtual void
      msg_kill(kill_message::ptr msg);

      virtual void
      msg_restart(restart_message::ptr msg);

    public:

      virtual
      ~reallaunch() throw ()
      {
      }

      static reallaunch::ptr
      construct(sim_parameters::ptr params);



      virtual std::string
      to_string() const
      {
        return "reallaunch";
      }

      virtual void
      handle(const sst_message::ptr &msg);

      virtual void
      start();

      virtual service::ptr
      clone();



      virtual void
      apps_failed();



    };

  }
} // end of namespace sstmac

#endif
