/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_MPICONGESTION_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_MPICONGESTION_H_INCLUDED

#include <sstmac/software/api/mpi/mpiapp.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * Basic MPI ping-pong.
     */
    class mpicongestion : public mpiapp
    {
      /// How many iterations we do.
      int iterations_, count_, window_;

    protected:
      /// Hi.
      mpicongestion(softwareid id) :
          mpiapp(id, to_string())
      {

      }

      mpicongestion(const sim_parameters::ptr& params) : mpiapp(params) {}

    public:
      typedef boost::intrusive_ptr<mpicongestion> ptr;

      /// Hello.
      static mpicongestion::ptr
      construct(softwareid id);

      static ptr
      construct(const sim_parameters::ptr& params)
      {
          return new mpicongestion(params);
      }

      /// Goodbye.
      virtual
      ~mpicongestion() throw ();

      /// Get a copy.
      virtual app::ptr
      clone(softwareid id);

      /// Go.
      void
      skeleton_main();

      virtual void
      consume_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "mpicongestion";
      }
    };

  }
} // end of namespace sstmac

#endif
