/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/mpipairwise.h>
#include <sstmac/software/api/mpi/mpiapi.h>
#include <cstring>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <sstream>

namespace sstmac
{
  namespace sw
  {
    SSTRegister("MPI_pairwise | mpi_pairwise", app, mpipairwise);
    // Pointer types.

    void
    mpipairwise::consume_params(const sim_parameters::ptr &params)
    {

      iterations_ = params->get_int_param("mpipairwise_iterations");

      count_ = params->get_int_param("mpipairwise_count");

      std::string line;
      std::string fname = params->get_param("mpipairwise_file");
      std::ifstream myfile;
      myfile.open(fname.c_str());

      if(!myfile.good()){
          sst_throw(ioerror, "mpipairwise: could not open file %s", fname.c_str());
      }

      while (!myfile.eof())
        {
          std::getline(myfile, line);
          std::istringstream iss(line);
          std::string temp;
          iss >> temp;
          this->from_.push_back(mpiid(atoi(temp.c_str())));
          iss >> temp;
          this->to_.push_back(mpiid(atoi(temp.c_str())));
          iss >> temp;
          this->when_.push_back(timestamp(atof(temp.c_str())));
        }
      myfile.close();

    }

    //
    // Goodbye.
    //
    mpipairwise::~mpipairwise() throw ()
        {
        }

    //
    // Get a copy.
    //
    sstmac::sw::app::ptr
    mpipairwise::clone(softwareid id)
    {
      mpipairwise::ptr obj = mpipairwise::construct(id);

      obj->from_.resize(this->from_.size());
      obj->to_.resize(this->to_.size());
      obj->when_.resize(this->when_.size());
      for (uint i = 0; i < obj->from_.size(); i++)
        {
          obj->from_[i] = this->from_[i];
          obj->to_[i] = this->to_[i];
          obj->when_[i] = this->when_[i];
        }
      return obj;
    }

    //
    // Go.
    //
    void
    mpipairwise::skeleton_main()
    {
      if (mpi() == 0)
        {
          throw nullerror("mpipairwise::run:  mpiapi pointer is null.");
        }
      timestamp start = mpi()->init();
      mpicomm::ptr world = mpi()->comm_world();

      mpiid rank = world->rank();
      mpiid size = world->size();
      //std::cerr << "mpipairwise[" << rank << "]::run(): mpi()->init() at " << start << "\n";
      mpitag tag(0);

      for (int j = 0; j < iterations_; j++)
        {

          for (uint i = 0; i < from_.size(); i++)
            {
              compute(when_[i]);
              if (rank == from_[i])
                {
                  SSTMAC_DEBUG << "rank " << rank.id << " sending to rank "
                      << to_[i].id << "\n";
                  // do send
                  mpi()->send(count_, mpitype::mpi_double, to_[i], tag, world);
                }
              if (rank == to_[i])
                {
                  SSTMAC_DEBUG << "rank " << rank.id << " receiving from rank "
                      << from_[i].id << "\n";
                  // do receive
                  mpistatus::const_ptr stat = mpistatus::construct();
                  mpi()->recv(count_, mpitype::mpi_double, from_[i], tag, world,
                      stat);
                }
            }
        }

      mpi()->finalize();

      //std::cerr << "mpipairwise[" << rank << "]::run():  mpi()->finalize() at " << done << "\n";
    }

  }
} //end of namespace sstmac
