#include <sstmac/common/test/test.h>
#include <sstmac/software/process/app.h>
#include <sstmac/sstmacro.h>
#include <sstmac/software/process/thread.h>
#include <sstmac/common/logger.h>

using sstmac::sstprintf;

namespace pingall {

sstmac_register_app(mpipingall);

void sleep(int seconds)
{
    sstmac::sw::thread::ptr thr = sstmac::sw::operatingsystem::get_current_thread();
    sstmac::sw::app::ptr app = sstmac::safe_cast(thr, sstmac::sw::app);
    app->sleep(timestamp(seconds));
}

int
mpipingall_main(int argc, char** argv)
{
    MPI_Init(&argc, &argv);

    int me, nproc;
    MPI_Comm_rank(MPI_COMM_WORLD, &me);
    MPI_Comm_size(MPI_COMM_WORLD, &nproc);

    //send/recv from all the other procs
    void* null_buffer = 0;
    int count = 100;
    int tag = 42;
    //one for each send, one for each recv
    MPI_Request* reqs = new MPI_Request[2*nproc];
    MPI_Request* reqptr = reqs;
    for (int i=0; i < nproc; ++i){
        if (i == me)
            continue;

        MPI_Isend(null_buffer, 100, MPI_INT, i, tag, MPI_COMM_WORLD, reqptr);
        ++reqptr;
        MPI_Irecv(null_buffer, count, MPI_INT, i, tag, MPI_COMM_WORLD, reqptr);
        ++reqptr;
    }

    // wait on the first quarter
    int quarter_size = nproc / 4;
    int remainder = nproc % 4;

    reqptr = reqs;
    for (int q=0; q < 4; ++q){
        MPI_Waitall(quarter_size, reqptr, MPI_STATUSES_IGNORE);
        reqptr += quarter_size;
        sleep(1);
    }

    if (remainder){
        MPI_Waitall(remainder, reqptr, MPI_STATUSES_IGNORE);
        sleep(1);
    }

    MPI_Finalize();
    return 0;
}

}
