/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/mpipingpong.h>
#include <sstmac/software/api/mpi/mpiapi.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>
#include <cstring>
#include <stdio.h>

namespace sstmac
{
  namespace sw
  {
    SSTRegister("MPI_pingpong | mpi_pingpong", app, mpipingpong);

// Pointer types.

//
// Hello.
//
    mpipingpong::ptr
    mpipingpong::construct(softwareid id)
    {
      mpipingpong::ptr obj(new mpipingpong(id));
      //  obj->set_self(obj);
      return obj;
    }

//
// Goodbye.
//
    mpipingpong::~mpipingpong() throw ()
    {
    }

//
// Get a copy.
//
    app::ptr
    mpipingpong::clone(softwareid id)
    {
      return mpipingpong::construct(id);
    }

    void
    mpipingpong::consume_params(const sim_parameters::ptr &params)
    {

      iterations_ = params->get_int_param("mpipingpong_iterations");

      count_ = params->get_int_param("mpipingpong_count");

    }

//
// Go.
//
    void
    mpipingpong::skeleton_main()
    {
      if (mpi() == 0)
        {
          throw nullerror("mpipingpong::run:  mpiapi pointer is null.");
        }
      SSTMAC_DEBUG << "mpipingpong[" << id_.to_string()
            << "]: mpi initializing... \n";
      timestamp start = mpi()->init();
      mpicomm::ptr world = mpi()->comm_world();

      mpiid rank = world->rank();
      mpiid size = world->size();

      SSTMAC_DEBUG << "mpipingpong[" << rank.to_string()
            << "]: mpi initialized, executing " << iterations_
            << " iterations \n";
      //std::cerr << "mpipingpong[" << rank << "]::run(): mpi()->init() at " << start << "\n";
      mpitag tag(0);
      bool participant = true;
      if ((size.id % 2) && (rank.id + 1 >= size.id))
        {
          SSTMAC_DEBUG << "mpipingpong[" << rank.to_string()
                << "]: opting out, because of size " << size.id << "\n";
          // This is the odd-node-out -- communicating with no-one.
          participant = false;
        }
      if (participant)
        {
          mpiid buddy(rank.id ^ 1); // 0<=>1, 2<=>3, etc.
          const_mpistatus_t stat;
          for (int half_cycle = 0; half_cycle < 2 * iterations_; ++half_cycle)
            {
              if ((half_cycle + rank.id) & 1)
                { // even values of half-cycle plus rank.
                  SSTMAC_DEBUG << "mpipingpong[" << rank.to_string()
                        << "]::main:  Peer " << rank.id
                        << " sending data at half-cycle " << half_cycle
                        << " (out of " << 2 * iterations_ << ")\n";
                  timestamp t(1E-8);
                  compute(t);
                  timestamp st = mpi()->send(count_, mpitype::mpi_double, buddy,
                      tag, world);
                  //std::cerr << "mpipingpong[" << rank << "]::run(): mpi()->send() at " << st << "\n";
                }
              else
                {
                  SSTMAC_DEBUG << "mpipingpong[" << rank.to_string()
                        << "]::main:  Peer " << rank.id
                        << " receiving data at half-cycle " << half_cycle
                        << " (out of " << 2 * iterations_ << ")\n";
                  timestamp st = mpi()->recv(count_, mpitype::mpi_double, buddy,
                      tag, world, stat);

                  //std::cerr << "mpipingpong[" << rank << "]::run(): mpi()->recv() at " << st << "\n";
                }
            }
        }
      timestamp done = mpi()->finalize();

      SSTMAC_DEBUG << "mpipingpong[" << rank.id << "] finalized\n";

      /*if (rank.id == 0) {
       double runtime = (done - start).sec();

       SSTMAC_DEBUG << "Peer " << rank.id << " started at " << start
       << " and finished at " << done
       << ", with a total runtime of " << runtime << " seconds\n";
       }*/
    }

  }
} // end of namespace sstmac.
