
/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_MPI_PROTOCOL_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_MPI_PROTOCOL_H_INCLUDED

#include <sstmac/software/api/mpi/mpiapp.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * Basic MPI ping-pong.
     */
    class mpiprotocol_test : public mpiapp
    {

    protected:
      /// Hi.
      mpiprotocol_test(softwareid id) :
          mpiapp(id, to_string())
      {

      }

      mpiprotocol_test(const sim_parameters::ptr& params) : mpiapp(params) {}

    public:
      typedef boost::intrusive_ptr<mpiprotocol_test> ptr;

      /// Hello.
      static mpiprotocol_test::ptr
      construct(softwareid id);

      static ptr
      construct(const sim_parameters::ptr& params)
      {
          return new mpiprotocol_test(params);
      }

      /// Goodbye.
      virtual
      ~mpiprotocol_test() throw ();

      /// Get a copy.
      virtual app::ptr
      clone(softwareid id);

      /// Go.
      void
      skeleton_main();

      virtual void
      consume_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "mpiprotocol_test";
      }
    };

  }
} // end of namespace sstmac


#endif // MPIPROTOCOL_H
