/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/test_compute_eiger.h>
#include <sstmac/software/api/eiger/sstmac_compute_eiger.h>

#include <sstmac/common/fileio.h>

namespace sstmac { namespace sw
{
    SSTRegister("test_compute_eiger", app, test_compute_eiger,
                "this test kernel constructs a set of dummy parameters and makes a single call to an eiger model");
  

  test_compute_eiger::test_compute_eiger(softwareid id) :
    app(id)
  {}

  test_compute_eiger::ptr
  test_compute_eiger::construct(softwareid id)
  {
    test_compute_eiger::ptr retval(new test_compute_eiger(id));
    return retval;
  }

  void
  test_compute_eiger::consume_params(const sim_parameters::ptr &params)
  {

  }

  //
  // Get a copy of this object.
  //
  app::ptr
  test_compute_eiger::clone(softwareid id)
  {
    return construct(id);
  }

  test_compute_eiger::~test_compute_eiger()
  {

  }

  void
  test_compute_eiger::skeleton_main()
  {
    SSTMAC_DEBUG << "t=" << now() << ": Hello World, i'm computer number "
                 << id_.task_.id << ", about to begin computing.\n";

		SSTMAC_compute_eiger3("test.model", "test.model",
				"veclen", 8.0,
				"nproc", 4.2,
				"nrepeat", 3.0);
		SSTMAC_DEBUG << "Done!\n";
  }

} } //end of namespace sstmac
