/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/test_file_system.h>

namespace sstmac
{
  namespace sw
  {
    SSTRegister("test_file_system | test_fs", app, test_file_system);

    void
    test_file_system::consume_params(const sim_parameters::ptr &params)
    {

      access_size_ = params->get_int_param("testfilesystem_access_size");
      seek_length_ = params->get_int_param("testfilesystem_seek_length");
    }

    void
    test_file_system::skeleton_main()
    {
      SSTMAC_DEBUG << "test_file_system: Hello World, i'm tester number "
            << id_.task_.id << ", about to begin accessing " << access_size_
            << " bytes \n";

      openfile::ptr file = fs_->open("");
      SSTMAC_DEBUG << "test_file_system: file is open, begin read \n";

      timestamp t1 = now();
      fs_->read(file, access_size_);
      timestamp t2 = now();

      SSTMAC_DEBUG << "test_file_system: read took " << (t2 - t1) << "\n";
      SSTMAC_DEBUG << "test_file_system: begin write \n";

      t1 = now();
      fs_->write(file, access_size_);
      t2 = now();

      SSTMAC_DEBUG << "test_file_system: write took " << (t2 - t1) << "\n";

      SSTMAC_DEBUG << "test_file_system: seeking \n";

      fs_->seek(file,  seek_length_);

      t1 = now();
      fs_->write(file, access_size_ * 2);
      t2 = now();

      SSTMAC_DEBUG << "test_file_system: big write after seek took " << (t2 - t1)
            << "\n";

      fs_->close(file);

    }

  }
} //end of namespace sstmac
