/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_TEST_FILE_SYSTEM_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_TEST_FILE_SYSTEM_H_INCLUDED

#include <sstmac/software/process/app.h>
#include <sstmac/software/libraries/filesystem/local_file_system.h>

namespace sstmac
{
  namespace sw
  {

    class test_file_system : public app
    {
    protected:
      test_file_system(softwareid id) :
          app(id)
      {
        fs_ = local_file_system::construct(id);
        register_lib(fs_);
      }

      test_file_system(const sim_parameters::ptr& params) : app(params) {}

    public:
      typedef boost::intrusive_ptr<test_file_system> ptr;

      virtual
      ~test_file_system()
      {
      }

      virtual void
      skeleton_main();

      static ptr
      construct(softwareid id)
      {
        return ptr(new test_file_system(id));
      }

      static ptr
      construct(const sim_parameters::ptr& params)
      {
          return new test_file_system(params);
      }

      virtual app::ptr
      clone(softwareid id)
      {
        return construct(id);
      }

      virtual void
      consume_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "test_file_system";
      }

    private:
      local_file_system::ptr fs_;
      int access_size_;
      long seek_length_;
    };
  }
} //end of namespace sstmac

#endif
