/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/test_mpi_barrier.h>
#include <sstmac/software/api/mpi/mpiapi.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>
#include <cstring>
#include <stdio.h>

namespace sstmac
{
  namespace sw
  {
    SSTRegister("test_mpi_barrier", app, test_mpi_barrier);

// Pointer types.

//
// Hello.
//
    test_mpi_barrier::ptr
    test_mpi_barrier::construct(softwareid id)
    {
      test_mpi_barrier::ptr obj(new test_mpi_barrier(id));
      //  obj->set_self(obj);
      return obj;
    }

//
// Goodbye.
//
    test_mpi_barrier::~test_mpi_barrier() throw ()
    {
    }

//
// Get a copy.
//
    app::ptr
    test_mpi_barrier::clone(softwareid id)
    {
      return test_mpi_barrier::construct(id);
    }

    void
    test_mpi_barrier::consume_params(const sim_parameters::ptr &params)
    {
      iterations_ = params->get_optional_int_param("test_mpi_barrier_iterations", 10);
    }

//
// Go.
//
    void
    test_mpi_barrier::skeleton_main()
    {
      if (mpi() == 0)
        {
          throw nullerror("test_mpi_barrier::run:  mpiapi pointer is null.");
        }
      SSTMAC_DEBUG << "test_mpi_barrier[" << id_.to_string()
            << "]: mpi initializing... \n";
      timestamp start = mpi()->init();
      mpicomm::ptr world = mpi()->comm_world();

      mpiid rank = world->rank();
      mpiid size = world->size();

      SSTMAC_DEBUG << "test_mpi_barrier[" << rank.to_string()
            << "]: mpi initialized, executing " << iterations_
            << " iterations \n";

      for(int i = 0; i < iterations_; i++){
          mpi()->barrier(world);
          SSTMAC_DEBUG << "rank " << rank.id << " is through iteration " << i << "\n";
      }


      timestamp done = mpi()->finalize();


    }

  }
} // end of namespace sstmac.
