/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_TEST_ONESIDED_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_TEST_ONESIDED_H_INCLUDED

#include <sstmac/software/api/mpi/mpiapp.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * Basic MPI ping-pong.
     */
    class test_onesided : public mpiapp
    {
      /// How many iterations we do.
      int iterations_;
      int mode_;
      int msize_;

    protected:
      /// Hi.
      test_onesided(softwareid id) :
        mpiapp(id, to_string())
      {
      }

      test_onesided(const sstmac::sim_parameters::ptr& params)
          : mpiapp(params)
      {
      }

    public:
      typedef boost::intrusive_ptr<test_onesided> ptr;

      /// Hello.
      static ptr
      construct(softwareid id)
      {
        test_onesided::ptr obj(new test_onesided(id));
        return obj;
      }

      static ptr
      construct(const sstmac::sim_parameters::ptr& params)
      {
          return new test_onesided(params);
      }

      /// Goodbye.
      virtual
      ~test_onesided() throw ()
      {
      }

      /// Get a copy.
      virtual app::ptr
      clone(softwareid id)
      {
        return test_onesided::construct(id);
      }

      /// Go.
      void
      skeleton_main();

      virtual void
      consume_params(const sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "test_onesided";
      }
    };

  }
} // end of namespace sstmac

#endif
