/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#include <sstmac/software/skeletons/undumpi/dumpiutil.h>
#include <errno.h>
#include <cstring>

namespace sstmac
{
  namespace sw
  {

    std::string
    dumpi_file_name(int rank, std::string fileroot_)
    {
      static std::string here("dumpi_file_name: ");
      // Figure out our input file name.
      // Try fileroot-"%01d", fileroot-"%02d", ... up to some reasonable length
      // (in the short term that we will probably not have over 10 billion peers)
      FILE *fp = NULL;
      char fname[256];
      fname[255] = '\0';
      const char *fnamefmt = "%%s-%%0%dd.bin";

      char fmt[30];
      fmt[29] = '\0';
      for (int i = 1; i < 10; ++i)
        {
          snprintf(fmt, 29, fnamefmt, i);
          snprintf(fname, 255, fmt, fileroot_.c_str(), rank);
          //std::cerr << here << "looking for file " << fname << std::endl;
          //SSTMAC_DEBUG << "dumpi_file_name: checking file name: " << fname << "\n";
          fp = fopen(fname, "r");
          if (fp)
            {
              fclose(fp);
              return std::string(fname);
            }
           else {
              if (errno != ENOENT) //if the error is anything else than a no such file or directory, crash
              {
                  std::string error_msg = here + "error " + ::strerror(errno) + " opening file " + fname;
                  throw ioerror(error_msg);
              }
          }
        }
      // If we get here, then no file was found.
      throw ioerror(
          here + "failed to find a file " + fname + " starting from the fileroot \""
              + fileroot_ + "\": ");// + strerror(errno));
    }

    int
    getnumprocs(dumpimeta::ptr dmeta_)
    {
      static std::string here("parsedumpi::getnumprocs: ");
      if (dmeta_)
        {
          return dmeta_->getnumprocs();
        }
      int nrank = 0;
      try
        {
          while (true)
            {
              std::string fname = dumpi_file_name(nrank,
                  dmeta_->dirplusfileprefix_);
              nrank++;
            }
        }
      catch (ioerror&)
        {
          // ioerror is thrown when no more trace files can be found
        }
      // SSTMAC_DEBUG << here << "nrank = " << nrank << "\n";
      return nrank;
    }

  }
}


