/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED

//#include <sstmac/backends/native/launch/allocationstrategy.h>
//#include <sstmac/backends/native/launch/indexstrategy.h>
//#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/software/api/mpi/mpiapp.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/software/services/launch/indexing/indexstrategy.h>

#include <boost/lexical_cast.hpp>

namespace sstmac
{
  namespace sw
  {

    /**
     * A refactored dumpi parser to read the newer binary-format dumpi
     * trace files.  Discards support for older-style ascii-based
     * files since nobody was using those except us anyway.
     *
     * TODO:  Fix this to read the metafile rather than the trace files directly
     * (would avoid hack needed where MPI world size is smaller than the total
     * cores allocated to the job).
     */
    class parsedumpi : public mpiapp
    {
    public:
      typedef boost::intrusive_ptr<parsedumpi> ptr;

    private:
      /// Private parts:
      class callbacks;

      /// The fileroot we plan to parse.
      std::string fileroot_;

      /// The time scaling factor.
      double timescaling_;

      /// Use topomap indexing?
      bool use_topomap_;

      std::string metafilename_;

      /// Convenience method.
      inline const mpicomm::ptr
      world()
      {
        return mpi()->comm_world();
      }

    protected:
      /// Hi there.
      parsedumpi(softwareid id);

      parsedumpi(const sim_parameters::ptr& params) : mpiapp(params) {}

    public:
      /// Good news everyone!  We are getting a new parsedumpi.
      static parsedumpi::ptr
      construct(softwareid id);

      static ptr
      construct(const sim_parameters::ptr& params)
      {
          return new parsedumpi(params);
      }

      /// Wait!  That's not good news at all!
      virtual
      ~parsedumpi() throw ();

      /// IT'S AN ADDAMS!
      virtual app::ptr
      clone(softwareid newid);

      virtual void
      consume_params(const sim_parameters::ptr &params);

      /// Parse the tracefile.
      virtual void
      skeleton_main();

      virtual std::string
      to_string() const
      {
        return "parsedumpi";
      }

    };

  }
} // end of namespace sstmac

#endif // ! SSTMAC_SOFTWARE_SKELETONS_UNDUMPI_PARSEDUMPI_H_INCLUDED
