#include <tests/unittests/util/util.h>
#include <sstmac/hardware/network/topology/butterfly.h>


using namespace sstmac;
using namespace sstmac::hw;



void test_butterfly(UnitTest& unit)
{
    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "4 3";
    //params["network_nodes_per_switch"] = "2";
    topology::ptr top = topology_factory::get_value("butterfly", params);
    regular_topology::ptr bfly = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "butterfly cast topology", bfly);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        coordinates coords = get_vector(3,2,0);
        long nid = bfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = bfly->get_node_id(get_vector(1,0,0,1));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        //this should tell me that the productive ports are -X, -Y, -Z
        assertEqual(unit, "num productive ports", ports.size(), 1);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 1));
    }

    {
        coordinates coords = get_vector(3,1,0);
        long nid = bfly->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = bfly->get_node_id(get_vector(3,2,0,1));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        //this should tell me that the productive ports are -X, -Y, -Z
        assertEqual(unit, "num productive ports", ports.size(), 1);

        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(1, 2));
    }

    /** test some distances */
    {
        coordinates c1 = get_vector(3,1,0);
        coordinates c2 = get_vector(3,2,2);
        int dist = bfly->minimal_distance(c1, c2);
        assertEqual(unit, "min distance", dist, 2);
    }

    {
        coordinates c1 = get_vector(3,1,1);
        coordinates c2 = get_vector(0,2,2);
        int dist = bfly->minimal_distance(c1, c2);
        assertEqual(unit, "min distance", dist, 1);
    }

}
