#include <tests/unittests/util/util.h>

using namespace sstmac;
using namespace sstmac::hw;



void test_crossbar(UnitTest& unit)
{
    unittest_params params;
    sstmac_env::params = params;
    params["topology_geometry"] = "10";
    params["network_nodes_per_switch"] = "3";
    topology::ptr top = topology_factory::get_value("crossbar", params);
    regular_topology::ptr xbar = boost::dynamic_pointer_cast<regular_topology>(top);
    assertTrue(unit, "crossbar cast topology", xbar);

    topology::maptype switches;
    init_switches(switches, params, top);

    {
        coordinates coords = get_vector(3);
        long nid = xbar->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = xbar->get_node_id(get_vector(2,2));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        //this should tell me that the productive ports are -X, -Y, -Z
        assertEqual(unit, "num productive ports", ports.size(), 1);


        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, 2));

    }

    {
        coordinates coords = get_vector(3);
        long nid = xbar->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = xbar->get_node_id(get_vector(3,2));

        router::outports ports;
        bool eject = router->get_productive_ports_to_node(naddr(dst), ports);
        // should only have the ejection port
        assertEqual(unit, "num productive ports", ports.size(), 1);
        assertTrue(unit, "is eject port", eject);
    }

}
