#include <tests/unittests/util/util.h>
#include <sstmac/hardware/network/topology/hdtorus.h>


using namespace sstmac;
using namespace sstmac::hw;

void test_torus(UnitTest& unit)
{
    unittest_params params;
    params["topology_geometry"] = "5 5 5";
    params["topology_redundant"] = "1 1 1";
    params["network_nodes_per_switch"] = "2";
    topology::ptr top = topology_factory::get_value("hdtorus", params);
    regular_topology::ptr torus = boost::dynamic_pointer_cast<regular_topology>(top);

    topology::maptype switches;
    init_switches(switches, params, top);

    assertTrue(unit, "torus cast topology", torus);

    {
        coordinates coords = get_vector(3,2,3);
        long nid = torus->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = torus->get_node_id(get_vector(1,1,1,0));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        //this should tell me that the productive ports are -X, -Y, -Z
        assertEqual(unit, "num productive ports", ports.size(), 3);


        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(0, topology::DOWN));
        if (ports.size() > 1)
            assertEqual(unit, "productive port 1", ports[1], router->convert_to_outport(1, topology::DOWN));
        if (ports.size() > 2)
            assertEqual(unit, "productive port 2", ports[2], router->convert_to_outport(2, topology::DOWN));
    }


    {
        coordinates coords = get_vector(3,2,3);
        long nid = torus->get_switch_id(coords);
        networkswitch::ptr sw = switches[naddr(nid)];
        router::ptr router = sw->get_router();

        long dst = torus->get_node_id(get_vector(3,2,0,1));

        router::outports ports;
        router->get_productive_ports_to_node(naddr(dst), ports);
        //this should tell me that the productive ports are -X, -Y, -Z
        assertEqual(unit, "num productive ports", ports.size(), 1);


        if (ports.size() > 0)
            assertEqual(unit, "productive port 0", ports[0], router->convert_to_outport(2, topology::POS));
    }

    {
        coordinates c1 = get_vector(3,2,3);
        coordinates c2 = get_vector(2,4,0);
        int dist = torus->minimal_distance(c1, c2);
        int correct = 1 + 2 + 2;
        assertEqual(unit, "min distance", dist, correct);
    }

    {
        coordinates c1 = get_vector(4,2,3);
        coordinates c2 = get_vector(1,2,2);
        int dist = torus->minimal_distance(c1, c2);
        int correct = 2 + 0 + 1;
        assertEqual(unit, "min distance", dist, correct);
    }
}


