#include <sstmac/sstmacro.h>
#include <sstmac/common/test/test.h>
#include <sstmac/backends/native/mpiparallel/mpi_serializer.h>
#include <sstmac/backends/native/nodeid.h>
#include <sstmac/hardware/network/interconnect/interconnect_message.h>

sstmac::nodeaddress::ptr
naddr(long nid)
{
  return sstmac::nodeaddress::convert_from_uniqueid(nid);
}

using namespace sstmac;
using namespace sstmac::sw;
using namespace sstmac::hw;
using namespace sstmac::native;

char buf[4096];

sst_message::ptr
ser_cycle(const sst_message::ptr& msg, UnitTest &unit)
{
  mpi_ser_sizer::ptr sz = mpi_ser_sizer::construct();
  sst_serializer sersz(sz);
  sersz.do_obj(msg);
  size_t size = sz->get_size();

  mpi_ser_packer::ptr pck = mpi_ser_packer::construct(buf);
  sst_serializer serpck(pck);
  serpck.do_obj(msg);
  assertEqual(unit, "pack size", pck->get_bytes_serialized(), size);

  mpi_ser_unpacker::ptr unpck = mpi_ser_unpacker::construct(buf);
  sst_serializer serunpck(unpck);
  sst_message::ptr ret;
  serunpck.do_obj(ret);

  assertEqual(unit, "unpack size", unpck->get_bytes_deserialized(), size);

  return ret;
}

template<class T>
  T
  cycle(const T& msg, UnitTest &unit)
  {
    typedef typename T::element_type PTR;
    sst_message::ptr new_msg = ser_cycle(msg, unit);
    return boost::dynamic_pointer_cast<PTR>(new_msg);
  }

int
main(int argc, char** argv)
{
  native::nodeid::init_statics();
  UnitTest unit;

  try
  {

    {
      network_message::ptr msg = network_message::construct(naddr(7), naddr(3));
      msg->set_needs_ack(true);
      msg->set_route_algo(routing_info::valiant);
      network_message::ptr test = cycle(msg, unit);
      assertTrue(unit, "network msg", test);
      if (test)
      {
        assertTrue(unit, "needs ack", test->get_needs_ack());
        assertEqual(unit, "route algo", test->route_algo(), routing_info::valiant);
        assertEqual(unit, "deser fromaddr", test->fromaddr()->unique_id(), 3);
        assertEqual(unit, "deser toaddr", test->toaddr()->unique_id(), 7);
      }
    }

    {
      sst_message::ptr parent = network_message::construct(naddr(7), naddr(3));
      interconnect_message::ptr msg = interconnect_message::construct(parent);
      interconnect_message::ptr test = cycle(msg, unit);
      assertTrue(unit, "inter message", test);
      if (test)
      {
        assertEqual(unit, "deser fromaddr", test->fromaddr()->unique_id(), 3);
        assertEqual(unit, "deser toaddr", test->toaddr()->unique_id(), 7);
      }
    }

  }
  catch (std::exception& e)
  {
    std::cerr << e.what() << std::endl;
  }

  unit.validate();
  return 0;
}

