//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <algorithm>
#include "common/utilities/containerhelper.h"
#include "analyzer/trace/node/record/callrecordbase.h"
#include "analyzer/stats/statsgenerate.h"
#include "analyzer/stats/entry/entrygenerateimpl.h"
#include "analyzer/trace/trace.h"

namespace sstmac { namespace tools { namespace analyzer {

struct OnCallRecordFunctor
{
    OnCallRecordFunctor(const CallRecordBase& record)
        : m_record(record)
    {}

    void operator()(Stats::ValueType& val)
    {
        EntryGenerateBase::Ptr entryGen = boost::dynamic_pointer_cast<EntryGenerateBase>(val.second);
        entryGen->OnCallRecord(m_record);    
    }

    const CallRecordBase& m_record;
};

struct StartFunctor
{
    void operator()(Stats::ValueType& val)
    {
        EntryGenerateBase::Ptr entryGen = boost::dynamic_pointer_cast<EntryGenerateBase>(val.second);
        entryGen->Start();    
    }
};

struct StopFunctor
{
    void operator()(Stats::ValueType& val)
    {
        EntryGenerateBase::Ptr entryGen = boost::dynamic_pointer_cast<EntryGenerateBase>(val.second);
        entryGen->Stop();    
    }
};

#define ADD_ENTRY(TYPE) \
    Register(TYPE::StaticClassId(), EntryDataBase::Ptr(new EntryGenerate<TYPE>(nodeCount)))

StatsGenerate::StatsGenerate(Trace& trace)
    : m_trace(trace)
{
    int nodeCount = m_trace.GetNodeCount();

    ADD_ENTRY(statsentry::Overall);
    ADD_ENTRY(statsentry::ByNode);
    ADD_ENTRY(statsentry::MpiCall);
    ADD_ENTRY(statsentry::Collective);
    ADD_ENTRY(statsentry::TransmissionTime);
    ADD_ENTRY(statsentry::TransmissionBytes);
}

bool StatsGenerate::Generate()
{
    bool ret = false;

    m_meta = m_trace.GetMetaData();

    std::for_each(Begin(), End(), StartFunctor());
    ret = m_trace.Parse(boost::bind(&StatsGenerate::OnCallRecord, this, _1));
    std::for_each(Begin(), End(), StopFunctor());

    return ret;
}

void StatsGenerate::OnCallRecord(const CallRecordBase& record)
{
    std::for_each(Begin(), End(), OnCallRecordFunctor(record));
}

} } } //namespace 
