//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <fstream>
#include "analyzer/trace/meta/metadata.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { namespace analyzer {

bool MetaData::Read(const std::string& filePath)
{
    OUTPUT_LOG("Loading meta file %s\n", filePath.c_str());

    std::ifstream fileStream(filePath.c_str());

    if (!fileStream)
    {
        OUTPUT_ERROR("Failed to open meta file %s\n", filePath.c_str());
        return false;
    }

    int lineCount = 0;

    while(fileStream.good()) 
    {
        ++lineCount;

        std::string line;
        std::getline(fileStream, line);

        if (!line.empty())
        {
            std::string::size_type eqPos = line.find_first_of('=');

            if(eqPos != std::string::npos)
            {
                std::string key = line.substr(0, eqPos);
                std::string val = line.substr(eqPos + 1);
                Register(key, val);
            }
            else
            {
                OUTPUT_ERROR("Invalid line %d in metafile, missing '='.\n", lineCount);
                return false;
            }
        }
    }

    return true;
}

} } } //namespace
