#include "analyzer/trace/node/context/mpicontext.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { namespace analyzer {

std::vector<int> MPIContext::m_typeSize;
int MPIContext::m_worldSize;
int MPIContext::m_logWorldSize;

MPIContext::MPIContext(int nodeRank, int worldSize)
    : m_worldRank(nodeRank)
{
    m_worldSize = worldSize;
    int pow2 = 1;
    m_logWorldSize = 0;
    while(pow2 < m_worldSize)
    {
        pow2 = pow2 << 1; 
        m_logWorldSize++;
    }
}

MPIContext::~MPIContext()
{}

void MPIContext::SetTypeSize(int* begin, int* end)
{
    m_typeSize.assign(begin, end);
}

int MPIContext::GetTypeSize(int dataType) const
{
    OUTPUT_ERROR_IF(dataType > (int)m_typeSize.size() || dataType < 0, "Invalide data type [%d]\n", dataType);

    return m_typeSize[dataType];
}

int MPIContext::GetWorldRank() const
{
    return m_worldRank;
}

int MPIContext::GetWorldSize() const
{
    return m_worldSize;
}

int MPIContext::GetLogSize() const
{
    return m_logWorldSize;
}

int MPIContext::GetCommSize(int communicator) const
{
    //TODO: Return actually comm size
    return m_worldSize;
}

int MPIContext::GetCommRank(int communicator) const
{
    //TODO: Translate the local rank to world rank
    return m_worldRank;
}

int MPIContext::CommRankToWorldRank(int commRank, int communicator) const
{
    //TODO: Translate the local rank to world rank
    return commRank;
}

bool MPIContext::context_dumpi_comm_dup(const dumpi_comm_dup* param)
{
	return true;
}

bool MPIContext::context_dumpi_comm_create(const dumpi_comm_create* param)
{
	return true;
}

bool MPIContext::context_dumpi_comm_split(const dumpi_comm_split* param)
{
	return true;
}

bool MPIContext::context_dumpi_comm_free(const dumpi_comm_free* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_union(const dumpi_group_union* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_intersection(const dumpi_group_intersection* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_difference(const dumpi_group_difference* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_incl(const dumpi_group_incl* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_excl(const dumpi_group_excl* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_range_incl(const dumpi_group_range_incl* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_range_excl(const dumpi_group_range_excl* param)
{
	return true;
}

bool MPIContext::context_dumpi_group_free(const dumpi_group_free* param)
{
	return true;
}

} } } //namespace
