#ifndef SSTMAC_TOOLS_ANALYZER_MPICONTEXT
#define SSTMAC_TOOLS_ANALYZER_MPICONTEXT

#include <vector>
#include "dumpi/common/argtypes.h"

#define DECLARE_STATUS_HANDLER(FUNC)     \
bool context_##FUNC(const FUNC* param)   \

namespace sstmac { namespace tools { namespace analyzer {

// Maintain relevent MPI context when reading the trace stream,
// customized group, communicator and types should be recovered.
// These information will be used in statistics
class MPIContext 
{
public:
    MPIContext (int nodeRank, int worldSize);
    virtual ~MPIContext ();

    void SetTypeSize(int* begin, int* end);
    int GetTypeSize(int dataType) const;

    int GetWorldRank() const; 
    int GetWorldSize() const;
    int GetLogSize() const;

    int GetCommSize(int communicator) const;
    int GetCommRank(int communicator) const;

    int CommRankToWorldRank(int commRank, int communicator) const;

    // Visitor functions used by CallRecord
    DECLARE_STATUS_HANDLER(dumpi_comm_dup);
    DECLARE_STATUS_HANDLER(dumpi_comm_create);
    DECLARE_STATUS_HANDLER(dumpi_comm_split);
    DECLARE_STATUS_HANDLER(dumpi_comm_free);
    DECLARE_STATUS_HANDLER(dumpi_group_union);
    DECLARE_STATUS_HANDLER(dumpi_group_intersection);
    DECLARE_STATUS_HANDLER(dumpi_group_difference);
    DECLARE_STATUS_HANDLER(dumpi_group_incl);
    DECLARE_STATUS_HANDLER(dumpi_group_excl);
    DECLARE_STATUS_HANDLER(dumpi_group_range_incl);
    DECLARE_STATUS_HANDLER(dumpi_group_range_excl);
    DECLARE_STATUS_HANDLER(dumpi_group_free);

private:
    static std::vector<int> m_typeSize;
    static int m_worldSize;
    static int m_logWorldSize;
    int m_worldRank;
};

} } } //namespace

#endif //SSTMAC_TOOLS_ANALYZER_MPICONTEXT
