#include <fstream>
#include "dumpi/libundumpi/libundumpi.h"
#include "analyzer/trace/node/nodedata.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { namespace analyzer {

NodeData::NodeData(int nodeRank, int worldSize)
    : MPIContext(nodeRank, worldSize)
{}

NodeData::~NodeData()
{}

bool NodeData::SetFilePath(const std::string& filePath)
{    
    m_filePath = filePath;
    
    // Test if file exist
    return std::ifstream(m_filePath.c_str()); 
}

bool NodeData::Parse(const RecordCallback& callback)
{
    dumpi_profile* profile = undumpi_open(m_filePath.c_str());

    if (!profile)
    {
        OUTPUT_ERROR("DUMPI profile dosen't exsit!\n");
        return false;
    }

    OUTPUT_LOG("Parsing trace file %s\n", m_filePath.c_str());

    // Read type sizes
    dumpi_sizeof dumpiTypeSize = undumpi_read_datatype_sizes(profile);
    SetTypeSize(dumpiTypeSize.size, dumpiTypeSize.size + dumpiTypeSize.count);
    free(dumpiTypeSize.size);
    
    // Read stream
    libundumpi_callbacks dumpiCallback;
    libundumpi_clear_callbacks(&dumpiCallback);
    SetDumpiCallbacks(dumpiCallback);
    
    DumpiCallbackArg userArg(this, callback);

    int ret = undumpi_read_stream(profile, &dumpiCallback, &userArg);

    undumpi_close(profile);

    return ret;
}

} } } //namespace
