//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_FACTORY_FACTORY_INCLUDED_
#define SSTMAC_TOOLS_COMMON_FACTORY_FACTORY_INCLUDED_

#include <string>
#include <boost/functional/factory.hpp>
#include <boost/function.hpp>
#include "common/utilities/classhelper.h"
#include "common/registry/registry.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { 

template<typename AbstractType>
class Factory : public Registry< std::string, boost::function<AbstractType*()> >
{
    DECLARE_SINGLETON(Factory);

public:
    typedef boost::function<AbstractType*()> Creator;
    typedef Registry< std::string, boost::function<AbstractType*()> > Base;

public:
    AbstractType* Create(const std::string& classId) const {
        const Creator* creator = Base::Get(classId);
        return creator ? (*creator)() : NULL;
    }
};

template<typename AbstractType>
Factory<AbstractType>::Factory(){}

template<typename AbstractType>
Factory<AbstractType>::~Factory(){}

template<typename AbstractType>
Factory<AbstractType> * Factory<AbstractType>::ms_singletonInstance = NULL;

} } //namespace

#define FACTORY_REGISTER_CLASS(BASECLASSNAMESPACE, BASECLASS, CONCRETECLASSNAMESPACE, CONCRETECLASS) \
    sstmac::tools::Factory<BASECLASSNAMESPACE::BASECLASS>::Creator creator_##BASECLASS##CONCRETECLASS = boost::factory<CONCRETECLASSNAMESPACE::CONCRETECLASS*>(); \
    bool register_Factory_##BASECLASS##CONCRETECLASS = sstmac::tools::Factory<BASECLASSNAMESPACE::BASECLASS>::SafeSingleton().Register(CONCRETECLASSNAMESPACE::CONCRETECLASS::StaticClassId(),creator_##BASECLASS##CONCRETECLASS)

// Using class 'use' the registration variable, so that the compiler won't optimize out the registration
#define FACTORY_USING_CLASS( BASECLASS, CONCRETECLASS)  \
    extern bool register_Factory_##BASECLASS##CONCRETECLASS; \
    register_Factory_##BASECLASS##CONCRETECLASS = true

#define FACTORY_DESTROY( BASECLASSNAMESPACE, BASECLASS)  \
    sstmac::tools::Factory<BASECLASSNAMESPACE::BASECLASS>::DestroySingleton()

#endif //SSTMAC_TOOLS_COMMON_FACTORY_FACTORY_INCLUDED_
