//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <iomanip>
#include "common/numeric/difference.h"
#include "common/numeric/numericdefine.h"
#include "common/utilities/serializeadapter.h"

namespace sstmac { namespace tools {

Difference::Difference()
    : m_diff(0.0)
    , m_diffRel(0.0)
{}

Difference::Difference(double refVal, double val)
{
    Set(refVal, val);
}

void Difference::Set(double refVal, double val)
{
    m_diff = val - refVal;
    m_diffRel = IsEqual(refVal, 0.0) ?  numeric_limits<double>::infinity() : (val - refVal) / refVal;

    //We clamp the value between 6 sigma, so gaussian function will have reasonable value
    m_diffRel = Clamp(-6.0, 6.0, m_diffRel);
}

double Difference::Get() const
{
    return m_diff;
}

double Difference::GetRel() const
{
    return m_diffRel;
}

Difference::operator double() const
{
    return Get();
}

void Difference::Serialize(ostream& os) const
{
    os << std::showpos << m_diff << std::noshowpos;
}

void Difference::Deserialize(istream& is)
{
    is >> m_diff;
}

} } //namespace
