//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_UTILITIES_NUMERIC_INCLUDED_
#define SSTMAC_TOOLS_COMMON_UTILITIES_NUMERIC_INCLUDED_

#include <cmath>
#include <limits>

namespace sstmac { namespace tools {

template<typename T>
inline bool IsEqual(T lhs, T rhs)
{
    return fabs(lhs - rhs) < numeric_limits<T>::epsilon();
}

template<typename T>
inline T Ratio(T lhs, T rhs)
{
    return IsEqual(lhs,0.0) ?  numeric_limits<T>::infinity() : (rhs - lhs / lhs);
}

inline float Gaussian(float x, float r)
{
    return exp(-2.0 * (x - r) * (x - r) / ((x + r) * (x + r)));
}

template<typename T>
inline T Clamp(T min, T max, T val)
{
    if(val < min)
    {
        return min;
    }
    else if(val > max)
    {
        return max;
    }

    return val;
}

} } //namespace

#endif //SSTMAC_TOOLS_COMMON_UTILITIES_NUMERIC_INCLUDED_
