//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_ENTRY_ENTRYDATAIMPL_INCLUDED_
#define SSTMAC_TOOLS_COMMON_ENTRY_ENTRYDATAIMPL_INCLUDED_

#include <vector>
#include <set>
#include "common/numeric/accumulatorimpl.h"
#include "common/stats/entry/entrydatabase.h"

namespace sstmac { namespace tools { namespace statsentry {

struct Overall : public EntryDataBase
{
    DECLARE_ENTRYDATA(Overall);

    Sum m_bytes;
    Sum m_waitTime;
    Sum m_computeTime;
};

struct ByNode : public EntryDataBase
{
    DECLARE_ENTRYDATA(ByNode);

    vector<Sum> m_bytes;
    vector<Sum> m_waitTime;
    vector<Sum> m_computeTime;
    vector<int> m_inDegree;
    vector<int> m_outDegree;
    
    Average m_avgBytes;
    Average m_avgWaitTime;
    Average m_avgComputeTime;
    Average m_avgInDegree;
    Average m_avgOutDegree;
    Deviation m_sdBytes;
    Deviation m_sdWaitTime;
    Deviation m_sdComputeTime;
    Deviation m_sdInDegree;
    Deviation m_sdOutDegree;

    vector< set<int> > m_inNodes;
    vector< set<int> > m_outNodes;
};

struct MpiCall : public EntryDataBase
{
    DECLARE_ENTRYDATA(MpiCall);

    vector<int> m_count;
    vector<Average> m_avgBlockTime;
    vector<Deviation> m_sdBlockTime;
    vector< vector<Sum> > m_time;
};

struct Collective : public EntryDataBase
{
    DECLARE_ENTRYDATA(Collective);
    
    struct CollRegion
    {
        DECLARE_SERIALIZABLE(CollRegion);

        CollRegion();
        CollRegion(int seqId, int beginFunc, int endFunc);

        bool operator==(const CollRegion& rhs) const;

        int m_seqId;
        int m_beginFunc;
        int m_endFunc;
    };

    int m_currRegion;                   //Intermediate counter

    vector< vector<Sum> > m_time;       //2d vector of [CollRegion][NodeRank]
    vector<Average> m_avgTime;
    vector<Deviation> m_sdTime;
    vector<CollRegion> m_collective;
};

struct TransmissionTime : public EntryDataBase
{
    DECLARE_ENTRYDATA(TransmissionTime);

    vector< vector<Sum> > m_time;
   // vector< vector<Sum> > m_traffic;
};

struct TransmissionBytes : public EntryDataBase
{
    DECLARE_ENTRYDATA(TransmissionBytes);

    //vector< vector<Sum> > m_time;
    vector< vector<Sum> > m_traffic;
};

} } } //namespace

#endif //SSTMAC_TOOLS_COMMON_ENTRY_ENTRYDATAIMPL_INCLUDED_
