//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_UTILITIES_CLASSHELPER_INCLUDED_
#define SSTMAC_TOOLS_COMMON_UTILITIES_CLASSHELPER_INCLUDED_

#include "common/utilities/assert.h"

namespace sstmac { namespace tools { typedef const char* ClassId; } }

#define CLASS_ID(TYPE) \
static ClassId StaticClassId(){return #TYPE;} \
virtual ClassId GetClassId() const {return #TYPE;}

//------------------------------------------------------------------------
#define NOT_COPIABLE(className)                 \
    private:                                    \
        className(const className&);            \
        className& operator=(const className&)

//------------------------------------------------------------------------
// 
// Singleton pattern
// 
//------------------------------------------------------------------------
#define DECLARE_SINGLETON( ClassName )\
    DECLARE_SINGLETON_PARAM(ClassName, (), () )

//------------------------------------------------------------------------
#define DECLARE_SINGLETON_PARAM( ClassName, CreatorSignature, ConstructorVariable )\
    \
    public:\
        static bool IsSingletonCreated() {return ms_singletonInstance != NULL;}\
        DECLARE_SINGLETON_CREATOR(ClassName, CreatorSignature, ConstructorVariable)\
        DECLARE_SINGLETON_QUERY(ClassName, ConstructorVariable)\
        DECLARE_SINGLETON_DESTROYER(ClassName)\
    \
    private:\
        ClassName CreatorSignature;\
        ~ClassName ();\
    \
    private:\
        static ClassName * ms_singletonInstance;\
        NOT_COPIABLE(ClassName)

//------------------------------------------------------------------------
#define IMPLEMENT_SINGLETON( ClassName )\
    ClassName * ClassName::ms_singletonInstance = NULL

//------------------------------------------------------------------------
#define DECLARE_SINGLETON_CREATOR( ClassName, CreatorSignature, ConstructorVariable )\
    static void CreateSingleton CreatorSignature\
    {\
        ASSERT(ms_singletonInstance == NULL, "The %s singleton is already created", #ClassName);\
        ms_singletonInstance = new (ClassName) ConstructorVariable;\
    }

//------------------------------------------------------------------------
#define DECLARE_SINGLETON_QUERY( ClassName, ConstructorVariable)\
    static ClassName & Singleton()\
    {\
        ASSERT(ms_singletonInstance, "The %s singleton is invalid", #ClassName);\
        return *ms_singletonInstance;\
    }\
    static ClassName & SafeSingleton()\
    {\
        if (!ms_singletonInstance) CreateSingleton ConstructorVariable;\
        return *ms_singletonInstance;\
    }

//------------------------------------------------------------------------
#define DECLARE_SINGLETON_DESTROYER( ClassName )\
\
    static void DestroySingleton()\
    {\
        if (!ms_singletonInstance) \
        {  \
            return; \
        } \
        delete ms_singletonInstance;\
        ms_singletonInstance = NULL;\
    }

#endif //SSTMAC_TOOLS_COMMON_UTILITIES_CLASSHELPER_INCLUDED_
