//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#ifndef SSTMAC_TOOLS_COMMON_UTILITIES_TIME_INCLUDED_
#define SSTMAC_TOOLS_COMMON_UTILITIES_TIME_INCLUDED_

#include <string>
#include "common/utilities/serializehelper.h"

namespace sstmac { namespace tools {

////////////////////////////////////////////////////////////////////////////////
// Time
class Time 
{
public:
    static double GetWallTime();
    static double GetCpuTime();

    static std::string ToString(double time);
};

////////////////////////////////////////////////////////////////////////////////
// Timer
class Timer
{
public:
    Timer();
    virtual ~Timer();

    void Start();
    void Stop();
    void Resume();

    double ElapsedCpu() const;
    double ElapsedWall() const;

    std::string ToString() const;

private:
    double m_cpuStart;
    double m_wallStart;
    double m_cpuElapse;
    double m_wallElapse;
    bool m_stopped;
};

////////////////////////////////////////////////////////////////////////////////
// ScopedTimer
class ScopedTimer : public Timer
{
public:
    ScopedTimer();
    ScopedTimer(const std::string& tag);
    virtual ~ScopedTimer();

private:
    std::string m_tag;
};

} } //namespace

#endif //SSTMAC_TOOLS_COMMON_UTILITIES_TIME_INCLUDED_
