//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <getopt.h>
#include "comparator/setting/setting.h"
#include "common/log/logger.h"

namespace sstmac { namespace tools { namespace comparator {

Setting::Setting()
{}

Setting::~Setting()
{}

bool Setting::Load(int argc, char* argv[])
{
    //Not enough arugments provided
    if (argc <= 1)
    {
        OUTPUT_INFO("Usage:\t%s [options] ReferenceReport SimulationReport\n\t%s -h for more information\n", argv[0], argv[0]);
        return false;
    }

    bool isValid = true;
    int optVal;

    while((optVal = getopt(argc, argv, ":hvo:")) != -1)
    {
        switch(optVal)
        {
        case 'h':
            OUTPUT_INFO( 
                "Trace Comparator compare statistics from dumpi trace reports.\n"
	            "Usage: %s [options] ReferenceReport SimulationReport\n"
                "   Options:\n"
                "       -h              Print this help\n"
                "       -v              Verbose status output\n"
                "       -o  filename    Write result into file\n",
                argv[0]);
            isValid = false;
            break;
        case 'o':
            m_reportPath = optarg;
            break;
        case 'v':
            SetVerbose(true);
            break;
        case ':':
            OUTPUT_ERROR("Missing arugment for option \"%c\"\n", optopt);
            isValid = false;
            break;
        case '?':
        default:
            OUTPUT_ERROR("Unknown option \"%c\"\n", optopt);
            isValid = false;
            break;
        }
    }

    if (optind < argc)
    {
        //Expect one more agument as input metafile name
        if(optind + 2 == argc)    
        {
            m_referencePath = argv[optind];
            m_simulationPath = argv[optind + 1];
        }
        else
        {
            OUTPUT_ERROR("Wrong number of arguments, expecting only two\n");
            isValid = false;
        }
    }
    
    return isValid;
}

const std::string& Setting::GetReferencePath() const
{
    return m_referencePath;
}

const std::string& Setting::GetSimulationPath() const
{
    return m_simulationPath;
}

const std::string& Setting::GetReportPath() const
{
    return m_reportPath;
}

} } } //namespace
