//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <wx/log.h>
#include "common/factory/factory.h"
#include "common/stats/stats.h"
#include "common/report/reportstats.h"
#include "visualizer/document/statsdocument.h"
#include "visualizer/plot/entry/entryplotbase.h"

namespace sstmac { namespace tools { namespace visualizer {

IMPLEMENT_DYNAMIC_CLASS(StatsDocument, wxDocument);

EntryDataBase* EntryPlotCreator(const std::string& classId)
{
    return dynamic_cast<EntryDataBase*>(Factory<EntryPlotBase>::Singleton().Create(classId));
}

bool StatsDocument::DoOpenDocument(const wxString& filePath)
{
    wxLogMessage(wxT("Openning file %s"), filePath.c_str());

    ReportStats reportStats(m_stats, EntryPlotCreator);

    if(!reportStats.Read(filePath.mb_str().data()))
    {
        wxLogError(wxT("Failed to open file %s"), filePath.c_str());
        return false;
    }

    return true;
}

bool StatsDocument::DoSaveDocument(const wxString& filePath)
{
    return true;
}

const Stats& StatsDocument::GetStats() const
{
    return m_stats;
}

} } } //namespace
