//------------------------------------------------------------------------------
// 
//     SST/Macro tools set
//
//------------------------------------------------------------------------------

#include <wx/log.h>
#include <wx/string.h>
#include "common/stats/stats.h"
#include "visualizer/view/statsview.h"
#include "visualizer/document/statsdocument.h"
#include "visualizer/ui/mainframe.h"
#include "visualizer/ui/plotframe.h"
#include "visualizer/plot/entry/entryplotbase.h"

namespace sstmac { namespace tools { namespace visualizer {

IMPLEMENT_DYNAMIC_CLASS(StatsView, wxView);

bool StatsView::OnCreate(wxDocument* document, long flag)
{
    wxString printName;
    document->GetPrintableName(printName);

    //Creat MDI child frame
    m_frame = new PlotFrame(document, this, GetMainFrame(), printName);
    m_frame->Show(true);
    Activate(true);

    return true;
}

void StatsView::OnDraw(wxDC* WXUNUSED(dc))
{}

void StatsView::OnUpdate(wxView* WXUNUSED(sender), wxObject* WXUNUSED(hint))
{
     //Add plots into frame
    StatsDocument* statsDoc = static_cast<StatsDocument*>(GetDocument());

    for(Stats::ConstIterator iter = statsDoc->GetStats().Begin(); iter != statsDoc->GetStats().End(); ++iter )
    {
        EntryPlotBase::Ptr plot = boost::dynamic_pointer_cast<EntryPlotBase>(iter->second);
        m_frame->AddPlotWindow(plot);
    }
}

bool StatsView::OnClose(bool deleteWindow)
{
    if(!GetDocument()->Close())
    {
        return false;
    }

    if(deleteWindow)
    {
        delete m_frame;
        m_frame = NULL;
    }

    return true;
}

} } } //namespace
